/* -*- mode:C++ ; compile-command: "g++-3.4 -I.. -g -c misc.cc -DHAVE_CONFIG_H -DIN_GIAC" -*- */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "first.h"
/*
 *  Copyright (C) 2001, 2007 R. De Graeve, B. Parisse, Institut Fourier, 38402 St Martin d'Heres
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by 
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using namespace std;
#include <fstream>
#include <time.h>
#include <string>
#include "misc.h"
#include "usual.h"
#include "sym2poly.h"
#include "rpn.h"
#include "prog.h"
#include "derive.h"
#include "subst.h"
#include "intg.h"
#include "vecteur.h"
#include "ifactor.h"
#include "solve.h"
#include "modpoly.h"
#include "permu.h"
#include "sym2poly.h"
#include "plot.h"
#include "lin.h"
#include "modpoly.h"
#include "desolve.h"
#include "alg_ext.h"
#include "input_parser.h"
#include "input_lexer.h"
#include "maple.h"

#ifndef NO_NAMESPACE_GIAC
namespace giac {
#endif // ndef NO_NAMESPACE_GIAC

  gen _scalar_product(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT || args._VECTptr->size()!=2)
      setsizeerr();
    vecteur & v=*args._VECTptr;
    return scalar_product(v[0],v[1],contextptr);
  }    
  const char _scalar_product_s []="scalar_product";
  const unary_function_eval __scalar_product(&_scalar_product,_scalar_product_s);
  const unary_function_ptr at_scalar_product (&__scalar_product,0,true);

  const char _dot_s []="dot";
  const unary_function_eval __dot(&_scalar_product,_dot_s);
  const unary_function_ptr at_dot (&__dot,0,true);

  gen _compare(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT || args._VECTptr->size()!=2)
      setsizeerr();
    vecteur & v=*args._VECTptr;
    return v[0].islesscomplexthan(v[1]);
  }    
  const char _compare_s []="compare";
  const unary_function_eval __compare(&_compare,_compare_s);
  const unary_function_ptr at_compare (&__compare,0,true);

  gen _preval(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT)
      return symbolic(at_preval,args);
    vecteur & v=*args._VECTptr;
    int s=v.size();
    if (s<3)
      toofewargs("");
    gen f(v[0]),x,a,b;
    a=v[1];
    b=v[2];
    if (s==3)
      x=vx_var;
    else 
      x=v[3];
    if (x.type!=_IDNT)
      settypeerr();
    return preval(f,x,a,b,contextptr);
  }    
  const char _preval_s []="preval";
  const unary_function_eval __preval(&_preval,_preval_s);
  const unary_function_ptr at_preval (&__preval,0,true);

  vecteur divided_differences(const vecteur & x,const vecteur & y){
    vecteur res(y);
    int s=x.size();
    for (int k=1;k<s;++k){
      for (int j=s-1;j>=k;--j){
	res[j]=(res[j]-res[j-1])/(x[j]-x[j-k]);
      }
    }
    return res;
  }
  gen _lagrange(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT)
      return symbolic(at_lagrange,args);
    vecteur & v=*args._VECTptr;
    int s=v.size();
    if (s<2)
      toofewargs("");
    gen v0(v[0]),v1(v[1]),x=vx_var;
    if (ckmatrix(v0) && v0._VECTptr->size()==2){
      x=v1;
      v1=v0._VECTptr->back();
      v0=v0._VECTptr->front();
    }
    if (s>=3)
      x=v[2];
    if ( (v0.type!=_VECT) || (v1.type!=_VECT) )
      setsizeerr();
    vecteur & vx =*v0._VECTptr;
    vecteur & vy=*v1._VECTptr;
    s=vx.size();
    if (!s || vy.size()!=unsigned(s))
      setdimerr();
    // Using divided difference instead of the theoretical formula
    if (x.type==_VECT && x._VECTptr->empty()){
      vecteur res;
      interpolate(vx,vy,res,0);
      return res;
    }
    vecteur w=divided_differences(vx,vy);
    gen pi(1),res(w[s-1]);
    for (int i=s-2;i>=0;--i){
      res = res*(x-vx[i])+w[i];
    }
    return res;
    /*
    gen res(zero);
    for (int i=0;i<s;++i){
      gen pix(plus_one),pix0(plus_one),x0(vx[i]);
      for (int j=0;j<s;++j){
	if (j==i)
	  continue;
	pix=pix*(x-vx[j]);
	pix0=pix0*(x0-vx[j]);
      }
      res=res+vy[i]*rdiv(pix,pix0);
    }
    return res;
    */
  }    
  const char _lagrange_s []="lagrange";
  const unary_function_eval __lagrange(&_lagrange,_lagrange_s);
  const unary_function_ptr at_lagrange (&__lagrange,0,true);

  gen _reorder(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT)
      return symbolic(at_reorder,args);
    vecteur & v=*args._VECTptr;
    int s=v.size();
    if (s<2)
      toofewargs("");
    gen e(v[0]),l(v[1]);
    if (l.type!=_VECT)
      setsizeerr();
    vecteur w(*l._VECTptr);
    lvar(e,w);
    e=e2r(e,w,contextptr);
    return r2e(e,w,contextptr);
  }    
  const char _reorder_s []="reorder";
  const unary_function_eval __reorder(&_reorder,_reorder_s);
  const unary_function_ptr at_reorder (&__reorder,0,true);

  gen _adjoint_matrix(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT)
      return symbolic(at_adjoint_matrix,args);
    matrice mr(*args._VECTptr);
    if (!is_squarematrix(mr))
      setsizeerr();
    matrice m_adj;
    vecteur p_car;
    p_car=mpcar(mr,m_adj,true,true,contextptr);
    return makevecteur(p_car,m_adj);
  }    
  const char _adjoint_matrix_s []="adjoint_matrix";
  const unary_function_eval __adjoint_matrix(&_adjoint_matrix,_adjoint_matrix_s);
  const unary_function_ptr at_adjoint_matrix (&__adjoint_matrix,0,true);

  gen _equal2diff(const gen & args,GIAC_CONTEXT){
    return apply(args,equal2diff);
  }    
  const char _equal2diff_s []="equal2diff";
  const unary_function_eval __equal2diff(&_equal2diff,_equal2diff_s);
  const unary_function_ptr at_equal2diff (&__equal2diff,0,true);

  gen equal2list(const gen & arg){
    if ( (arg.type!=_SYMB) || (arg._SYMBptr->sommet!=at_equal))
      return makevecteur(arg,zero);
    return arg._SYMBptr->feuille;
  }
  gen _equal2list(const gen & args,GIAC_CONTEXT){
    return apply(args,equal2list);
  }    
  const char _equal2list_s []="equal2list";
  const unary_function_eval __equal2list(&_equal2list,_equal2list_s);
  const unary_function_ptr at_equal2list (&__equal2list,0,true);

  gen _rank(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT)
      return symbolic(at_adjoint_matrix,args);
    matrice mr(*args._VECTptr);
    if (!ckmatrix(mr))
      setsizeerr();
    mr=mrref(mr,contextptr);
    int r=mr.size();
    for (;r;--r){
      if (!is_zero(mr[r-1]))
	break;
    }
    return r;
  }    
  const char _rank_s []="rank";
  const unary_function_eval __rank(&_rank,_rank_s);
  const unary_function_ptr at_rank (&__rank,0,true);

  gen _sec(const gen & args,GIAC_CONTEXT){
    return inv(cos(args,contextptr),contextptr);
  }    
  const char _sec_s []="sec";
  const unary_function_eval __sec(&_sec,_sec_s);
  const unary_function_ptr at_sec (&__sec,0,true);

  gen _csc(const gen & args,GIAC_CONTEXT){
    return inv(sin(args,contextptr),contextptr);
  }    
  const char _csc_s []="csc";
  const unary_function_eval __csc(&_csc,_csc_s);
  const unary_function_ptr at_csc (&__csc,0,true);

  gen _cot(const gen & args,GIAC_CONTEXT){
    return rdiv(cos(args,contextptr),sin(args,contextptr));
  }    
  const char _cot_s []="cot";
  const unary_function_eval __cot(&_cot,_cot_s);
  const unary_function_ptr at_cot (&__cot,0,true);

  gen _asec(const gen & args,GIAC_CONTEXT){
    return acos(inv(args,contextptr),contextptr);
  }    
  const char _asec_s []="asec";
  const unary_function_eval __asec(&_asec,_asec_s);
  const unary_function_ptr at_asec (&__asec,0,true);

  gen _acsc(const gen & args,GIAC_CONTEXT){
    return asin(inv(args,contextptr),contextptr);
  }    
  const char _acsc_s []="acsc";
  const unary_function_eval __acsc(&_acsc,_acsc_s);
  const unary_function_ptr at_acsc (&__acsc,0,true);

  gen _acot(const gen & args,GIAC_CONTEXT){
    return atan(inv(args,contextptr),contextptr);
  }    
  const char _acot_s []="acot";
  const unary_function_eval __acot(&_acot,_acot_s);
  const unary_function_ptr at_acot (&__acot,0,true);

  // args=[u'*v,v] or [[F,u'*v],v] -> [F+u*v,-u*v']
  // a third argument would be the integration var
  // if v=0 returns F+integrate(u'*v,x)
  gen _ibpu(const gen & args,GIAC_CONTEXT) {
    if ( (args.type!=_VECT) || (args._VECTptr->size()<2) )
      return symbolic(at_ibpu,args);
    vecteur & w=*args._VECTptr;
    gen X(vx_var),x(vx_var),a,b;
    bool bound=false;
    if (w.size()>=3)
      x=X=w[2];
    if (x.is_symb_of_sommet(at_equal))
      x=x._SYMBptr->feuille[0];
    if (w.size()>=5)
      X=symb_equal(x,symb_interval(w[3],w[4]));
    if (X.is_symb_of_sommet(at_equal) && X._SYMBptr->feuille[1].is_symb_of_sommet(at_interval)){
      a=X._SYMBptr->feuille[1]._SYMBptr->feuille[0];
      b=X._SYMBptr->feuille[1]._SYMBptr->feuille[1];
      bound=true;
    }
    gen u,v(w[1]),uprimev,F;
    if (w.front().type==_VECT){
      vecteur & ww=*w.front()._VECTptr;
      if (ww.size()!=2)
	setsizeerr();
      F=ww.front();
      uprimev=ww.back();
    }
    else 
      uprimev=w.front();
    if (is_zero(v) || is_one(v)){
      gen tmp=integrate(uprimev,x,contextptr);
      if (bound)
	tmp=preval(tmp,x,a,b,contextptr);
      return tmp+F;
    }
    gen uprime(normal(rdiv(uprimev,v),contextptr));
    u=integrate(uprime,x,contextptr);
    F += u*v;
    if (bound)
      F = preval(F,x,a,b,contextptr);
    return makevecteur(F,normal(-u*derive(v,x,contextptr),contextptr));
  }
  const char _ibpu_s []="ibpu";
  const unary_function_eval __ibpu(&_ibpu,_ibpu_s);
  const unary_function_ptr at_ibpu (&__ibpu,0,true);

  gen _changebase(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT)
      return symbolic(at_changebase,args);
    vecteur & v=*args._VECTptr;
    if (v.size()!=2)
      settypeerr();
    gen a=v.front(),p=v.back();
    if (!is_squarematrix(p))
      setsizeerr();
    return minv(*p._VECTptr,contextptr)*a*p;
  }    
  const char _changebase_s []="changebase";
  const unary_function_eval __changebase(&_changebase,_changebase_s);
  const unary_function_ptr at_changebase (&__changebase,0,true);

  gen epsilon2zero(const gen & g,GIAC_CONTEXT){
    switch (g.type){
    case _DOUBLE_:
      if (fabs(g._DOUBLE_val)<epsilon(contextptr))
	return zero;
      else
	return g;
    case _CPLX:
      return epsilon2zero(re(g,contextptr),contextptr)+cst_i*epsilon2zero(im(g,contextptr),contextptr);
    case _SYMB:
      return symbolic(g._SYMBptr->sommet,epsilon2zero(g._SYMBptr->feuille,contextptr));
    case _VECT:
      return apply(g,epsilon2zero,contextptr);
    default:
      return g;
    }
  }
  gen _epsilon2zero(const gen & args,GIAC_CONTEXT){
    return epsilon2zero(args,contextptr);
  }    
  const char _epsilon2zero_s []="epsilon2zero";
  const unary_function_eval __epsilon2zero(&_epsilon2zero,_epsilon2zero_s);
  const unary_function_ptr at_epsilon2zero (&__epsilon2zero,_QUOTE_ARGUMENTS,true);

  gen _suppress(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT)
      return symbolic(at_suppress,args);
    vecteur & v=*args._VECTptr;
    if (v.size()!=2)
      settypeerr();
    gen l=v.front(),i=v.back();
    int ii=0;
    if (i.type==_INT_ )
      ii=i.val-(xcas_mode(contextptr)!=0);
    if (l.type==_STRNG){
      string res;
      string & s=*l._STRNGptr;
      int n=s.size();
      if (i.type==_INT_ && ii>=0 && ii<n)
	res=s.substr(0,ii)+s.substr(ii+1,n-ii-1);
      if (i.type==_STRNG){
	string & remove=*i._STRNGptr;
	int removen=remove.size();
	for (int j=0;j<n;++j){
	  int k=remove.find(s[j]);
	  if (k<0 || k>=removen)
	    res += s[j];
	}
      }
      return string2gen(res,false);
    }
    if ( (l.type!=_VECT) || (i.type!=_INT_) )
      setsizeerr();
    const_iterateur it=l._VECTptr->begin(),itend=l._VECTptr->end();
    vecteur res;
    res.reserve(itend-it);
    for (int j=0;it!=itend;++it,++j){
      if (j!=ii)
	res.push_back(*it);
    }
    return gen(res,l.subtype);
  }    
  const char _suppress_s []="suppress";
  const unary_function_eval __suppress(&_suppress,_suppress_s);
  const unary_function_ptr at_suppress (&__suppress,0,true);

  int valuation(const polynome & p){
    if (p.coord.empty())
      return -1;
    return p.coord.back().index.front();
  }
  gen _valuation(const gen & args,GIAC_CONTEXT){
    gen p,x;
    if (args.type!=_VECT){
      x=vx_var;
      p=args;
    }
    else {
      vecteur & v=*args._VECTptr;
      int s=v.size();
      if (!s)
	return minus_inf;
      if ( (args.subtype==_POLY1__VECT) || (s!=2) || (v[1].type!=_IDNT) ){
	int j=s;
	for (;j;--j){
	  if (!is_zero(v[j-1]))
	    break;
	}
	return s-j;
      }
      x=v.back();
      p=v.front();
    }
    vecteur lv(1,x);
    lvar(p,lv);
    gen aa=e2r(p,lv,contextptr),aan,aad;
    if (is_zero(aa))
      return minus_inf;
    fxnd(aa,aan,aad);
    if ( (aad.type==_POLY) && (aad._POLYptr->lexsorted_degree() ) )
      setsizeerr();
    if (aan.type!=_POLY)
      return zero;
    int res=valuation(*aan._POLYptr);
    if (res==-1)
      return minus_inf;
    else
      return res;
  }    
  const char _valuation_s []="valuation";
  const unary_function_eval __valuation(&_valuation,_valuation_s);
  const unary_function_ptr at_valuation (&__valuation,0,true);

  const char _ldegree_s []="ldegree";
  const unary_function_eval __ldegree(&_valuation,_ldegree_s);
  const unary_function_ptr at_ldegree (&__ldegree,0,true);

  gen _degree(const gen & args,GIAC_CONTEXT){
    gen p,x;
    if (args.type!=_VECT){
      x=vx_var;
      p=args;
    }
    else {
      vecteur & v=*args._VECTptr;
      int s=v.size();
      if ( (args.subtype==_POLY1__VECT) || (s!=2) || (v[1].type!=_IDNT) )
	return s-1;
      x=v.back();
      p=v.front();
    }
    vecteur lv(1,x);
    lvar(p,lv);
    gen aa=e2r(p,lv,contextptr),aan,aad;
    if (is_zero(aa))
      return zero;
    fxnd(aa,aan,aad);
    if ( (aad.type==_POLY) && (aad._POLYptr->lexsorted_degree() ) )
      setsizeerr();
    if (aan.type!=_POLY)
      return zero;
    return aan._POLYptr->lexsorted_degree();
  }    
  const char _degree_s []="degree";
  const unary_function_eval __degree(&_degree,_degree_s);
  const unary_function_ptr at_degree (&__degree,0,true);

  gen _lcoeff(const gen & args,GIAC_CONTEXT){
    gen x,p,order;
    int s=2;
    if (args.type!=_VECT){
      x=vx_var;
      p=args;
    }
    else {
      vecteur & v=*args._VECTptr;
      s=v.size();
      if (!s)
	return args;
      if ( (args.subtype==_POLY1__VECT) || (s<2) )
	return v.front();
      x=v[1];
      p=v[0];
      if (s>2)
	order=v[2];
    }
    gen g=_e2r(makevecteur(p,x),contextptr),n,d;
    fxnd(g,n,d);
    if (n.type!=_VECT){
      if (n.type==_POLY){
	polynome nlcoeff(*n._POLYptr);
	if (!nlcoeff.coord.empty()){
	  if (order.type==_INT_)
	    change_monomial_order(nlcoeff,order);
	  nlcoeff.coord.erase(nlcoeff.coord.begin()+1,nlcoeff.coord.end());
	}
	n=nlcoeff;
      }
      return _r2e(makevecteur(n/d,x),contextptr);
    }
    return n._VECTptr->front()/d;
  }
  const char _lcoeff_s []="lcoeff";
  const unary_function_eval __lcoeff(&_lcoeff,_lcoeff_s);
  const unary_function_ptr at_lcoeff (&__lcoeff,0,true);

  gen tcoeff(const vecteur & v){
    int s=v.size();
    gen g;
    for (;s;--s){
      g=v[s-1];
      if (!is_zero(g))
	return g;
    }
    return zero;
  }
  gen _tcoeff(const gen & args,GIAC_CONTEXT){
    gen x,p;
    if (args.type!=_VECT){
      x=vx_var;
      p=args;
    }
    else {
      vecteur& v=*args._VECTptr;
      int s=v.size();
      if ( (args.subtype==_POLY1__VECT) || (s!=2) || (v[1].type!=_IDNT) )
	return tcoeff(v);
      x=v[1];
      p=v[0];
    }
    gen g=_e2r(makevecteur(p,x),contextptr),n,d;
    fxnd(g,n,d);
    if (n.type!=_VECT)
      return zero;
    return tcoeff(*n._VECTptr)/d;
  }
  const char _tcoeff_s []="tcoeff";
  const unary_function_eval __tcoeff(&_tcoeff,_tcoeff_s);
  const unary_function_ptr at_tcoeff (&__tcoeff,0,true);

  gen sqrfree(const gen & g,const vecteur & l,GIAC_CONTEXT){
    if (g.type!=_POLY)
      return r2sym(g,l,contextptr);
    factorization f(sqff(*g._POLYptr));
    factorization::const_iterator it=f.begin(),itend=f.end();
    gen res(plus_one);
    for (;it!=itend;++it)
      res=res*pow(r2e(it->fact,l,contextptr),it->mult);
    return res;
  }
  gen _sqrfree(const gen & args,GIAC_CONTEXT){
    if (args.type==_VECT)
      return apply(args,_sqrfree,contextptr);
    if (args.type!=_SYMB)
      return args;
    vecteur l(alg_lvar(args));
    fraction f(e2r(args,l,contextptr));
    return sqrfree(f.num,l,contextptr)/sqrfree(f.den,l,contextptr);
  }
  const char _sqrfree_s []="sqrfree";
  const unary_function_eval __sqrfree(&_sqrfree,_sqrfree_s);
  const unary_function_ptr at_sqrfree (&__sqrfree,0,true);

  gen _truncate(const gen & args,GIAC_CONTEXT){
    gen e(args);
    int n,s=1;
    vecteur w(1,vx_var);
    gen gn(5);
    if (args.type==_VECT){
      vecteur & v=*args._VECTptr;
      s=v.size();
      if (s==0)
	setsizeerr();
      e=v[0];
      if (s==3){
	w=gen2vecteur(v[1]);
	gn=v[2];
      }
      else {
	if (s==2)
	  gn=v[1];
      }
    }
    if (gn.type!=_INT_)
      setsizeerr();
    n=gn.val;
    int nvar=w.size(); // number of var w.r.t. which we truncate
    vecteur l(lop(e,at_order_size));
    vecteur lp(l.size(),zero);
    e=subst(e,l,lp,false,contextptr);
    // FIXME if l not empty, adjust order of truncation using arg of order_size
    lvar(e,w);
    e=e2r(e,w,contextptr);
    gen num,den;
    fxnd(e,num,den);
    if ( (den.type==_POLY) && (den._POLYptr->lexsorted_degree() ) )
      setsizeerr();
    if (num.type==_POLY){
      vector< monomial<gen> >::const_iterator it=num._POLYptr->coord.begin(),itend=num._POLYptr->coord.end();
      vector< monomial<gen> > res;
      for (;it!=itend;++it){
	index_t::const_iterator i=it->index.begin();
	int deg=0;
	for (int j=0;j<nvar;++j,++i)
	  deg=deg+(*i);
	if (deg<=n)
	  res.push_back(*it);
      }
      num._POLYptr->coord=res;
    }
    return r2e(rdiv(num,den),w,contextptr);
  }    
  const char _truncate_s []="truncate";
  const unary_function_eval __truncate(&_truncate,_truncate_s);
  const unary_function_ptr at_truncate (&__truncate,0,true);

  gen _canonical_form(const gen & args,GIAC_CONTEXT){
    gen p,x,a,b,c;
    if (args.type!=_VECT){
      x=vx_var;
      p=args;
    }
    else {
      vecteur & v=*args._VECTptr;
      if (v.size()!=2)
	settypeerr();
      p=v.front();
      x=v.back();
    }
    if (x.type!=_IDNT)
      settypeerr();
    if (!is_quadratic_wrt(p,*x._IDNTptr,a,b,c,contextptr))
      setsizeerr();
    if (is_zero(a))
      return b*x+c;
    // a*x^2+b*x+c -> a*(x+b/(2*a))^2+(b^2-4*a*c)/(4*a)
    return a*pow(x+b/(2*a),2)+(4*a*c-pow(b,2))/(4*a);
  }    
  const char _canonical_form_s []="canonical_form";
  const unary_function_eval __canonical_form(&_canonical_form,_canonical_form_s);
  const unary_function_ptr at_canonical_form (&__canonical_form,0,true);

  gen _fcoeff(const gen & args,GIAC_CONTEXT){
    gen x;
    vecteur p;
    if (args.type!=_VECT)
      return symbolic(at_fcoeff,args);
    vecteur & v=*args._VECTptr;
    if ( (v.size()!=2) || (v.front().type!=_VECT) ){
      p=v;
      x=vx_var;
    }
    else {
      p=*v.front()._VECTptr;
      x=v.back();
    }
    if (x.type!=_IDNT)
      settypeerr();    
    const_iterateur it=p.begin(),itend=p.end();
    if ( (itend-it)%2 )
      setsizeerr();
    gen res(plus_one);
    for (;it!=itend;it+=2){
      res=res*pow(x-*it,*(it+1),contextptr);
    }
    return res;
  }    
  const char _fcoeff_s []="fcoeff";
  const unary_function_eval __fcoeff(&_fcoeff,_fcoeff_s);
  const unary_function_ptr at_fcoeff (&__fcoeff,0,true);

  void addfactors(const gen & p,const gen & x,int mult,vecteur & res,GIAC_CONTEXT){
    vecteur v=factors(p,x,contextptr);
    const_iterateur it=v.begin(),itend=v.end();
    for (;it!=itend;){
      vecteur w=solve(*it,x,1,contextptr);
      ++it;
      int n=it->val;
      ++it;
      const_iterateur jt=w.begin(),jtend=w.end();
      for (;jt!=jtend;++jt){
	res.push_back(*jt);
	res.push_back(n*mult);
      }
    }
  }    

  gen _froot(const gen & args,GIAC_CONTEXT){
    gen p,x;
    if (args.type!=_VECT){
      x=vx_var;
      p=args;
    }
    else {
      vecteur & v=*args._VECTptr;
      if (v.size()!=2)
	setsizeerr();
      x=v.back();
      if (x.type!=_IDNT)
	setsizeerr("2nd arg");
      p=v.front();
    }
    vecteur lv(lvar(p));
    gen aa=e2r(p,lv,contextptr),aan,aad;
    fxnd(aa,aan,aad);
    vecteur res;
    addfactors(r2e(aan,lv,contextptr),x,1,res,contextptr);
    addfactors(r2e(aad,lv,contextptr),x,-1,res,contextptr);
    return res;
  }

  const char _froot_s []="froot";
  const unary_function_eval __froot(&_froot,_froot_s);
  const unary_function_ptr at_froot (&__froot,0,true);

  gen _roots(const gen & g,GIAC_CONTEXT){
    gen r=_froot(g,contextptr);
    if (r.type!=_VECT || (r._VECTptr->size() % 2) )
      setsizeerr();
    vecteur & v = *r._VECTptr;
    vecteur res;
    int s=v.size()/2;
    for (int i=0;i<s;++i){
      if (v[2*i+1].val>0)
	res.push_back(makevecteur(v[2*i],v[2*i+1]));
    }
    return res;
  }
  const char _roots_s []="roots";
  const unary_function_eval __roots(&_roots,_roots_s);
  const unary_function_ptr at_roots (&__roots,0,true);

  gen _divpc(const gen & args,GIAC_CONTEXT){
    gen p,q,x;
    if (args.type!=_VECT)
      return symbolic(at_divpc,args);
    vecteur & v=*args._VECTptr;
    int s=v.size();
    if (s<3)
      setsizeerr();
    p=v.front();
    q=v[1];
    if (v[2].type!=_INT_)
      setsizeerr();
    if (s==3)
      x=vx_var;
    else 
      x=v.back();
    vecteur lv(1,x);
    lvar(p,lv);
    lvar(q,lv);
    gen aa=e2r(p,lv,contextptr),aan,aad;
    fxnd(aa,aan,aad);
    gen ba=e2r(q,lv,contextptr),ban,bad;
    fxnd(ba,ban,bad);
    if ( ( (aad.type==_POLY)&&(aad._POLYptr->lexsorted_degree()) )
	 || (bad.type==_POLY)&&(bad._POLYptr->lexsorted_degree())
	 )
      setsizeerr();
    if (ban.type!=_POLY)
      return r2e(rdiv(aan*bad,ban*aad),lv,contextptr);
    vecteur a;
    if (aan.type==_POLY)
      a=polynome2poly1(*aan._POLYptr,1);
    else
      a=vecteur(1,aan);
    vecteur b=polynome2poly1(*ban._POLYptr,1);
    if (is_zero(b.back()))
      divisionby0err(q);
    reverse(a.begin(),a.end());
    reverse(b.begin(),b.end());
    int n=b.size()-a.size()+v[2].val;
    for (int i=0;i<n;++i)
      a.push_back(zero);
    vecteur quo,rem;
    environment * env=new environment;
    DivRem(a,b,env,quo,rem);
    delete env;
    reverse(quo.begin(),quo.end());
    gen res(vecteur2polynome(quo,lv.size()));
    res=rdiv(res*bad,aad);
    return r2e(res,lv,contextptr);
  }

  const char _divpc_s []="divpc";
  const unary_function_eval __divpc(&_divpc,_divpc_s);
  const unary_function_ptr at_divpc (&__divpc,0,true);

  gen _ptayl(const gen & args,GIAC_CONTEXT){
    gen p,q,x;
    if (args.type!=_VECT)
      return symbolic(at_ptayl,args);
    vecteur & v=*args._VECTptr;
    int s=v.size();
    if (s<2)
      setsizeerr();
    p=v.front();
    q=v[1];
    if (p.type==_VECT)
      return taylor(*p._VECTptr,q,0);
    if (s==2)
      x=vx_var;
    else 
      x=v.back();
    if (!is_zero(derive(q,x,contextptr)))
      setsizeerr();
    vecteur lv(1,x);
    lvar(p,lv);
    lvar(q,lv);
    gen aa=e2r(p,lv,contextptr),aan,aad;
    fxnd(aa,aan,aad);
    if ( ( (aad.type==_POLY)&&(aad._POLYptr->lexsorted_degree()) )
	 )
      setsizeerr();
    if (aan.type!=_POLY)
      return p;
    gen ba=e2r(q,vecteur(lv.begin()+1,lv.end()),contextptr);
    vecteur a(polynome2poly1(*aan._POLYptr,1));
    vecteur res=taylor(a,ba,0);
    return r2e(vecteur2polynome(res,lv.size()),lv,contextptr)/r2e(aad,lv,contextptr);
  }

  const char _ptayl_s []="ptayl";
  const unary_function_eval __ptayl(&_ptayl,_ptayl_s);
  const unary_function_ptr at_ptayl (&__ptayl,0,true);
  
  vecteur gen2continued_fraction(const gen & g,int n,GIAC_CONTEXT){
    // Compute a vector of size n+1 with last element=remainder
    vecteur res,remain;
    gen tmp(g),f;
    int i=0,j;
    for (;i<n;++i){
      if ( (j=equalposcomp(remain,tmp)) ){
	// int s=remain.size();
	res.push_back(vecteur(res.begin()+j-1,res.end()));
	return res;
      }
      else
	remain.push_back(tmp);
      f=_floor(tmp,0);
      res.push_back(f);
      if (is_zero(tmp-f))
	return res;
      tmp=normal(inv(tmp-f,contextptr),contextptr);
    }
    res.push_back(tmp);
    return res;
  }
  gen _dfc(const gen & g_orig,GIAC_CONTEXT){
    gen g=g_orig;
    if (g.type==_FRAC){
      gen tmp=_floor(g,contextptr);
      vecteur res(1,tmp);
      g -= tmp;
      for (;!is_zero(g);){
	g = inv(g,contextptr);
	tmp = _floor(g,contextptr);
	res.push_back(tmp);
	g -=tmp;
      }
      return res;
    }
    double eps=epsilon(contextptr);
    if (g.type==_VECT && g._VECTptr->size()==2){
      if (g._VECTptr->back().type==_DOUBLE_){
	eps=g._VECTptr->back()._DOUBLE_val;
	g=evalf_double(g._VECTptr->front(),1,contextptr);
      }
      else {
	if (g._VECTptr->back().type==_INT_)
	  return gen2continued_fraction(g._VECTptr->front(),g._VECTptr->back().val,contextptr);
      }
    }
    g=evalf_double(g,1,contextptr);
    if (g.type!=_DOUBLE_)
      setsizeerr();
    return vector_int_2_vecteur(float2continued_frac(g._DOUBLE_val,eps),contextptr);
  }
  const char _dfc_s []="dfc";
  const unary_function_eval __dfc(&_dfc,_dfc_s);
  const unary_function_ptr at_dfc (&__dfc,0,true);

  gen _dfc2f(const gen & g,GIAC_CONTEXT){
    if (g.type!=_VECT || g._VECTptr->empty())
      setsizeerr();
    vecteur & v (*g._VECTptr);
    gen res(v.back());
    if (v.back().type==_VECT){
      // represent a quadratic x=[... x], find equation
      identificateur tmp(" x");
      gen eq(tmp);
      const_iterateur it=v.back()._VECTptr->end()-1,itend=v.back()._VECTptr->begin()-1;
      for (;it!=itend;--it)
	eq=inv(eq,contextptr)+(*it);
      vecteur w=solve(eq-tmp,tmp,0,contextptr);
      gen ws=_sort(w,0);
      if (ws.type!=_VECT || ws._VECTptr->empty())
	setsizeerr();
      res=ws._VECTptr->back();
    }
    for (;;){
      v.pop_back();
      if (v.empty())
	return res;
      res=inv(res,contextptr);
      res=res+v.back();
    }
    // return continued_frac2gen(vecteur_2_vector_int(*g._VECTptr),nan(),epsilon);
  }
  const char _dfc2f_s []="dfc2f";
  const unary_function_eval __dfc2f(&_dfc2f,_dfc2f_s);
  const unary_function_ptr at_dfc2f (&__dfc2f,0,true);

  gen float2rational(double d_orig,double eps,GIAC_CONTEXT){
    double d=d_orig;
    if (d<0)
      return -float2rational(-d,eps,contextptr);
    if (d>RAND_MAX)
      return d;    // reconstruct
    vector<int> v(float2continued_frac(d,eps));
    return continued_frac2gen(v,d_orig,eps,contextptr);
  }
  gen _float2rational(const gen & g,GIAC_CONTEXT){
    switch (g.type){
    case _DOUBLE_:
      return float2rational(g._DOUBLE_val,epsilon(contextptr),contextptr);
    case _CPLX:
      return _float2rational(re(g,contextptr),contextptr)+cst_i*_float2rational(im(g,contextptr),contextptr);
    case _SYMB:
      return symbolic(g._SYMBptr->sommet,_float2rational(g._SYMBptr->feuille,contextptr));
    case _VECT:
      return apply(g,_float2rational,contextptr);
    default:
      return g;
    }
  }
  const char _float2rational_s []="float2rational";
  const unary_function_eval __float2rational(&_float2rational,_float2rational_s);
  const unary_function_ptr at_float2rational (&__float2rational,0,true);

  gen _gramschmidt(const gen & g,GIAC_CONTEXT){
    if (g.type!=_VECT)
      return symbolic(at_gramschmidt,g);
    vecteur & v(*g._VECTptr);
    if (ckmatrix(v))
      return gramschmidt(v,true,contextptr);
    if (v.size()==2){
      gen lvect=v[0];
      gen scalaire=v[1];
      if (lvect.type!=_VECT)
	setsizeerr();
      vecteur lv=*lvect._VECTptr;
      int s=lv.size();
      if (!s)
	return lv;
      vecteur sc(1,scalaire(gen(makevecteur(lv[0],lv[0]),_SEQ__VECT),contextptr));
      for (int i=1;i<s;++i){
	gen cl;
	for (int j=0;j<i;++j)
	  cl=cl+rdiv(scalaire(gen(makevecteur(lv[i],lv[j]),_SEQ__VECT),contextptr),sc[j])*lv[j];
	lv[i]=lv[i]-cl;
	sc.push_back(scalaire(gen(makevecteur(lv[i],lv[i]),_SEQ__VECT),contextptr));
      }
      for (int i=0;i<s;++i)
	lv[i]=rdiv(lv[i],sqrt(sc[i],contextptr));
      return lv;
    }
    setsizeerr();
    return 0;
  }
  const char _gramschmidt_s []="gramschmidt";
  const unary_function_eval __gramschmidt(&_gramschmidt,_gramschmidt_s);
  const unary_function_ptr at_gramschmidt (&__gramschmidt,0,true);

  void aplatir(const matrice & m,vecteur & v){
    int s=m.size();
    v.clear();
    v.reserve(s*s);
    const_iterateur it=m.begin(),itend=m.end(),jt,jtend;
    for (;it!=itend;++it){
      if (it->type!=_VECT)
	v.push_back(*it);
      else {
	jt=it->_VECTptr->begin(),jtend=it->_VECTptr->end();
	for (;jt!=jtend;++jt)
	  v.push_back(*jt);
      }
    }
  }

  gen pmin(const matrice & m,GIAC_CONTEXT){
    int s=m.size();
    matrice mpow(midn(s));
    matrice res;
    vecteur v;
    for (int i=0;i<=s;++i){
      aplatir(mpow,v);
      v.push_back(pow(vx_var,i));
      res.push_back(v);
      mpow=mmult(mpow,m);
    }
    matrice r;
    gen det;
    mrref(res,r,v,det,0,s+1,0,s*s,
	  false,1,true,1,0,
	  contextptr);
    // find 1st line with zeros (except in the last col)
    const_iterateur it=r.begin(),itend=r.end();
    for (;it!=itend;++it){
      if (is_zero(vecteur(it->_VECTptr->begin(),it->_VECTptr->end()-1)))
	break;
    }
    if (it==itend)
      setsizeerr();
    gen t= _e2r(makevecteur(it->_VECTptr->back(),vx_var),contextptr);
    if (t.type==_VECT)
      return t/lgcd(*t._VECTptr);
    else
      return t;
  }
  gen _pmin(const gen & g,GIAC_CONTEXT){
    if (is_squarematrix(g)){
      matrice &m =*g._VECTptr;
      vecteur w;
      if (proba_epsilon(contextptr) && probabilistic_pmin(m,w,true,contextptr))
	return w;
      return pmin(m,contextptr);
    }
    if (g.type!=_VECT || g._VECTptr->size()!=2)
      return symbolic(at_pmin,g);
    vecteur & v(*g._VECTptr);
    if (!is_squarematrix(v.front()))
      setsizeerr();
    matrice &m=*v.front()._VECTptr;
    // probabilistic minimal polynomial
    vecteur w;
    if (proba_epsilon(contextptr) &&probabilistic_pmin(m,w,true,contextptr))
      return symb_horner(w,v.back());
    else
      return _r2e(makevecteur(pmin(m,contextptr),v.back()),contextptr);
  }
  const char _pmin_s []="pmin";
  const unary_function_eval __pmin(&_pmin,_pmin_s);
  const unary_function_ptr at_pmin (&__pmin,0,true);

  // a faire: vpotential, signtab
  gen _potential(const gen & g,GIAC_CONTEXT){
    if ( (g.type!=_VECT) || (g._VECTptr->size()!=2) )
      return symbolic(at_potential,g);
    vecteur v(plotpreprocess(g,contextptr));
    gen f=v[0];
    gen x=v[1];
    if ( (f.type!=_VECT) || (x.type!=_VECT) )
      setsizeerr();
    vecteur & fv=*f._VECTptr;
    vecteur & xv=*x._VECTptr;
    int s=fv.size();
    if (unsigned(s)!=xv.size())
      setdimerr();
    for (int i=0;i<s;++i){
      for (int j=i+1;j<s;++j){
	if (!is_zero(simplify(derive(fv[i],xv[j],contextptr)-derive(fv[j],xv[i],contextptr),contextptr)))
	  setsizeerr("Not a potential");
      }
    }
    gen res;
    for (int i=0;i<s;++i){
      res=res+integrate(simplify(fv[i]-derive(res,xv[i],contextptr),contextptr),xv[i],contextptr);
    }
    return res;
  }
  const char _potential_s []="potential";
  const unary_function_eval __potential(&_potential,_potential_s);
  const unary_function_ptr at_potential (&__potential,_QUOTE_ARGUMENTS,true);

  gen _vpotential(const gen & g,GIAC_CONTEXT){
    if ( (g.type!=_VECT) || (g._VECTptr->size()!=2) )
      return symbolic(at_vpotential,g);
    vecteur v(plotpreprocess(g,contextptr));
    gen f=v[0];
    gen x=v[1];
    if ( (f.type!=_VECT) || (x.type!=_VECT) )
      setsizeerr();
    vecteur & fv=*f._VECTptr;
    vecteur & xv=*x._VECTptr;
    unsigned int s=fv.size();
    if ( (s!=3) || (s!=xv.size()) )
      setdimerr();
    if (!is_zero(simplify(_divergence(g,contextptr),contextptr)))
      setsizeerr("Not a vector potential");
    vecteur res(3);
    /* return A0=0, A1=int[B_2,x0], A2=-int[B_1,x0]+F(x1,x2)
     * where F=int[B0+d_2[int[B_2,x0]]+d_1[int[B_1,x0]],x1]
     * F does not depend on x0 since divergence[B]=0 */
    res[1]=integrate(fv[2],xv[0],contextptr);
    res[2]=integrate(fv[1],xv[0],contextptr);
    gen F=simplify(fv[0]+derive(res[1],xv[2],contextptr)+derive(res[2],xv[1],contextptr),contextptr);
    F=integrate(F,xv[1],contextptr);
    res[2]=F-res[2];
    return res;
  }
  const char _vpotential_s []="vpotential";
  const unary_function_eval __vpotential(&_vpotential,_vpotential_s);
  const unary_function_ptr at_vpotential (&__vpotential,_QUOTE_ARGUMENTS,true);

  gen _poly2symb(const gen & g,GIAC_CONTEXT){
    if (g.type==_VECT && g.subtype!=_SEQ__VECT)
      return _r2e(gen(makevecteur(g,vx_var),_SEQ__VECT),contextptr);      
    return _r2e(g,contextptr);
  }
  const char _poly2symb_s []="poly2symb";
  const unary_function_eval __poly2symb(&_poly2symb,_poly2symb_s);
  const unary_function_ptr at_poly2symb (&__poly2symb,0,true);

  gen _symb2poly(const gen & g,GIAC_CONTEXT){
    return _e2r(g,contextptr);
  }
  const char _symb2poly_s []="symb2poly";
  const unary_function_eval __symb2poly(&_symb2poly,_symb2poly_s);
  const unary_function_ptr at_symb2poly (&__symb2poly,0,true);

  gen _exp2trig(const gen & g,GIAC_CONTEXT){
    return _sincos(g,contextptr);
  }
  const char _exp2trig_s []="exp2trig";
  const unary_function_eval __exp2trig(&_exp2trig,_exp2trig_s);
  const unary_function_ptr at_exp2trig (&__exp2trig,0,true);

  gen _nrows(const gen & g,GIAC_CONTEXT){
    if (!ckmatrix(g))
      setsizeerr();
    return int(g._VECTptr->size());
  }
  const char _nrows_s []="nrows";
  const unary_function_eval __nrows(&_nrows,_nrows_s);
  const unary_function_ptr at_nrows (&__nrows,0,true);

  gen _ncols(const gen & g,GIAC_CONTEXT){
    if (!ckmatrix(g))
      setsizeerr();
    if (g._VECTptr->empty())
      return zero;
    return int(g._VECTptr->front()._VECTptr->size());
  }
  const char _ncols_s []="ncols";
  const unary_function_eval __ncols(&_ncols,_ncols_s);
  const unary_function_ptr at_ncols (&__ncols,0,true);

  gen _l2norm(const gen & g0,GIAC_CONTEXT){
    gen g=remove_at_pnt(g0);
    if (g.type==_VECT && g.subtype==_VECTOR__VECT && g._VECTptr->size()==2)
      g=g._VECTptr->back()-g._VECTptr->front();
    if (g.type!=_VECT)
      return abs(g,contextptr);
    vecteur v;
    if (ckmatrix(g))
      aplatir(*g._VECTptr,v);
    else
      v=*g._VECTptr;
    return l2norm(v,contextptr);
  }
  const char _l2norm_s []="l2norm";
  const unary_function_eval __l2norm(&_l2norm,_l2norm_s);
  const unary_function_ptr at_l2norm (&__l2norm,0,true);

  const char _norm_s []="norm";
  const unary_function_eval __norm(&_l2norm,_norm_s);
  const unary_function_ptr at_norm (&__norm,0,true);

  gen _normalize(const gen & a,GIAC_CONTEXT){
    return a/_l2norm(a,contextptr);
  }
  const char _normalize_s []="normalize";
  const unary_function_eval __normalize(&_normalize,_normalize_s);
  const unary_function_ptr at_normalize (&__normalize,0,true);

  const char _randmatrix_s []="randmatrix";
  const unary_function_eval __randmatrix(&_ranm,_randmatrix_s);
  const unary_function_ptr at_randmatrix (&__randmatrix,0,true);

  extern const unary_function_ptr at_lgcd;
  gen _lgcd(const gen & g,GIAC_CONTEXT){
    if (g.type!=_VECT)
      return symbolic(at_lgcd,g);
    return lgcd(*g._VECTptr);
  }
  const char _lgcd_s []="lgcd";
  const unary_function_eval __lgcd(&_lgcd,_lgcd_s);
  const unary_function_ptr at_lgcd (&__lgcd,0,true);

  // synonyms
  gen _float(const gen & g,GIAC_CONTEXT){
    return evalf(g,1,contextptr);
  }
  const char _float_s []="float";
  const unary_function_eval __float(&_float,_float_s);
  const unary_function_ptr at_float (&__float,0,true);

  gen _hold(const gen & g,GIAC_CONTEXT){
    return g;
  }
  const char _hold_s []="hold";
  const unary_function_eval __hold(&_hold,_hold_s);
  const unary_function_ptr at_hold (&__hold,_QUOTE_ARGUMENTS,true);

  gen _eigenvals(const gen & g,GIAC_CONTEXT){
    return gen(*_diag(_egvl(g,contextptr),contextptr)._VECTptr,_SEQ__VECT);
  }
  const char _eigenvals_s []="eigenvals";
  const unary_function_eval __eigenvals(&_eigenvals,_eigenvals_s);
  const unary_function_ptr at_eigenvals (&__eigenvals,0,true);

  const char _giackernel_s []="kernel";
  const unary_function_eval __giackernel(&_ker,_giackernel_s);
  const unary_function_ptr at_kernel (&__giackernel,0,true);

  const char _eigenvects_s []="eigenvects";
  const unary_function_eval __eigenvects(&_egv,_eigenvects_s);
  const unary_function_ptr at_eigenvects (&__eigenvects,0,true);

  const char _eigenvalues_s []="eigenvalues";
  const unary_function_eval __eigenvalues(&_egvl,_eigenvalues_s);
  const unary_function_ptr at_eigenvalues (&__eigenvalues,0,true);

  const char _charpoly_s []="charpoly";
  const unary_function_eval __charpoly(&giac::_pcar,_charpoly_s);
  const unary_function_ptr at_charpoly (&__charpoly,0,true);

  const char _eigenvectors_s []="eigenvectors";
  const unary_function_eval __eigenvectors(&_egv,_eigenvectors_s);
  const unary_function_ptr at_eigenvectors (&__eigenvectors,0,true);

  const char _rowdim_s []="rowdim";
  const unary_function_eval __rowdim(&_nrows,_rowdim_s);
  const unary_function_ptr at_rowdim (&__rowdim,0,true);

  const char _coldim_s []="coldim";
  const unary_function_eval __coldim(&_ncols,_coldim_s);
  const unary_function_ptr at_coldim (&__coldim,0,true);

  const char _multiply_s []="multiply";
  const unary_function_eval __multiply(&_prod,_multiply_s);
  const unary_function_ptr at_multiply (&__multiply,0,true);

  /* Maple inert forms */
  gen _Gcd(const gen & g,GIAC_CONTEXT){
    return symbolic(at_gcd,g);
  }
  const char _Gcd_s []="Gcd";
  const unary_function_eval __Gcd(&_Gcd,_Gcd_s);
  const unary_function_ptr at_Gcd (&__Gcd,0,true);

  gen _Gcdex(const gen & g,GIAC_CONTEXT){
    return symbolic(at_gcdex,g);
  }
  const char _Gcdex_s []="Gcdex";
  const unary_function_eval __Gcdex(&_Gcdex,_Gcdex_s);
  const unary_function_ptr at_Gcdex (&__Gcdex,0,true);

  gen _Factor(const gen & g,GIAC_CONTEXT){
    return symbolic(at_factor,g);
  }
  const char _Factor_s []="Factor";
  const unary_function_eval __Factor(&_Factor,_Factor_s);
  const unary_function_ptr at_Factor (&__Factor,0,true);

  gen _Rref(const gen & g,GIAC_CONTEXT){
    return symbolic(at_rref,g);
  }
  const char _Rref_s []="Rref";
  const unary_function_eval __Rref(&_Rref,_Rref_s);
  const unary_function_ptr at_Rref (&__Rref,0,true);

  gen _Rank(const gen & g,GIAC_CONTEXT){
    return symbolic(at_rank,g);
  }
  const char _Rank_s []="Rank";
  const unary_function_eval __Rank(&_Rank,_Rank_s);
  const unary_function_ptr at_Rank (&__Rank,0,true);

  gen _Det(const gen & g,GIAC_CONTEXT){
    return symbolic(at_det,g);
  }
  const char _Det_s []="Det";
  const unary_function_eval __Det(&_Det,_Det_s);
  const unary_function_ptr at_Det (&__Det,0,true);

  gen _Quo(const gen & g,GIAC_CONTEXT){
    return symbolic(at_quo,g);
  }
  const char _Quo_s []="Quo";
  const unary_function_eval __Quo(&_Quo,_Quo_s);
  const unary_function_ptr at_Quo (&__Quo,0,true);

  gen _Rem(const gen & g,GIAC_CONTEXT){
    return symbolic(at_rem,g);
  }
  const char _Rem_s []="Rem";
  const unary_function_eval __Rem(&_Rem,_Rem_s);
  const unary_function_ptr at_Rem (&__Rem,0,true);

  gen _Int(const gen & g,GIAC_CONTEXT){
    return symbolic(at_integrate,g);
  }
  const char _Int_s []="Int";
  const unary_function_eval __Int(&_Int,_Int_s);
  const unary_function_ptr at_Int (&__Int,0,true);

  gen _divisors(const gen & g,GIAC_CONTEXT){
    return gen(*_idivis(g,contextptr)._VECTptr,_SET__VECT);
  }
  const char _divisors_s []="divisors";
  const unary_function_eval __divisors(&_divisors,_divisors_s);
  const unary_function_ptr at_divisors (&__divisors,0,true);

  gen _maxnorm(const gen & g0,GIAC_CONTEXT){
    gen g=remove_at_pnt(g0);
    if (g.type==_VECT && g.subtype==_VECTOR__VECT)
      g=vector2vecteur(*g._VECTptr);
    return linfnorm(g,contextptr);
  }
  const char _maxnorm_s []="maxnorm";
  const unary_function_eval __maxnorm(&_maxnorm,_maxnorm_s);
  const unary_function_ptr at_maxnorm (&__maxnorm,0,true);

  gen _l1norm(const gen & g0,GIAC_CONTEXT){
    gen g=remove_at_pnt(g0);
    if (g.type==_VECT && g.subtype==_VECTOR__VECT)
      g=vector2vecteur(*g._VECTptr);
    if (g.type!=_VECT)
      return linfnorm(g,contextptr);
    gen res;
    const_iterateur it=g._VECTptr->begin(),itend=g._VECTptr->end();
    for (;it!=itend;++it)
      res=res+linfnorm(*it,contextptr);
    return res;
  }
  const char _l1norm_s []="l1norm";
  const unary_function_eval __l1norm(&_l1norm,_l1norm_s);
  const unary_function_ptr at_l1norm (&__l1norm,0,true);

  gen _dotprod(const gen & g,GIAC_CONTEXT){
    if ( (g.type!=_VECT) || (g._VECTptr->size()!=2))
      settypeerr();
    vecteur v=*g._VECTptr;
    return dotvecteur(v[0],v[1]);
  }
  const char _dotprod_s []="dotprod";
  const unary_function_eval __dotprod(&_dotprod,_dotprod_s);
  const unary_function_ptr at_dotprod (&__dotprod,0,true);

  const char _crossproduct_s []="crossproduct";
  const unary_function_eval __crossproduct(&_cross,_crossproduct_s);
  const unary_function_ptr at_crossproduct (&__crossproduct,0,true);

  gen _diag(const gen & g,GIAC_CONTEXT){
    if (g.type!=_VECT || g._VECTptr->empty())
      setsizeerr();
    vecteur v=*g._VECTptr;
    int l=v.size();
    if (is_squarematrix(v)){
      vecteur res(l);
      for (int i=0;i<l;++i)
	res[i]=v[i][i];
      return res;
    }
    if (ckmatrix(v)){
      if (l==1 && v[0].type==_VECT){
	v=*v[0]._VECTptr;
      }
      else
	v=*mtran(v)[0]._VECTptr;
    }
    l=v.size();
    matrice res;
    if (l && ckmatrix(v.front()) ){
      int s=0,r=0;
      for (int i=0;i<l;++i){
	if (!is_squarematrix(v[i]))
	  settypeerr();
	s += v[i]._VECTptr->size();
      }
      for (int i=0;i<l;++i){
	vecteur & current=*v[i]._VECTptr;
	int c=current.size();
	for (int j=0;j<c;++j){
	  vecteur tmp(r);
	  vecteur & currentj=*current[j]._VECTptr;
	  for (int k=0;k<c;++k){
	    tmp.push_back(currentj[k]);
	  }
	  for (int k=c+r;k<s;++k)
	    tmp.push_back(zero);
	  res.push_back(tmp);
	}
	r += c;
      }
      return res;
    }
    for (int i=0;i<l;++i){
      vecteur tmp(i);
      tmp.push_back(v[i]);
      res.push_back(mergevecteur(tmp,vecteur(l-1-i)));
    }
    return res;
  }
  const char _diag_s []="diag";
  const unary_function_eval __diag(&_diag,_diag_s);
  const unary_function_ptr at_diag (&__diag,0,true);

  const char _BlockDiagonal_s []="BlockDiagonal";
  const unary_function_eval __BlockDiagonal(&_diag,_BlockDiagonal_s);
  const unary_function_ptr at_BlockDiagonal (&__BlockDiagonal,0,true);

  gen _input(const gen & args,GIAC_CONTEXT){
    return _input(args,false,contextptr);
  }
  const char _input_s []="input";
  unary_function_eval __input(&_input,_input_s);
  const unary_function_ptr at_input (&__input,_QUOTE_ARGUMENTS,true);

  gen _textinput(const gen & args,GIAC_CONTEXT){
    return _input(args,true,contextptr);
  }
  const char _textinput_s []="textinput";
  const unary_function_eval __textinput(&_textinput,_textinput_s);
  const unary_function_ptr at_textinput (&__textinput,_QUOTE_ARGUMENTS,true);

  const char _f2nd_s []="f2nd";
  const unary_function_eval __f2nd(&_fxnd,_f2nd_s);
  const unary_function_ptr at_f2nd (&__f2nd,0,true);

  // service=-3 for content -2 for primpart -1 for coeff, degree for coeff
  gen primpartcontent(const gen& g,int service,GIAC_CONTEXT){
    vecteur v;
    if (g.type==_VECT && g.subtype !=_SEQ__VECT)
      v=makevecteur(g,vx_var);
    else
      v=gen2vecteur(g);
    int s=v.size();
    if (s==2 && v[1].is_symb_of_sommet(at_pow)){
      gen & f = v[1]._SYMBptr->feuille;
      if (f.type==_VECT && f._VECTptr->size()==2 && f._VECTptr->back().type==_INT_){
	v[1]=f._VECTptr->front();
	service=f._VECTptr->back().val;
      }
    }
    if (s>=2 && v[1].type==_VECT){
      vecteur l(*v[1]._VECTptr);
      int outerdim=l.size();
      lvar(v[0],l);
      int innerdim=l.size()-outerdim;
      fraction f(sym2r(v[0],l,contextptr));
      vecteur ll(l.begin()+outerdim,l.end());
      if (f.num.type!=_POLY){
	if (service==-1){
	  gen res=r2e(v[0],l,contextptr);
	  if (s==3){
	    if (is_zero(v[2]))
	      return res;
	    else
	      return zero;
	  }
	  return makevecteur(res);
	}
	if (service==-2)
	  return r2e(inv(f.den,contextptr),l,contextptr);
	if (service==-3)
	  return r2e(f.num,l,contextptr);
	setsizeerr();
      }
      polynome & p_aplati=*f.num._POLYptr;
      polynome p=splitmultivarpoly(p_aplati,innerdim); 
      vector< monomial<gen> >::const_iterator it=p.coord.begin(),itend=p.coord.end();
      vecteur coeffs;
      coeffs.reserve(itend-it);
      for (;it!=itend;++it)
	coeffs.push_back(it->value);
      if (service==-1){
	gen gden=r2e(f.den,l,contextptr);
	if (s==3 && v[2].type==_VECT){
	  index_t ind;
	  if (!vecteur2index(*v[2]._VECTptr,ind))
	    return zero;
	  index_m i(ind);
	  it=p.coord.begin();
	  for (;it!=itend;++it){
	    if (it->index==i)
	      return r2e(it->value,ll,contextptr)/gden;
	  }
	  return zero;
	}
	return r2e(coeffs,ll,contextptr)/gden;
      }
      if (service==-2){
	p=p/_lgcd(coeffs,contextptr);
	p=unsplitmultivarpoly(p,innerdim);
	return r2e(p/f.den,l,contextptr);
      }
      if (service==-3)
	return r2e(_lgcd(coeffs,contextptr),ll,contextptr);
      setsizeerr();
    }
    if (s!=1 && s!=2)
      setsizeerr();
    gen x(vx_var);
    if (s==2)
      x=v[1];
    gen f(_e2r(makevecteur(v[0],x),contextptr));
    gen deno(1);
    if (f.type==_FRAC){
      deno=f._FRACptr->den;
      f=f._FRACptr->num;
    }
    if (f.type!=_VECT){
      switch(service){
      case -1:
	return makevecteur(f)/deno;
      case -2:
	return plus_one;
      case -3:
	return f;
      default:
	if (service>0)
	  return zero;
	else
	  return f;
      }
    }
    switch (service){
    case -1:
      return f/deno;
    case -2:
      return symb_horner(*f._VECTptr/_lgcd(f,contextptr),x);
    case -3:
      return _lgcd(f,contextptr)/deno;
    }
    vecteur & w=*f._VECTptr;
    int ss=w.size();
    if (service>=ss)
      return zero;
    return w[ss-service-1]/deno;
  }
  gen _primpart(const gen & g,GIAC_CONTEXT){
    return primpartcontent(g,-2,contextptr);
  }
  const char _primpart_s []="primpart";
  const unary_function_eval __primpart(&_primpart,_primpart_s);
  const unary_function_ptr at_primpart (&__primpart,0,true);

  gen _content(const gen & g,GIAC_CONTEXT){
    return primpartcontent(g,-3,contextptr);
  }
  const char _content_s []="content";
  const unary_function_eval __content(&_content,_content_s);
  const unary_function_ptr at_content (&__content,0,true);

  const char _icontent_s []="icontent";
  const unary_function_eval __icontent(&_content,_icontent_s);
  const unary_function_ptr at_icontent (&__icontent,0,true);

  gen _coeff(const gen & g,GIAC_CONTEXT){
    if (g.type==_VECT && !g._VECTptr->empty() && g._VECTptr->back().type==_INT_){
      vecteur v=*g._VECTptr;
      int n=absint(v.back().val);
      v.pop_back();
      return primpartcontent(gen(v,g.subtype),n,contextptr);
    }
    if (xcas_mode(contextptr)==1 && g.type==_VECT && g._VECTptr->size()==2 && g._VECTptr->back().type==_IDNT){
      return primpartcontent(g,1,contextptr);
    }
    return primpartcontent(g,-1,contextptr);
  }
  const char _coeff_s []="coeff";
  const unary_function_eval __coeff(&_coeff,_coeff_s);
  const unary_function_ptr at_coeff (&__coeff,0,true);

  const char _coeffs_s []="coeffs";
  const unary_function_eval __coeffs(&_coeff,_coeffs_s);
  const unary_function_ptr at_coeffs (&__coeffs,0,true);

  const char _ichrem_s []="ichrem";
  const unary_function_eval __ichrem(&_ichinrem,_ichrem_s);
  const unary_function_ptr at_ichrem (&__ichrem,0,true);

  gen _chrem(const gen & g,GIAC_CONTEXT){
    if (!ckmatrix(g) || g._VECTptr->size()!=2)
      setsizeerr();
    matrice m=mtran(*g._VECTptr);
    const_iterateur it=m.begin(),itend=m.end();
    if (it==itend)
      setsizeerr();
    gen res=*it;
    for (++it;it!=itend;++it){
      res=_ichinrem(makevecteur(res,*it),contextptr);
    }
    return res;
  }
  const char _chrem_s []="chrem";
  const unary_function_eval __chrem(&_chrem,_chrem_s);
  const unary_function_ptr at_chrem (&__chrem,0,true);

  gen _genpoly(const gen & g,GIAC_CONTEXT){
    if (g.type!=_VECT || g._VECTptr->size()!=3)
      settypeerr();
    vecteur & v=*g._VECTptr;
    gen n=v[0],b=v[1],x=v[2];
    if (b.type!=_INT_ && b.type!=_ZINT)
      settypeerr();
    b=abs(b,contextptr);
    if (is_zero(b)||is_one(b))
      setsizeerr();
    vecteur l(lvar(n));
    fraction f(e2r(n,l,contextptr));
    if (is_integer(f.num))
      f.num=pzadic(polynome(f.num,0),b);
    else {
      if (f.num.type==_POLY)
	f.num=pzadic(*f.num._POLYptr,b);
    }
    if (is_integer(f.den))
      f.den=pzadic(polynome(f.den,0),b);
    else {
      if (f.den.type==_POLY)
	f.den=pzadic(*f.den._POLYptr,b);
    }
    l.insert(l.begin(),x);
    return r2e(f,l,contextptr);
  }
  const char _genpoly_s []="genpoly";
  const unary_function_eval __genpoly(&_genpoly,_genpoly_s);
  const unary_function_ptr at_genpoly (&__genpoly,0,true);

  gen freq_quantile(const matrice & v,double d,GIAC_CONTEXT){
    if (!ckmatrix(v))
      return undef;
    matrice w;
    if (v.size()==2)
      w=mtran(v);
    else
      w=v;
    if (w.front()._VECTptr->size()!=2)
      return undef;
    // Row Sort (using row 1)
    sort(w.begin(),w.end(),first_ascend_sort);
    w=mtran(w);
    // w[0]=data, w[1]=frequencies
    vecteur data=*w[0]._VECTptr;
    vecteur freq=*w[1]._VECTptr;
    gen sigma=d*prodsum(freq,false);
    int s=freq.size();
    gen partial_sum;
    for (int i=0;i<s;++i){
      partial_sum=partial_sum+freq[i];
      if (!is_zero(partial_sum) && is_greater(partial_sum,sigma,contextptr))
	return data[i];
    }
    return undef;
  }
  gen _median(const gen & g,GIAC_CONTEXT){
    vecteur v(gen2vecteur(g));
    if (g.type==_VECT && g.subtype==_SEQ__VECT && v.size()==2)
      return freq_quantile(v,0.5,contextptr);
    if (!ckmatrix(v)){
      if (!is_fully_numeric(evalf(v,1,contextptr))){
	sort(v.begin(),v.end(),islesscomplexthanf);
	return v[int(std::ceil(v.size()/2.0))-1]; // v[(v.size()-1)/4];
      }
      return mtran(ascsort(mtran(vecteur(1,v)),true))[int(std::ceil(v.size()/2.0))-1][0];
    }
    else
      v=ascsort(v,true);
    v=mtran(v);
    return v[int(std::ceil(v.size()/2.0))-1]; // v[(v.size()-1)/2];
  }
  const char _median_s []="median";
  const unary_function_eval unary_median(&_median,_median_s);
  const unary_function_ptr at_median (&unary_median,0,true);

  gen _quartile1(const gen & g,GIAC_CONTEXT){
    vecteur v(gen2vecteur(g));
    if (g.type==_VECT && g.subtype==_SEQ__VECT && v.size()==2)
      return freq_quantile(v,0.25,contextptr);
    if (!ckmatrix(v)){
      if (!is_fully_numeric(evalf(v,1,contextptr))){
	sort(v.begin(),v.end(),islesscomplexthanf);
	return v[int(std::ceil(v.size()/4.0))-1]; // v[(v.size()-1)/4];
      }
      return mtran(ascsort(mtran(vecteur(1,v)),true))[int(std::ceil(v.size()/4.0))-1][0];
    }
    else
      v=ascsort(v,true);
    v=mtran(v);
    return v[int(std::ceil(v.size()/4.0))-1]; // v[(v.size()-1)/4];
  }
  const char _quartile1_s []="quartile1";
  const unary_function_eval unary_quartile1(&_quartile1,_quartile1_s);
  const unary_function_ptr at_quartile1 (&unary_quartile1,0,true);

  gen _quartile3(const gen & g,GIAC_CONTEXT){
    vecteur v(gen2vecteur(g));
    if (g.type==_VECT && g.subtype==_SEQ__VECT && v.size()==2)
      return freq_quantile(v,0.75,contextptr);
    if (!ckmatrix(v)){
      if (!is_fully_numeric(evalf(v,1,contextptr))){
	sort(v.begin(),v.end(),islesscomplexthanf);
	return v[int(std::ceil(3*v.size()/4.0))-1]; // v[(v.size()-1)/4];
      }
      return mtran(ascsort(mtran(vecteur(1,v)),true))[int(std::ceil(3*v.size()/4.0))-1][0];
    }
    else
      v=ascsort(v,true);
    v=mtran(v);
    return v[int(std::ceil(3*v.size()/4.0))-1]; // v[(3*(v.size()-1))/4];
  }
  const char _quartile3_s []="quartile3";
  const unary_function_eval unary_quartile3(&_quartile3,_quartile3_s);
  const unary_function_ptr at_quartile3 (&unary_quartile3,0,true);

  gen _quantile(const gen & g,GIAC_CONTEXT){
    vecteur v(gen2vecteur(g));
    if (v.size()<2 || v.front().type!=_VECT || v.back().type!=_DOUBLE_)
      setsizeerr();
    double d=v.back()._DOUBLE_val;
    if (d<0 || d>=1)
      setdimerr("");
    if (g.type==_VECT && g.subtype==_SEQ__VECT && v.size()==3)
      return freq_quantile(makevecteur(v[0],v[1]),d,contextptr);
    v=*v.front()._VECTptr;
    if (!ckmatrix(v)){
      if (!is_fully_numeric(evalf(v,1,contextptr))){
	sort(v.begin(),v.end(),islesscomplexthanf);
	return v[int(std::ceil(d*v.size()))-1]; // v[(v.size()-1)/4];
      }
      v=ascsort(mtran(vecteur(1,v)),true);
    }
    else
      v=ascsort(v,true);
    v=mtran(v);
    return v[int(std::ceil(d*v.size()))-1];
  }
  const char _quantile_s []="quantile";
  const unary_function_eval unary_quantile(&_quantile,_quantile_s);
  const unary_function_ptr at_quantile (&unary_quantile,0,true);

  gen _quartiles(const gen & g,GIAC_CONTEXT){
    vecteur v(gen2vecteur(g));
    if (g.type==_VECT && g.subtype==_SEQ__VECT && v.size()==2)
      return makevecteur(freq_quantile(v,0.0,contextptr),freq_quantile(v,0.25,contextptr),freq_quantile(v,0.5,contextptr),freq_quantile(v,0.75,contextptr),freq_quantile(v,1.0,contextptr));
    if (!ckmatrix(v)){
      if (!is_fully_numeric(evalf(v,1,contextptr))){
	sort(v.begin(),v.end(),islesscomplexthanf);
	int s=v.size();
	return makevecteur(v[0],v[int(std::ceil(s/4.))-1],v[int(std::ceil(s/2.))-1],v[int(std::ceil(3*s/4.))-1],v[s-1]);
      }
      v=ascsort(mtran(vecteur(1,v)),true);
    }
    else
      v=ascsort(v,true);
    v=mtran(v);
    int s=v.size();
    if (s==0)
      setsizeerr();
    return makevecteur(v[0],v[int(std::ceil(s/4.))-1],v[int(std::ceil(s/2.))-1],v[int(std::ceil(3*s/4.))-1],v[s-1]);
  }
  const char _quartiles_s []="quartiles";
  const unary_function_eval unary_quartiles(&_quartiles,_quartiles_s);
  const unary_function_ptr at_quartiles (&unary_quartiles,0,true);

  gen _moustache(const gen & g_orig,GIAC_CONTEXT){
    vecteur attributs(1,default_color(contextptr));
    gen g(g_orig);
    bool horizontal=true;
    double ymin=global_window_ymin;
    double ymax=global_window_ymax;
    if (g.type==_VECT && g.subtype==_SEQ__VECT){
      vecteur v(*g._VECTptr);
      int s=read_attributs(v,attributs,contextptr);
      if (s>1){
	gen tmp=v[s-1];
	if (tmp.is_symb_of_sommet(at_equal)){
	  if (tmp._SYMBptr->feuille[0]==x__IDNT_e)
	    horizontal=false;
	  tmp=tmp._SYMBptr->feuille[1];
	}
	if (tmp.is_symb_of_sommet(at_interval)){
	  ymin=evalf_double(tmp._SYMBptr->feuille[0],1,contextptr)._DOUBLE_val;
	  ymax=evalf_double(tmp._SYMBptr->feuille[1],1,contextptr)._DOUBLE_val;
	  --s;
	}
      }
      if (s==1)
	g=v.front();
      else
	g=gen(vecteur(v.begin(),v.begin()+s),g_orig.subtype);
    }
    vecteur v0(*_quartiles(g,contextptr)._VECTptr),v;
    if (!ckmatrix(v0))
      v=vecteur(1,v0);
    else
      v=mtran(v0);
    // _Pictsize(0);
    int s=v.size();
    vecteur res;
    double xmin=gnuplot_xmin,xmax=gnuplot_xmax;
    gen tmpx=_min(v0[0],contextptr),tmpxx=_max(v0[4],contextptr);
    if (tmpx.type==_DOUBLE_)
      xmin=tmpx._DOUBLE_val;
    if (tmpxx.type==_DOUBLE_)
      xmax=tmpxx._DOUBLE_val;
    vecteur attr(attributs);
    vecteur legendes(1,string2gen("",false));
    if (attributs.size()>=2)
      legendes=gen2vecteur(attributs[1]);
    else
      attr.push_back(legendes);
    int ls=legendes.size();
    vecteur affichages(gen2vecteur(attributs[0]));
    int as=affichages.size();
    if (horizontal){
      double y_scale=(ymax-ymin)/(4*s);
      for (int i=0;i<s;++i){
	attr[0]=(i<as?affichages[i]:affichages[0]);
	attr[1]=(i<ls?legendes[i]:legendes[0]);
	double y_up=ymax-(4*i+1)*y_scale;
	double y_middle=ymax-(4*i+2)*y_scale;
	double y_down=ymax-(4*i+3)*y_scale;
	vecteur current=gen2vecteur(v[i]);
	if (current.size()!=5)
	  continue;
	// trait min -> 1er quartile
	res.push_back(symb_segment(current[0]+y_middle*cst_i,current[1]+y_middle*cst_i,attr,_GROUP__VECT,contextptr));
	// rectangle 
	res.push_back(pnt_attrib(gen(makevecteur(current[3]+y_down*cst_i,current[1]+y_down*cst_i,current[1]+y_up*cst_i,current[3]+y_up*cst_i,current[3]+y_down*cst_i),_GROUP__VECT),attr,contextptr));
	// mediane
	res.push_back(symb_segment(current[2]+y_down*cst_i,current[2]+y_up*cst_i,attr,_GROUP__VECT,contextptr));
	// trait 3eme quartile -> fin
	res.push_back(symb_segment(current[3]+y_middle*cst_i,current[4]+y_middle*cst_i,attr,_GROUP__VECT,contextptr));
      }
    } else { // vertical picture
      giac::swapdouble(xmin,ymin);
      giac::swapdouble(xmax,ymax);
      double x_scale=(xmax-xmin)/(4*s);
      for (int i=0;i<s;++i){
	attr[0]=(i<as?affichages[i]:affichages[0]);
	attr[1]=(i<ls?legendes[i]:legendes[0]);
	double x_up=xmax-(4*i+1)*x_scale;
	double x_middle=xmax-(4*i+2)*x_scale;
	double x_down=xmax-(4*i+3)*x_scale;
	vecteur current=gen2vecteur(v[i]);
	if (current.size()!=5)
	  continue;
	// trait min -> 1er quartile
	res.push_back(symb_segment(x_middle+current[0]*cst_i,current[1]*cst_i+x_middle,attr,_GROUP__VECT,contextptr));
	// rectangle
	res.push_back(pnt_attrib(gen(makevecteur(current[1]*cst_i+x_up,current[1]*cst_i+x_down,current[3]*cst_i+x_down,current[3]*cst_i+x_up,current[1]*cst_i+x_up),_GROUP__VECT),attr,contextptr));
	// mediane
	res.push_back(symb_segment(current[2]*cst_i+x_down,current[2]*cst_i+x_up,attr,_GROUP__VECT,contextptr));
	// trait 3eme quartile -> fin
	res.push_back(symb_segment(current[3]*cst_i+x_middle,current[4]*cst_i+x_middle,attr,_GROUP__VECT,contextptr));
      }
    }
    return gen(res,_SEQ__VECT);
  }
  const char _moustache_s []="moustache";
  const unary_function_eval unary_moustache(&_moustache,_moustache_s);
  const unary_function_ptr at_moustache (&unary_moustache,0,true);

  const char _boxwhisker_s []="boxwhisker";
  const unary_function_eval unary_boxwhisker(&_moustache,_boxwhisker_s);
  const unary_function_ptr at_boxwhisker (&unary_boxwhisker,0,true);


  gen stddevmean(const vecteur & v,int withstddev,int xcol,int freqcol,GIAC_CONTEXT){
    int sv=v.size();
    if (xcol>=sv || freqcol>=sv)
      setdimerr();
    if (v[xcol].type!=_VECT || v[freqcol].type!=_VECT)
      setsizeerr();
    vecteur v1(*v[xcol]._VECTptr),v2(*v[freqcol]._VECTptr);
    // if v1 is made of intervals replace by the center of these intervals
    iterateur it=v1.begin(),itend=v1.end();
    for (;it!=itend;++it){
      if (it->is_symb_of_sommet(at_interval)){
	gen & f=it->_SYMBptr->feuille;
	if (f.type==_VECT && f._VECTptr->size()==2)
	  *it=(f._VECTptr->front()+f._VECTptr->back())/2;
      }
    }
    if (ckmatrix(v1) ^ ckmatrix(v2))
      setsizeerr();
    int n=v1.size();
    if (unsigned(n)!=v2.size())
      setsizeerr();
    gen m,m2,s;
    for (int i=0;i<n;++i){
      s = s + v2[i];
      m = m + apply(v2[i],v1[i],prod);
      if (withstddev)
	m2 = m2 + apply(v2[i],apply(v1[i],v1[i],prod),prod);
    }
    m = apply(m,s,rdiv);
    if (withstddev){
      m2=m2-apply(s,apply(m,m,prod),prod);
      m2=apply(m2,s-(withstddev==2),rdiv);
      if (withstddev==3)
	return m2;
      return apply(m2,sqrt,contextptr);
    }
    else
      return m;
  }
  // withstddev=0 (mean), 1 (stddev divided by n), 2 (by n-1), 3 (variance)
  gen stddevmean(const gen & g,int withstddev,GIAC_CONTEXT){
    vecteur & v=*g._VECTptr;
    int s=v.size();
    if (s<2)
      setsizeerr();
    if (v[1].type!=_INT_)
      return stddevmean(v,withstddev,0,1,contextptr);
    if (v[0].type!=_VECT)
      setsizeerr();
    int xcol=v[1].val;
    int freqcol=xcol+1;
    if (s>2 && v[2].type==_INT_)
      freqcol=v[2].val;
    return stddevmean(mtran(*v[0]._VECTptr),withstddev,xcol,freqcol,contextptr);
  }

  gen _mean(const gen & g,GIAC_CONTEXT){
    if (g.type==_VECT && g.subtype==_SEQ__VECT)
      return stddevmean(g,0,contextptr);
    vecteur v(gen2vecteur(g));
    if (!ckmatrix(v))
      return mean(mtran(vecteur(1,v)),true)[0];
    else
      v=mean(v,true);
    return v;
  }
  const char _mean_s []="mean";
  const unary_function_eval __mean(&_mean,_mean_s);
  const unary_function_ptr at_mean (&__mean,0,true);

  const char _moyenne_s []="moyenne";
  const unary_function_eval __moyenne(&_mean,_moyenne_s);
  const unary_function_ptr at_moyenne (&__moyenne,0,true);

  gen _stdDev(const gen & g,GIAC_CONTEXT){
    if (g.type==_VECT && g.subtype==_SEQ__VECT)
      return stddevmean(g,2,contextptr);
    vecteur v(gen2vecteur(g));
    if (!ckmatrix(v))
      return stddev(mtran(vecteur(1,v)),true,2)[0];
    else
      v=stddev(v,true,2);
    return v;
  }
  const char _stdDev_s []="stdDev";
  const unary_function_eval __stdDev(&_stdDev,_stdDev_s);
  const unary_function_ptr at_stdDev (&__stdDev,0,true);

  const char _stddevp_s []="stddevp";
  const unary_function_eval __stddevp(&_stdDev,_stddevp_s);
  const unary_function_ptr at_stddevp (&__stddevp,0,true);

  const char _ecart_type_population_s []="ecart_type_population";
  const unary_function_eval __ecart_type_population(&_stdDev,_ecart_type_population_s);
  const unary_function_ptr at_ecart_type_population (&__ecart_type_population,0,true);

  gen _stddev(const gen & g,GIAC_CONTEXT){
    if (g.type==_VECT && g.subtype==_SEQ__VECT)
      return stddevmean(g,1,contextptr);
    vecteur v(gen2vecteur(g));
    if (!ckmatrix(v))
      return stddev(mtran(vecteur(1,v)),true,1)[0];
    else
      v=stddev(v,true,1);
    return v;
  }
  const char _stddev_s []="stddev";
  const unary_function_eval __stddev(&_stddev,_stddev_s);
  const unary_function_ptr at_stddev (&__stddev,0,true);

  const char _ecart_type_s []="ecart_type";
  const unary_function_eval __ecart_type(&_stddev,_ecart_type_s);
  const unary_function_ptr at_ecart_type (&__ecart_type,0,true);

  gen _variance(const gen & g,GIAC_CONTEXT){
    if (g.type==_VECT && g.subtype==_SEQ__VECT)
      return stddevmean(g,3,contextptr);
    vecteur v(gen2vecteur(g));
    if (!ckmatrix(v))
      return stddev(mtran(vecteur(1,v)),true,3)[0];
    else
      v=stddev(v,true,3);
    return v;
  }
  const char _variance_s []="variance";
  const unary_function_eval __variance(&_variance,_variance_s);
  const unary_function_ptr at_variance (&__variance,0,true);

  vecteur covariance_correlation(const gen & g,const gen & u1,const gen & u2,int xcol,int ycol,int freqcol,GIAC_CONTEXT){
    vecteur v(gen2vecteur(g));
    if (!ckmatrix(v) || v.empty() || v.front()._VECTptr->size()<2)
      return makevecteur(undef,undef);
    gen sigmax,sigmay,sigmaxy,sigmax2,sigmay2,tmpx,tmpy,n,freq;
    if (freqcol<-1){
      // g is interpreted as a double-entry table with 1st col = x-values
      // 1st line=y-values, line/col is a frequency for this value of x/y
      int r,c;
      mdims(v,r,c);
      if (r<2 || c<2)
	setdimerr();
      vecteur & vy=*v[0]._VECTptr;
      for (int i=1;i<r;++i){
	vecteur & w=*v[i]._VECTptr;
	gen & currentx=w[0];
	for (int j=1;j<c;++j){
	  gen & currenty=vy[j];
	  freq=w[j];
	  n=n+freq;
	  sigmax=sigmax+currentx*freq;
	  sigmax2=sigmax2+currentx*currentx*freq;
	  sigmay=sigmay+currenty*freq;
	  sigmay2=sigmay2+currenty*currenty*freq;
	  sigmaxy=sigmaxy+currentx*currenty*freq;
	}
      }
    }
    else {
      const_iterateur it=v.begin(),itend=v.end();
      int s=it->_VECTptr->size();
      if (xcol>=s || ycol>=s || freqcol >=s)
	setdimerr();
      for (;it!=itend;++it){
	vecteur & w=*it->_VECTptr;
	if (u1.type==_FUNC)
	  tmpx=u1(w[xcol],contextptr);
	else
	  tmpx=w[xcol];
	if (u2.type==_FUNC)
	  tmpy=u2(w[ycol],contextptr);
	else
	  tmpy=w[ycol];
	if (freqcol>=0)
	  freq=w[freqcol];
	else
	  freq=plus_one;
	n = n+freq;
	sigmax = sigmax + tmpx*freq;
	sigmax2 = sigmax2 + tmpx*tmpx*freq;
	sigmay = sigmay + tmpy*freq;
	sigmay2 = sigmay2 + tmpy*tmpy*freq;
	sigmaxy = sigmaxy + tmpx*tmpy*freq;
      }
    }
    gen covariance=(n*sigmaxy-sigmax*sigmay)/(n*n);
    gen correlation=(n*sigmaxy-sigmax*sigmay)/sqrt((n*sigmax2-sigmax*sigmax)*(n*sigmay2-sigmay*sigmay),contextptr);
    return makevecteur(covariance,correlation);
  }
  
  void find_xyfreq(const gen & g,gen & gv,int & xcol,int & ycol,int &freqcol,GIAC_CONTEXT){
    xcol=0;
    ycol=1;
    freqcol=-1;
    if (g.type==_VECT && g.subtype==_SEQ__VECT && !g._VECTptr->empty()){
      vecteur v=*g._VECTptr;
      if (v[0].type!=_VECT)
	setsizeerr();
      int s=v.size();
      if (s==3 && v[1].type==_VECT){
	if (!ckmatrix(v[2]))
	  v[2]=_diag(v[2],contextptr);
	int n,c;
	mdims(*v[2]._VECTptr,n,c);
	if (unsigned(n)==v[0]._VECTptr->size() && unsigned(c)==v[1]._VECTptr->size()){
	  vecteur v0(*v[1]._VECTptr);
	  v0.insert(v0.begin(),zero);
	  matrice m(mtran(*v[2]._VECTptr));
	  m.insert(m.begin(),v[0]);
	  m=mtran(m);
	  m.insert(m.begin(),v0);
	  gv=m;
	  freqcol=-2;
	  return;
	}
      }
      if (s>1) {
	if (v[1].type==_INT_){
	  xcol=v[1].val;
	  if (xcol<0)
	    freqcol=-2;
	}
	else {
	  if (!ckmatrix(v))
	    setsizeerr();
	  gv=mtran(v);
	  return;
	}
      }
      if (s>2 && v[2].type==_INT_)
	ycol=v[2].val;
      if (s>3 && v[3].type==_INT_)
	freqcol=v[3].val;
      gv=v[0];
    }
    else {
      if (!ckmatrix(g) || g._VECTptr->empty())
	setsizeerr();
      gv=g;
      if (g._VECTptr->front()._VECTptr->size()>2)
	freqcol=2;
      if (g._VECTptr->front()._VECTptr->front().type==_STRNG)
	freqcol=-2;
    }
  }
  gen _covariance_correlation(const gen & g,GIAC_CONTEXT){
    int xcol,ycol,freqcol;
    gen gv;
    find_xyfreq(g,gv,xcol,ycol,freqcol,contextptr);
    return covariance_correlation(gv,zero,zero,xcol,ycol,freqcol,contextptr);      
  }
  const char _covariance_correlation_s []="covariance_correlation";
  const unary_function_eval __covariance_correlation(&_covariance_correlation,_covariance_correlation_s);
  const unary_function_ptr at_covariance_correlation (&__covariance_correlation,0,true);

  gen _covariance(const gen & g,GIAC_CONTEXT){
    int xcol,ycol,freqcol;
    gen gv;
    find_xyfreq(g,gv,xcol,ycol,freqcol,contextptr);
    return covariance_correlation(gv,zero,zero,xcol,ycol,freqcol,contextptr)[0];
  }
  const char _covariance_s []="covariance";
  const unary_function_eval __covariance(&_covariance,_covariance_s);
  const unary_function_ptr at_covariance (&__covariance,0,true);

  gen _correlation(const gen & g,GIAC_CONTEXT){
    int xcol,ycol,freqcol;
    gen gv;
    find_xyfreq(g,gv,xcol,ycol,freqcol,contextptr);
    return covariance_correlation(gv,zero,zero,xcol,ycol,freqcol,contextptr)[1];
  }
  const char _correlation_s []="correlation";
  const unary_function_eval __correlation(&_correlation,_correlation_s);
  const unary_function_ptr at_correlation (&__correlation,0,true);

  gen _interval2center(const gen & g,GIAC_CONTEXT){
    if (g.type==_VECT)
      return apply(g,_interval2center,contextptr);
    if (g.is_symb_of_sommet(at_interval)){
      gen & tmp=g._SYMBptr->feuille;
      if (tmp.type!=_VECT || tmp._VECTptr->size()!=2)
	setsizeerr();
      vecteur & v=*tmp._VECTptr;
      return (v.front()+v.back())/2;
    }
    return g;
  }
  const char _interval2center_s []="interval2center";
  const unary_function_eval __interval2center(&_interval2center,_interval2center_s);
  const unary_function_ptr at_interval2center (&__interval2center,0,true);

  void function_regression(const gen & g,const gen & u1,const gen & u2,gen & a,gen &b,double & xmin,double & xmax,gen & correl2,GIAC_CONTEXT){
    gen gv,freq;
    int xcol,ycol,freqcol;
    xmin=1e300;
    xmax=-xmin;
    find_xyfreq(g,gv,xcol,ycol,freqcol,contextptr);
    if (!ckmatrix(gv))
      setsizeerr();
    vecteur & v = *gv._VECTptr;
    gen n;
    gen sigmax,sigmay,sigmaxy,sigmax2,sigmay2,tmpx,tmpy;
    if (freqcol<-1){
      int r,c;
      mdims(v,r,c);
      if (r<2 || c<2)
	setdimerr();
      vecteur & vy=*v[0]._VECTptr;
      gen currentx,currenty;
      for (int i=1;i<r;++i){
	vecteur & w=*v[i]._VECTptr;
	gen tmpg=evalf_double(w[0],1,contextptr);
	if (tmpg.type==_DOUBLE_){
	  double tmp=tmpg._DOUBLE_val;
	  if (tmp<xmin)
	    xmin=tmp;
	  if (tmp>xmax)
	    xmax=tmp;
	}
	if (u1.type==_FUNC)
	  currentx=u1(w[0],contextptr);
	else
	  currentx=w[0];
	for (int j=1;j<c;++j){
	  if (u2.type==_FUNC)
	    currenty=u2(vy[j],contextptr);
	  else
	    currenty=vy[j];
	  currenty=_interval2center(currenty,contextptr);
	  freq=w[j];
	  n=n+freq;
	  sigmax=sigmax+currentx*freq;
	  sigmax2=sigmax2+currentx*currentx*freq;
	  sigmay=sigmay+currenty*freq;
	  sigmay2=sigmay2+currenty*currenty*freq;
	  sigmaxy=sigmaxy+currentx*currenty*freq;
	}
      }
    }
    else {
      const_iterateur it=v.begin(),itend=v.end();
      for (;it!=itend;++it){
	vecteur & w=*it->_VECTptr;
	gen tmpg=evalf_double(w[xcol],1,contextptr);
	if (tmpg.type==_DOUBLE_){
	  double tmp=tmpg._DOUBLE_val;
	  if (tmp<xmin)
	    xmin=tmp;
	  if (tmp>xmax)
	    xmax=tmp;
	}
	if (u1.type==_FUNC)
	  tmpx=u1(w[xcol],contextptr);
	else
	  tmpx=w[xcol];
	tmpx=_interval2center(tmpx,contextptr);
	if (u2.type==_FUNC)
	  tmpy=u2(w[ycol],contextptr);
	else
	  tmpy=w[ycol];
	tmpy=_interval2center(tmpy,contextptr);
	if (freqcol<0)
	  freq=plus_one;
	else
	  freq=w[freqcol];
	sigmax = sigmax + freq*tmpx;
	sigmax2 = sigmax2 + freq*tmpx*tmpx;
	sigmay = sigmay + freq*tmpy;
	sigmay2 = sigmay2 + freq*tmpy*tmpy;
	sigmaxy = sigmaxy + freq*tmpx*tmpy;
	n = n + freq;
      }
    }
    gen tmp=(n*sigmaxy-sigmax*sigmay);
    a=tmp/(n*sigmax2-sigmax*sigmax);
    b=(sigmay-a*sigmax)/n;
    correl2=(tmp*tmp)/(n*sigmax2-sigmax*sigmax)/(n*sigmay2-sigmay*sigmay);
    cerr << sigmax << " "<< sigmay << " " << sigmaxy << " " << n << " " << sigmax2 << " " << sigmay2 << endl;
  }

  gen function_regression(const gen & g,const gen & u1,const gen & u2,GIAC_CONTEXT){
    gen a,b,correl2;
    double xmin,xmax;
    function_regression(g,u1,u2,a,b,xmin,xmax,correl2,contextptr);
    return gen(makevecteur(a,b),_SEQ__VECT);
  }

  gen _linear_regression(const gen & g,GIAC_CONTEXT){
    return function_regression(g,zero,zero,contextptr);
  }
  const char _linear_regression_s []="linear_regression";
  const unary_function_eval __linear_regression(&_linear_regression,_linear_regression_s);
  const unary_function_ptr at_linear_regression (&__linear_regression,0,true);


  gen _exponential_regression(const gen & g,GIAC_CONTEXT){
    return exp(function_regression(g,zero,at_ln,contextptr),contextptr);
  }
  const char _exponential_regression_s []="exponential_regression";
  const unary_function_eval __exponential_regression(&_exponential_regression,_exponential_regression_s);
  const unary_function_ptr at_exponential_regression (&__exponential_regression,0,true);

  gen _logarithmic_regression(const gen & g,GIAC_CONTEXT){
    return function_regression(g,at_ln,zero,contextptr);
  }
  const char _logarithmic_regression_s []="logarithmic_regression";
  const unary_function_eval __logarithmic_regression(&_logarithmic_regression,_logarithmic_regression_s);
  const unary_function_ptr at_logarithmic_regression (&__logarithmic_regression,0,true);

  gen _power_regression(const gen & g,GIAC_CONTEXT){
    gen res= function_regression(g,at_ln,at_ln,contextptr);
    if (res.type==_VECT && res._VECTptr->size()==2){
      vecteur v(*res._VECTptr);
      v[1]=exp(v[1],contextptr);
      return gen(v,_SEQ__VECT);
    }
    return res;
  }
  const char _power_regression_s []="power_regression";
  const unary_function_eval __power_regression(&_power_regression,_power_regression_s);
  const unary_function_ptr at_power_regression (&__power_regression,0,true);

  gen regression_plot_attributs(const gen & g,vecteur & attributs,bool & eq,bool & r,GIAC_CONTEXT){
    gen res=g;
    r=false; eq=false;
    if (g.type==_VECT && g.subtype==_SEQ__VECT){
      int n=read_attributs(*g._VECTptr,attributs,contextptr);
      vecteur v=vecteur(g._VECTptr->begin(),g._VECTptr->begin()+n);
      vecteur & w=*g._VECTptr;
      int ws=w.size();
      for (int i=0;i<ws;++i){
	if (w[i]==at_equation){
	  eq=true;
	  if (i<n){
	    v.erase(v.begin()+i);
	    --n;
	    --i;
	  }
	}
	if (w[i]==at_correlation){
	  r=true;
	  if (i<n){
	    v.erase(v.begin()+i);
	    --n;
	    --i;
	  }
	}
      }
      if (n==1)
	res=g._VECTptr->front();
      else
	res=gen(v,_SEQ__VECT);
    }
    else
      attributs=vecteur(1,default_color(contextptr));
    return res;
  }

  gen _linear_regression_plot(const gen & g,GIAC_CONTEXT){
    gen a,b,correl2;
    double xmin,xmax;
    vecteur attributs;
    bool eq,r;
    gen G=regression_plot_attributs(g,attributs,eq,r,contextptr);
    function_regression(G,zero,zero,a,b,xmin,xmax,correl2,contextptr);
    xmax += (xmax-xmin); 
    gen ad(evalf_double(a,1,contextptr)),bd(evalf_double(b,1,contextptr)),cd(evalf_double(correl2,1,contextptr));
    if (ad.type==_DOUBLE_ && bd.type==_DOUBLE_ && cd.type==_DOUBLE_){
      string eqs="y="+print_DOUBLE_(ad._DOUBLE_val,3)+"*x+"+print_DOUBLE_(bd._DOUBLE_val,3);
      string R2s=" , R2="+print_DOUBLE_(cd._DOUBLE_val,3);
      *logptr(contextptr) << eqs << R2s << endl;
      string s;
      if (eq)
	s += eqs;
      if (r)
	s += R2s;
      attributs.push_back(string2gen(s,false));
    }
    return put_attributs(_droite(makevecteur(b*cst_i,1+(b+a)*cst_i),contextptr),attributs,contextptr);
  }
  const char _linear_regression_plot_s []="linear_regression_plot";
  const unary_function_eval __linear_regression_plot(&_linear_regression_plot,_linear_regression_plot_s);
  const unary_function_ptr at_linear_regression_plot (&__linear_regression_plot,0,true);

  gen _exponential_regression_plot(const gen & g,GIAC_CONTEXT){
    gen a,b,correl2;
    double xmin,xmax;
    vecteur attributs;
    bool eq,r;
    gen G=regression_plot_attributs(g,attributs,eq,r,contextptr);
    function_regression(G,zero,at_ln,a,b,xmin,xmax,correl2,contextptr);
    gen ad(evalf_double(a,1,contextptr)),bd(evalf_double(b,1,contextptr)),cd(evalf_double(correl2,1,contextptr));
    if (ad.type==_DOUBLE_ && bd.type==_DOUBLE_ && cd.type==_DOUBLE_){
      string eqs="y=exp("+print_DOUBLE_(ad._DOUBLE_val,3)+"*x+"+print_DOUBLE_(bd._DOUBLE_val,3)+")";
      string R2s=" , R2="+print_DOUBLE_(cd._DOUBLE_val,3);
      *logptr(contextptr) << eqs << R2s << endl;
      string s;
      if (eq)
	s += eqs;
      if (r)
	s += R2s;
      attributs.push_back(string2gen(s,false));
    }
    return put_attributs(_plotfunc(gen(makevecteur(evalf(exp(b,contextptr),1,contextptr)*exp(a*vx_var,contextptr),symb_equal(vx_var,symb_interval(xmin,xmax))),_SEQ__VECT),contextptr),attributs,contextptr);
  }
  const char _exponential_regression_plot_s []="exponential_regression_plot";
  const unary_function_eval __exponential_regression_plot(&_exponential_regression_plot,_exponential_regression_plot_s);
  const unary_function_ptr at_exponential_regression_plot (&__exponential_regression_plot,0,true);

  gen _logarithmic_regression_plot(const gen & g,GIAC_CONTEXT){
    gen a,b,correl2;
    double xmin,xmax;
    vecteur attributs;
    bool eq,r;
    gen G=regression_plot_attributs(g,attributs,eq,r,contextptr);
    function_regression(G,at_ln,zero,a,b,xmin,xmax,correl2,contextptr);
    xmax += (xmax-xmin); 
    gen ad(evalf_double(a,1,contextptr)),bd(evalf_double(b,1,contextptr)),cd(evalf_double(correl2,1,contextptr));
    if (ad.type==_DOUBLE_ && bd.type==_DOUBLE_ && cd.type==_DOUBLE_){
      string eqs="y=ln("+print_DOUBLE_(ad._DOUBLE_val,3)+"*x+"+print_DOUBLE_(bd._DOUBLE_val,3)+")";
      string R2s=" , R2="+print_DOUBLE_(cd._DOUBLE_val,3);
      *logptr(contextptr) << eqs << R2s << endl;
      string s;
      if (eq)
	s += eqs;
      if (r)
	s += R2s;
      attributs.push_back(string2gen(s,false));
    }
    return put_attributs(_plotfunc(gen(makevecteur(a*ln(vx_var,contextptr)+b,symb_equal(vx_var,symb_interval(xmin,xmax))),_SEQ__VECT),contextptr),attributs,contextptr);
  }
  const char _logarithmic_regression_plot_s []="logarithmic_regression_plot";
  const unary_function_eval __logarithmic_regression_plot(&_logarithmic_regression_plot,_logarithmic_regression_plot_s);
  const unary_function_ptr at_logarithmic_regression_plot (&__logarithmic_regression_plot,0,true);

  gen _power_regression_plot(const gen & g,GIAC_CONTEXT){
    gen a,b,correl2;
    double xmin,xmax;
    vecteur attributs;
    bool eq,r;
    gen G=regression_plot_attributs(g,attributs,eq,r,contextptr);
    function_regression(G,at_ln,at_ln,a,b,xmin,xmax,correl2,contextptr);
    xmax += (xmax-xmin); 
    gen ad(evalf_double(a,1,contextptr)),bd(evalf_double(b,1,contextptr)),cd(evalf_double(correl2,1,contextptr));
    if (ad.type==_DOUBLE_ && bd.type==_DOUBLE_ && cd.type==_DOUBLE_){
      string eqs="y="+print_DOUBLE_(exp(bd,contextptr)._DOUBLE_val,3)+"*x^"+print_DOUBLE_(ad._DOUBLE_val,3);
      string R2s=" , R2="+print_DOUBLE_(cd._DOUBLE_val,3);
      *logptr(contextptr) << eqs << R2s << endl;
      string s;
      if (eq)
	s += eqs;
      if (r)
	s += R2s;
      attributs.push_back(string2gen(s,false));
    }
    return put_attributs(_plotfunc(gen(makevecteur(exp(b,contextptr)*pow(vx_var,a,contextptr),symb_equal(vx_var,symb_interval(xmin,xmax))),_SEQ__VECT),contextptr),attributs,contextptr);
  }
  const char _power_regression_plot_s []="power_regression_plot";
  const unary_function_eval __power_regression_plot(&_power_regression_plot,_power_regression_plot_s);
  const unary_function_ptr at_power_regression_plot (&__power_regression_plot,0,true);

  gen polynomial_regression(const gen & g,int d,const gen & u1, const gen & u2,double & xmin, double & xmax,GIAC_CONTEXT){
    xmin=1e300,xmax=-xmin;
    vecteur v(gen2vecteur(g));
    if (!ckmatrix(v) || v.empty() || v.front()._VECTptr->size()<2)
      return undef;
    // use first and second column
    const_iterateur it=v.begin(),itend=v.end();
    // int n(itend-it);
    gen sigmax,sigmay,sigmaxy,sigmax2,sigmay2,tmpx,tmpxd,tmpy;
    vecteur xmoment(2*d+1),xymoment(d+1);
    for (;it!=itend;++it){
      vecteur & w=*it->_VECTptr;
      if (u1.type==_FUNC)
	tmpx=u1(w.front(),contextptr);
      else
	tmpx=w.front();
      tmpxd=evalf_double(tmpx,1,contextptr);
      if (tmpxd.type==_DOUBLE_){
	double tmpxdd=tmpxd._DOUBLE_val;
	if (tmpxdd<xmin)
	  xmin=tmpxdd;
	if (tmpxdd>xmax)
	  xmax=tmpxdd;
      }
      if (u2.type==_FUNC)
	tmpy=u2(w.back(),contextptr);
      else
	tmpy=w.back();
      xmoment[0]=xmoment[0]+1;
      xymoment[0]=xymoment[0]+tmpy;
      for (int i=1;i<=2*d;++i)
	xmoment[i]=xmoment[i]+pow(tmpx,i);
      for (int i=1;i<=d;++i)
	xymoment[i]=xymoment[i]+pow(tmpx,i)*tmpy;      
    }
    // make linear system
    matrice mat;
    for (int i=0;i<=d;++i){
      vecteur tmp;
      for (int j=d;j>=0;--j){
	tmp.push_back(xmoment[i+j]);
      }
      mat.push_back(tmp);
    }
    // return multmatvecteur(minv(mat,contextptr),xymoment); 
    return linsolve(mat,xymoment,contextptr);
  }
  gen polynomial_regression(const gen & g,double & xmin,double & xmax,GIAC_CONTEXT){
    if (g.type==_VECT && g._VECTptr->size()==3){
      vecteur & v=*g._VECTptr;
      if (v[0].type==_VECT && v[1].type==_VECT && v[0]._VECTptr->size()==v[1]._VECTptr->size())
	return polynomial_regression(makevecteur(mtran(makevecteur(v[0],v[1])),v[2]),xmin,xmax,contextptr);
    }
    gen last=_floor(g._VECTptr->back(),contextptr);
    if (g.type!=_VECT || g._VECTptr->size()!=2 || last.type!=_INT_)
      setsizeerr();
    return polynomial_regression(g._VECTptr->front(),absint(last.val),zero,zero,xmin,xmax,contextptr);
  }
  gen _polynomial_regression(const gen & g,GIAC_CONTEXT){
    double xmin,xmax;
    return polynomial_regression(g,xmin,xmax,contextptr);
  }
  const char _polynomial_regression_s []="polynomial_regression";
  const unary_function_eval __polynomial_regression(&_polynomial_regression,_polynomial_regression_s);
  const unary_function_ptr at_polynomial_regression (&__polynomial_regression,0,true);
  gen _polynomial_regression_plot(const gen & g,GIAC_CONTEXT){
    double xmin,xmax;
    vecteur attributs;
    bool eq,r;
    gen G=regression_plot_attributs(g,attributs,eq,r,contextptr);
    gen res=polynomial_regression(G,xmin,xmax,contextptr);
    xmax += (xmax-xmin); 
    res=horner(res,vx_var);
    return put_attributs(_plotfunc(gen(makevecteur(res,symb_equal(vx_var,symb_interval(xmin,xmax))),_SEQ__VECT),contextptr),attributs,contextptr);
  }
  const char _polynomial_regression_plot_s []="polynomial_regression_plot";
  const unary_function_eval __polynomial_regression_plot(&_polynomial_regression_plot,_polynomial_regression_plot_s);
  const unary_function_ptr at_polynomial_regression_plot (&__polynomial_regression_plot,0,true);

  // logistic_regression
  // Qt=instant production at time t
  // Pt=cumulative production at time t
  // arg1=Qt_1...Qt_n, arg2=t1..tn or t1, arg3=Pt_1
  // or arg1=Qt_1...Qt_n, arg2=t1..tn or t1, n>=10, using Pt_1=Qt_1/(1-tau)
  // where tau is fitted from the first 5 records 
  gen logistic_regression(const gen & g,double & xmin,double & xmax,gen & r,GIAC_CONTEXT){
    if (g.type!=_VECT)
      setsizeerr();
    vecteur & v = *g._VECTptr;
    int s=v.size();
    if (s<2 || s>3)
      setdimerr();
    gen data=v[0];
    if (data.type!=_VECT)
      setsizeerr();
    vecteur & w=*data._VECTptr;
    int n=w.size();
    gen Pinit;
    if (s==2){
      if (n<20)
	setdimerr("Guessing initial production requires more than 20 samples");
      gen args=gen(makevecteur(makevecteur(0,1,2,3,4),ln(vecteur(w.begin(),w.begin()+5),contextptr)),_SEQ__VECT);
      gen res=_linear_regression(args,contextptr);
      if (res.type!=_VECT || res._VECTptr->size()!=2)
	settypeerr();
      gen tmp=_correlation(evalf_double(args,1,contextptr),contextptr);
      Pinit=w[0]/(exp(res._VECTptr->front(),contextptr)-1);
      *logptr(contextptr) << "Initial cumulative estimated to " << Pinit << endl <<"Correlation for 5 first years to estimate initial cumulative : " << tmp << endl;
    }
    else
      Pinit=v[2];
    gen time=v[1],tinit,tend;
    if (time.is_symb_of_sommet(at_interval)){
      gen tmp=time._SYMBptr->feuille;
      if (tmp.type!=_VECT || tmp._VECTptr->size()!=2)
	setsizeerr();
      tinit=tmp._VECTptr->front();
      tend=tmp._VECTptr->back();
    }
    else {
      tinit=time;
      tend=time+w.size()-1;
    }
    tinit=evalf_double(tinit,1,contextptr);
    tend=evalf_double(tend,1,contextptr);
    if (tinit.type!=_DOUBLE_ || tend.type!=_DOUBLE_)
      setsizeerr();
    xmin = tinit._DOUBLE_val;
    xmax = tend._DOUBLE_val;
    gen tscale=(tend+1-tinit)/n;
    // compute cumulated production
    vecteur cum(n),quot(n);
    cum[0]=Pinit+w[0];
    quot[0]=w[0]/cum[0];
    for (int i=1;i<n;++i){
      cum[i]=cum[i-1]+w[i];
      quot[i]=w[i]/cum[i];
    }
    // linear regression of quot vs cum
    gen args=gen(makevecteur(cum,quot),_SEQ__VECT);
    gen res=_linear_regression(args,contextptr);
    r=_correlation(args,contextptr);
    if (res.type!=_VECT || res._VECTptr->size()!=2)
      setdimerr();
    gen a=res._VECTptr->front(),b=res._VECTptr->back(),urr=-b/a;
    *logptr(contextptr) << "Pinstant=" << a << "*Pcumul+" << b << endl << "Correlation " << r << ", Estimated total P=" << urr << endl << "Returning estimated Pcumul, Pinstant, Ptotal, Pinstantmax, tmax, R"<< endl;
    // y'/y=a*y+b -> y=urr/[1+exp(-b*(t-t0))]
    // urr/y-1=exp(-b*(t-t0))
    // -> -b*(t-t0) = ln(urr/y-1)
    vecteur lnurr(n),t(n);
    for (int i=0;i<n;++i){
      lnurr[i]=ln(urr/cum[i]-1,contextptr);
      t[i]=tinit+i*tscale;
    }
    args=gen(makevecteur(t,lnurr),_SEQ__VECT);
    res=_linear_regression(args,contextptr);
    if (res.type!=_VECT || res._VECTptr->size()!=2)
      setdimerr();
    gen b2=res._VECTptr->front(),bt0=res._VECTptr->back();
    return makevecteur(urr/(1+exp(b2*vx_var+bt0,contextptr)),urr*b/2/(1+cosh(b2*vx_var+bt0,contextptr)),urr,urr*b/4,-bt0/b2,r);
  }

  gen _logistic_regression(const gen & g,GIAC_CONTEXT){
    double xmin,xmax;
    gen r;
    return logistic_regression(g,xmin,xmax,r,contextptr);
  }
  const char _logistic_regression_s []="logistic_regression";
  const unary_function_eval __logistic_regression(&_logistic_regression,_logistic_regression_s);
  const unary_function_ptr at_logistic_regression (&__logistic_regression,0,true);

  gen _logistic_regression_plot(const gen & g,GIAC_CONTEXT){
    double xmin,xmax;
    vecteur attributs;
    bool eq,r;
    gen rcorr;
    gen G=regression_plot_attributs(g,attributs,eq,r,contextptr);
    gen res=logistic_regression(G,xmin,xmax,rcorr,contextptr);
    if (r){
      rcorr=rcorr*rcorr;
      string s = "R2="+rcorr.print(contextptr);
      attributs.push_back(string2gen(s,false));
    }
    xmax += (xmax-xmin); 
    if (res.type!=_VECT || res._VECTptr->empty())
      setsizeerr();
    res=res[1];
    return put_attributs(_plotfunc(gen(makevecteur(res,symb_equal(vx_var,symb_interval(xmin,xmax))),_SEQ__VECT),contextptr),attributs,contextptr);
  }
  const char _logistic_regression_plot_s []="logistic_regression_plot";
  const unary_function_eval __logistic_regression_plot(&_logistic_regression_plot,_logistic_regression_plot_s);
  const unary_function_ptr at_logistic_regression_plot (&__logistic_regression_plot,0,true);

  gen gen_interpolate(const gen & g,int deg,GIAC_CONTEXT){
    // args = matrix with 2 rows (x,y), xmin, xmax, xstep -> matrix of [x,y]
    if (g.type!=_VECT)
      setsizeerr();
    vecteur & v =*g._VECTptr;
    int s=v.size();
    if (s<4)
      setsizeerr();
    gen m=evalf_double(v[0],1,contextptr),
      Xmin=evalf_double(v[1],1,contextptr),
      Xmax=evalf_double(v[2],1,contextptr),
      Xstep=evalf_double(v[3],1,contextptr);
    if (!ckmatrix(m) || m._VECTptr->size()!=2 || Xmin.type!=_DOUBLE_ || Xmax.type!=_DOUBLE_ || Xstep.type!=_DOUBLE_)
      setsizeerr();
    double xmin=Xmin._DOUBLE_val,xmax=Xmax._DOUBLE_val,xstep=std::abs(Xstep._DOUBLE_val);
    // sort x in m
    matrice M(mtran(*m._VECTptr)); // 2 cols
    sort(M.begin(),M.end(),islesscomplexthanf);
    reverse(M.begin(),M.end());
    int Ms=M.size();
    if (Ms<2)
      setdimerr();
    gen X0=M[0]._VECTptr->front(),X1=M[Ms-1]._VECTptr->front();
    if (X0.type!=_DOUBLE_ || X1.type!=_DOUBLE_)
      setsizeerr();
    double x0=X0._DOUBLE_val,x1=X1._DOUBLE_val;
    if (xmin<x0 || xmax>x1)
      setsizeerr("Values out of range");
    matrice res;
    int pos=0; 
    gen Mcur=Xmin,Mnext=M[1]._VECTptr->front(),
      Ycur=M[0]._VECTptr->back(),Ynext=M[1]._VECTptr->back();
    double ycur;
    if (deg==1){
      for (double xcur=xmin;xcur<=xmax;xcur+=xstep){
	// find interval containing xcur in matrix
	for (;;){
	  if (Mnext._DOUBLE_val>xcur)
	    break;
	  ++pos;
	  if (pos==Ms)
	    break;
	  Mcur=M[pos]._VECTptr->front();
	  Ycur=M[pos]._VECTptr->back();
	  if (pos!=Ms-1){
	    Mnext = M[pos+1]._VECTptr->front();
	    Ynext = M[pos+1]._VECTptr->back();
	  }
	}
	if (pos>=Ms-1){ // use Ycur
	  res.push_back(makevecteur(xcur,Ycur));
	}
	else {
	  ycur = Ycur._DOUBLE_val+(xcur-Mcur._DOUBLE_val)/(Mnext._DOUBLE_val-Mcur._DOUBLE_val)*(Ynext._DOUBLE_val-Ycur._DOUBLE_val);
	  res.push_back(makevecteur(xcur,ycur));
	}
      }
    }
    else {
      vecteur current(deg+1); // contains the current Taylor expansion
      current[deg]=Ycur;
      // find z=current[0]: z*(Mnext-Mcur)^deg+Ycur=Ynext
      current[0]=(Ynext-Ycur)/pow(Mnext-Mcur,deg);
      for (double xcur=xmin;xcur<=xmax;xcur+=xstep){
	if (xcur>Mnext._DOUBLE_val){ // translate current, modify current[0]
	  current=taylor(current,Mnext-Mcur);
	  current[0]=0;
	  Ycur=Ynext;
	  Mcur=Mnext;
	  ++pos;
	  if (pos<Ms-1){
	    Mnext = M[pos+1]._VECTptr->front();
	    Ynext = M[pos+1]._VECTptr->back();
	    current[0]=(Ynext-horner(current,Mnext-Mcur))/pow(Mnext-Mcur,deg);
	  }
	}
	ycur=horner(current,xcur-Mcur)._DOUBLE_val;
	res.push_back(makevecteur(xcur,ycur));
      }
    }
    return mtran(res);
  }
  gen _linear_interpolate(const gen & g,GIAC_CONTEXT){
    return gen_interpolate(g,1,contextptr);
  }
  const char _linear_interpolate_s []="linear_interpolate";
  const unary_function_eval __linear_interpolate(&_linear_interpolate,_linear_interpolate_s);
  const unary_function_ptr at_linear_interpolate (&__linear_interpolate,0,true);

  gen _parabolic_interpolate(const gen & g,GIAC_CONTEXT){
    return gen_interpolate(g,2,contextptr);
  }
  const char _parabolic_interpolate_s []="parabolic_interpolate";
  const unary_function_eval __parabolic_interpolate(&_parabolic_interpolate,_parabolic_interpolate_s);
  const unary_function_ptr at_parabolic_interpolate (&__parabolic_interpolate,0,true);

  vector<double> prepare_effectifs(const vecteur & v,GIAC_CONTEXT){
    if (v.empty())
      setdimerr();
    vecteur w;
    if (ckmatrix(v)){
      int s=v.front()._VECTptr->size();
      if (s==1)
	w=*evalf_double(mtran(v)[0],1,contextptr)._VECTptr;
      else
	setdimerr();
    }
    else
      w=*evalf_double(v,1,contextptr)._VECTptr;
    // vector will be sorted keeping only DOUBLE data
    int s=w.size();
    vector<double> w1;
    w1.reserve(s);
    for (int i=0;i<s;++i){
      if (w[i].type==_DOUBLE_)
	w1.push_back(w[i]._DOUBLE_val);
    }
    sort(w1.begin(),w1.end());
    s=w1.size();
    if (!s)
      setdimerr();
    return w1;
  }
  matrice effectifs(const vecteur & data,double class_minimum,double class_size,GIAC_CONTEXT){
    if (data.empty())
      return data;
    vector<double>  w1;
    if (ckmatrix(data)){
      if (!data.empty() && data.front()._VECTptr->size()>1){
	matrice tmp=data;
	sort(tmp.begin(),tmp.end(),first_ascend_sort);
	tmp=mtran(tmp);
	vecteur tmpval=*evalf_double(tmp[0],1,contextptr)._VECTptr;
	vecteur tmpeff=*tmp[1]._VECTptr;
	if (tmpval.front().type!=_DOUBLE_ || tmpval.back().type!=_DOUBLE_)
	  setsizeerr();
	double kbegin=floor((tmpval.front()._DOUBLE_val-class_minimum)/class_size);
	double kend=floor((tmpval.back()._DOUBLE_val-class_minimum)/class_size);
	int s=tmpval.size(),i=0;
	vecteur res;
	for (;kbegin<=kend;++kbegin){
	  // count in this class
	  double min_class=kbegin*class_size+class_minimum;
	  double max_class=min_class+class_size;
	  gen effectif;
	  for (;i<s;effectif=effectif+tmpeff[i],++i){
	    if (tmpval[i].type!=_DOUBLE_)
	      setsizeerr();
	    if (tmpval[i]._DOUBLE_val>=max_class)
	      break;
	  }
	  res.push_back(makevecteur(symbolic(at_interval,makevecteur(min_class,max_class)),effectif));
	}
	return res;
      }
      w1=prepare_effectifs(*mtran(data)[0]._VECTptr,contextptr);
    }
    else
      w1=prepare_effectifs(data,contextptr);
    // class_min + k*class_size <= mini hence k
    double kbegin=floor((w1.front()-class_minimum)/class_size);
    double kend=floor((w1.back()-class_minimum)/class_size);
    vector<double>::const_iterator it=w1.begin(),itend=w1.end();
    vecteur res;
    for (;kbegin<=kend;++kbegin){
      // count in this class
      double min_class=kbegin*class_size+class_minimum;
      double max_class=min_class+class_size;
      int effectif=0;
      for (;it!=itend;++it,++effectif){
	if (*it>=max_class)
	  break;
      }
      res.push_back(makevecteur(symbolic(at_interval,makevecteur(min_class,max_class)),effectif));
    }
    return res;
  }

  matrice effectifs(const vecteur & data,const vecteur & intervalles,GIAC_CONTEXT){
    int s=intervalles.size();
    matrice res(s);
    vector<double> sorted_data;
    if (ckmatrix(data))
      sorted_data=prepare_effectifs(*mtran(data)[0]._VECTptr,contextptr);
    else
      sorted_data=prepare_effectifs(data,contextptr);
    vector<double>::const_iterator it=sorted_data.begin(),itend=sorted_data.end();
    for (int i=0;i<s;++i){
      gen cur_intervalle=intervalles[i];
      double debut,fin;
      if (!chk_double_interval(cur_intervalle,debut,fin,contextptr))
	setsizeerr();
      for (;it!=itend;++it){
	if (*it>=debut)
	  break;
      }
      int effectif=0;
      for (;it!=itend;++it,++effectif){
	if (*it>=fin)
	  break;
      }
      res[i]=makevecteur(cur_intervalle,effectif);
    }
    return res;
  }

  vecteur centres2intervalles(const vecteur & centres,double class_min,bool with_class_min,GIAC_CONTEXT){
    if (centres.size()<2)
      setsizeerr();
    double d0=evalf_double(centres[0],1,contextptr)._DOUBLE_val,d1=evalf_double(centres[1],1,contextptr)._DOUBLE_val;
    double debut=class_min;
    if (!with_class_min)
      debut=d0+(d0-d1)/2;
    vecteur res;
    const_iterateur it=centres.begin(),itend=centres.end();
    res.reserve(itend-it);
    for (;it!=itend;++it){
      gen g=evalf_double(*it,1,contextptr);
      if (g.type!=_DOUBLE_)
	setsizeerr();
      double milieu=g._DOUBLE_val;
      double fin=milieu+(milieu-debut);
      if (fin<=debut)
	setsizeerr();
      res.push_back(symb_interval(debut,fin));
      debut=fin;
    }
    return res;
  }

  gen _center2interval(const gen & g,GIAC_CONTEXT){
    if (g.type!=_VECT)
      settypeerr();
    if (g.subtype==_SEQ__VECT && g._VECTptr->size()==2){
      vecteur & v=*g._VECTptr;
      gen c=evalf_double(v[1],1,contextptr);
      if (v[0].type!=_VECT || c.type!=_DOUBLE_)
	settypeerr();
      return gen(centres2intervalles(*v[0]._VECTptr,c._DOUBLE_val,true,contextptr),_SEQ__VECT);
    }
    return gen(centres2intervalles(*g._VECTptr,0.0,false,contextptr),_SEQ__VECT);
  }
  const char _center2interval_s []="center2interval";
  const unary_function_eval __center2interval(&_center2interval,_center2interval_s);
  const unary_function_ptr at_center2interval (&__center2interval,0,true);


  gen histogram(const vecteur & v,double class_minimum,double class_size,const vecteur & attributs,GIAC_CONTEXT){
#ifndef WIN32
    bool old_iograph=io_graph(contextptr);
    io_graph(false,contextptr);
#endif
    if (ckmatrix(v) && !v.empty() && v.front()._VECTptr->size()==2){
      // matrix format is 2 columns 1st column=interval, 2nd column=frequency
      // OR value/frequencies
      // get total of population
      const_iterateur it=v.begin(),itend=v.end();
      double n=0;
      for (;it!=itend;++it)
	n += evalf_double(it->_VECTptr->back(),1,contextptr)._DOUBLE_val;
      // get surface
      gen g=v.front()._VECTptr->front();
      if (g.is_symb_of_sommet(at_interval)){
	g=g._SYMBptr->feuille;
	if (g.type!=_VECT || g._VECTptr->size()!=2)
	  settypeerr();
	g=evalf_double(g._VECTptr->front(),1,contextptr);
      }
      else
	g=g-class_size/2;
      gen h=(itend-1)->_VECTptr->front();
      if (h.is_symb_of_sommet(at_interval)){
	h=h._SYMBptr->feuille;
	if (h.type!=_VECT || h._VECTptr->size()!=2)
	  settypeerr();
	h=evalf_double(h._VECTptr->back(),1,contextptr);
      }
      else
	h=h+class_size/2;
      if (g.type!=_DOUBLE_ || h.type!=_DOUBLE_ || g._DOUBLE_val>=h._DOUBLE_val)
	setsizeerr();
      double inf,sup; // delta=h._DOUBLE_val-g._DOUBLE_val;
      it=v.begin();
      //  int nclass=itend-it;
      vecteur res;
      for (;it!=itend;++it){
	gen current=it->_VECTptr->front();
	if (current.is_symb_of_sommet(at_interval)){
	  if (!chk_double_interval(current,inf,sup,contextptr))
	    settypeerr();
	}
	else {
	  gen tmp=evalf_double(current,1,contextptr);
	  if (tmp.type!=_DOUBLE_)
	    settypeerr();
	  inf = tmp._DOUBLE_val -class_size/2;
	  sup = tmp._DOUBLE_val + class_size/2;
	}
	double height=1/(sup-inf);
	height=height*evalf_double(it->_VECTptr->back(),1,contextptr)._DOUBLE_val/n;
	gen mini(inf,height),maxi(sup,height);
	gen rectan(makevecteur(inf,mini,maxi,sup,inf),_LINE__VECT);
	res.push_back(pnt_attrib(rectan,attributs,contextptr));
	// res.push_back(_segment(makevecteur(inf,mini),contextptr));
	// res.push_back(_segment(makevecteur(mini,maxi),contextptr));
	// res.push_back(_segment(makevecteur(maxi,sup),contextptr));	    
      }
#ifndef WIN32
    io_graph(old_iograph,contextptr);
#endif
      return res;
    }
    vector<double>  w1=prepare_effectifs(v,contextptr);
    int s=w1.size();
    // class_min + k*class_size <= mini hence k
    double kbegin=floor((w1.front()-class_minimum)/class_size);
    double kend=floor((w1.back()-class_minimum)/class_size);
    vector<double>::const_iterator it=w1.begin(),itend=w1.end();
    vecteur res;
    for (;kbegin<=kend;++kbegin){
      // count in this class
      double min_class=kbegin*class_size+class_minimum;
      double max_class=min_class+class_size;
      double effectif=0;
      for (;it!=itend;++it,++effectif){
	if (*it>=max_class)
	  break;
      }
      effectif /= s*class_size; // height of the class
      gen ming=min_class+gen(0.0,effectif);
      gen maxg=max_class+gen(0.0,effectif);
      gen rectan(makevecteur(min_class,ming,maxg,max_class,min_class),_LINE__VECT);
      res.push_back(pnt_attrib(rectan,attributs,contextptr));
      // res.push_back(_segment(makevecteur(min_class,ming),contextptr));
      // res.push_back(_segment(makevecteur(ming,maxg),contextptr));
      // res.push_back(_segment(makevecteur(maxg,max_class),contextptr));
    }
#ifndef WIN32
    io_graph(old_iograph,contextptr);
#endif
    return gen(res,_SEQ__VECT);
  }
  gen _histogram(const gen & g,GIAC_CONTEXT){
    if (g.type!=_VECT)
      setsizeerr();
    vecteur args;
    if (g.subtype==_SEQ__VECT)
      args=*g._VECTptr;
    vecteur attributs(1,default_color(contextptr));
    int s=read_attributs(args,attributs,contextptr);
    if (s>=2){
      if (args[0].type!=_VECT)
	setsizeerr();
      vecteur data=*args[0]._VECTptr;
      gen arg1=evalf_double(args[1],1,contextptr);
      if (ckmatrix(data)&&arg1.type==_DOUBLE_){ // [ [center, effectif] ... ], min
	data=mtran(data); // 1st line = all centers
	if (data.size()!=2)
	  setsizeerr();
	data[0]=centres2intervalles(*data[0]._VECTptr,arg1._DOUBLE_val,true,contextptr);
	data=mtran(data);
	return histogram(data,0.0,0.0,attributs,contextptr);
      }
      if (s==3){
	gen arg2=evalf_double(args[2],1,contextptr);
	if (arg1.type==_DOUBLE_ && arg2.type==_DOUBLE_)
	  return histogram(data,arg1._DOUBLE_val,arg2._DOUBLE_val,attributs,contextptr);
      }
      if (s==2 && args[1].type==_VECT)
	return _histogram(gen(makevecteur(mtran(args),class_minimum),_SEQ__VECT),contextptr);
      setsizeerr();
    }
    if (s==1 && args.front().type==_VECT)
      args=*args.front()._VECTptr;
    else
      args=gen2vecteur(g);
    return histogram(args,class_minimum,class_size,attributs,contextptr);
  }
  const char _histogram_s []="histogram";
  const unary_function_eval __histogram(&_histogram,_histogram_s);
  const unary_function_ptr at_histogram (&__histogram,0,true);

  struct xeff {
    double x;
    double eff;
    xeff(double x0,double eff0): x(x0),eff(eff0) {}
  };

  bool operator <(const xeff & a,const xeff & b){
    return a.x<b.x;
  }

  gen _cumulated_frequencies(const gen & g,GIAC_CONTEXT){
    gen g0(g);
    double class_min=class_minimum,class_s=class_size;
    if (g0.type==_VECT && g0.subtype==_SEQ__VECT && g0._VECTptr->size()==2){
      vecteur v = *g._VECTptr;
      gen tmp=evalf_double(v[1],1,contextptr);
      if (tmp.type!=_DOUBLE_) {
	if (ckmatrix(g)){
	  if (!v[0]._VECTptr->front().is_symb_of_sommet(at_interval))
	    v[0]=centres2intervalles(*v[0]._VECTptr,class_min,true,contextptr);
	  g0=mtran(v);
	}
      }
      else {
	g0=v[0];
	class_min=tmp._DOUBLE_val;
      }
    }
    if (!ckmatrix(g0))
      setsizeerr();
    // 1st column = values (or classes), 2nd column = effectif
    matrice m= *g0._VECTptr ;
    if (m.empty() || m[0]._VECTptr->size()<2)
      setsizeerr();
    int s=m[0]._VECTptr->size();
    vecteur ans;
    for (int k=1;k<s;++k){
      // compute total eff
      iterateur it=m.begin(),itend=m.end();
      vector<xeff> veff;
      double n=0,x=0;
      if (it !=itend && it->_VECTptr->front().is_symb_of_sommet(at_interval)){
	gen tmp=it->_VECTptr->front()._SYMBptr->feuille;
	if (tmp.type!=_VECT || tmp._VECTptr->size()!=2)
	  setsizeerr();
	else
	  tmp=tmp._VECTptr->front();
	tmp=evalf_double(tmp,1,contextptr);
	if (tmp.type!=_DOUBLE_)
	  setsizeerr();
	veff.push_back(xeff(tmp._DOUBLE_val,0));
      }
      else
	veff.push_back(xeff(class_min,0));
      for (;it!=itend;++it){
	vecteur & v = *it->_VECTptr;
	gen tmp=evalf_double(v[k],1,contextptr);
	if (tmp.type!=_DOUBLE_)
	  setsizeerr();
	// class_s = tmp._DOUBLE_val - x;
	n = n + (x=tmp._DOUBLE_val) ;
	if (v.front().is_symb_of_sommet(at_interval)){
	  tmp=v.front()._SYMBptr->feuille;
	  if (tmp.type!=_VECT || tmp._VECTptr->size()!=2)
	    setsizeerr();
	  else
	    tmp=tmp._VECTptr->back();
	}
	else
	  tmp=v.front()+class_s/2;
	tmp=evalf_double(tmp,1,contextptr);
	if (tmp.type!=_DOUBLE_)
	  setsizeerr();
	veff.push_back(xeff(tmp._DOUBLE_val,x));
      }
      sort(veff.begin(),veff.end());
      vecteur res;
      vector<xeff>::const_iterator jt=veff.begin(),jtend=veff.end();
      double cumul=0;
      for (;jt!=jtend;++jt){
	cumul += jt->eff/n ;
	res.push_back(gen(jt->x)+cst_i*gen(cumul));
      }
      ans.push_back(symb_pnt(gen(res,_GROUP__VECT),k,contextptr));
    }
    return gen(ans,_SEQ__VECT);
  }
  const char _cumulated_frequencies_s []="cumulated_frequencies";
  const unary_function_eval __cumulated_frequencies(&_cumulated_frequencies,_cumulated_frequencies_s);
  const unary_function_ptr at_cumulated_frequencies (&__cumulated_frequencies,0,true);

  // classes(vector or column matrix,begin of class, class size)
  // "      ( "                     ,list of intervals)
  // "      ( "                     ,list of centers,begin of 1st class)
  gen _classes(const gen & g,GIAC_CONTEXT){
    if (g.type==_VECT && g.subtype==_SEQ__VECT){
      vecteur & args=*g._VECTptr;
      int s=args.size();
      if (s<2)
	setsizeerr();
      if (args[0].type!=_VECT)
	setsizeerr();
      vecteur data=*args[0]._VECTptr;
      if (s==2 && args[1].type==_VECT){ // 2nd arg=list of intervals
	return effectifs(data,*args[1]._VECTptr,contextptr);
      }
      if (s==3){
	gen arg2=evalf_double(args[2],1,contextptr);
	gen arg1=evalf_double(args[1],1,contextptr);
	if (args[2].type==_VECT && arg1.type==_DOUBLE_){
	  return effectifs(data,centres2intervalles(*args[2]._VECTptr,0.0,false,contextptr),contextptr);
	}
	if (arg1.type==_DOUBLE_ && arg2.type==_DOUBLE_)
	  return effectifs(data,arg1._DOUBLE_val,arg2._DOUBLE_val,contextptr);
      }
      setsizeerr();
    }
    vecteur v(gen2vecteur(g));
    return effectifs(v,class_minimum,class_size,contextptr);
  }
  const char _classes_s []="classes";
  const unary_function_eval __classes(&_classes,_classes_s);
  const unary_function_ptr at_classes (&__classes,0,true);

  vecteur listplot(const gen & g,vecteur & attributs,GIAC_CONTEXT){
    if (g.type!=_VECT || g._VECTptr->empty())
      setsizeerr();
    int s=read_attributs(*g._VECTptr,attributs,contextptr);
    vecteur v;
    if (s>=2 && g._VECTptr->front().type==_INT_ && g[1].type==_VECT){
      int l=g[1]._VECTptr->size();
      v=*g._VECTptr;
      v[0]=vecteur(l);
      for (int j=0;j<l;++j){
	(*v[0]._VECTptr)[j]=j+g._VECTptr->front().val;
      }
      if (!ckmatrix(v))
	setdimerr();
      v=mtran(v);
    }
    else {
      if (g._VECTptr->front().type==_VECT){
	vecteur & v0 = *g._VECTptr->front()._VECTptr;
	int v0s=v0.size();
	if (s==1)
	  v=v0;
	else {
	  if (v0s==1 && ckmatrix(g))
	    v=*mtran(*g._VECTptr).front()._VECTptr;
	  else
	    v=*g._VECTptr;
	}
      }
      else
	v=*g._VECTptr;
    }
    s=v.size();
    vecteur res;
    res.reserve(s);
    for (int i=0;i<s;++i){
      gen tmp=v[i];
      if (tmp.type==_VECT){
	if (tmp._VECTptr->size()==2)
	  res.push_back(tmp._VECTptr->front()+cst_i*tmp._VECTptr->back());
	else
	  setdimerr();
      }
      else
	res.push_back(i+(xcas_mode(contextptr)?1:0)+cst_i*tmp);
    }
    return res;
  }

  gen _listplot(const gen & g,GIAC_CONTEXT){
    vecteur attributs(1,default_color(contextptr));
    vecteur res=listplot(g,attributs,contextptr);
    if (attributs.size()>1)
      return symb_pnt_name(gen(res,_GROUP__VECT),attributs[0],attributs[1],contextptr);
    else
      return symb_pnt(gen(res,_GROUP__VECT),attributs[0],contextptr);
  }
  const char _listplot_s []="listplot";
  const unary_function_eval __listplot(&_listplot,_listplot_s);
  const unary_function_ptr at_listplot (&__listplot,0,true);
  const char _plotlist_s []="plotlist";
  const unary_function_eval __plotlist(&_listplot,_plotlist_s);
  const unary_function_ptr at_plotlist (&__plotlist,0,true);

  // [[x1 y1] [x2 y2] ...]
  gen scatterplot(const gen & g,bool polygone,bool scatter,GIAC_CONTEXT){
    vecteur v(gen2vecteur(g));
    vecteur attr(1,default_color(contextptr));
    int s=read_attributs(v,attr,contextptr);
    if (s==1 && ckmatrix(v.front()))
      v=*v.front()._VECTptr;
    else
      v=vecteur(v.begin(),v.begin()+s);
    if (g.type==_VECT && g.subtype==_SEQ__VECT && s==2){
      if (!ckmatrix(v))
	setsizeerr(); 
      v=mtran(v);
    }
    unsigned ncol=0;
    const gen & vf=v.front();
    if (vf.type!=_VECT){
      if (polygone)
	return _listplot(g,contextptr);
      vecteur attributs(1,default_color(contextptr));
      vecteur res=listplot(g,attributs,contextptr);
      int s=res.size();
      for (int i=0;i<s;++i){
	res[i]=symb_pnt(res[i],attributs[0],contextptr);
      }
      return gen(res,_SEQ__VECT);
    }
    if (!ckmatrix(v)||v.empty() || (ncol=vf._VECTptr->size())<2)
      setsizeerr();
    if (vf._VECTptr->front().type==_STRNG){
      if (attr.size()==1)
	attr.push_back(vecteur(vf._VECTptr->begin()+1,vf._VECTptr->end()));
      v.erase(v.begin());
    }
#ifndef WIN32
    bool old_iograph=io_graph(contextptr);
    io_graph(false,contextptr);
#endif
    const_iterateur it=v.begin(),itend=v.end();
    stable_sort(v.begin(),v.end(),first_ascend_sort);
    vecteur res;
    string nullstr;
    vecteur vres;
    for (unsigned j=1;j<ncol;++j){
      vecteur attributs(1,int(j<=FL_WHITE?j-1:j));
      attributs.push_back(string2gen("",false));
      if (!attr.empty()){ 
	if (ncol==2)
	  attributs[0]=attr[0];
	if (attr[0].type==_VECT && attr[0]._VECTptr->size()>=j)
	  attributs[0]=(*attr[0]._VECTptr)[j-1];
	if (attr.size()>1){
	  if (ncol==2)
	    attributs[1]=attr[1];
	  if (attr[1].type==_VECT && attr[1]._VECTptr->size()>=j)
	    attributs[1]=(*attr[1]._VECTptr)[j-1];
	}
      }
      res.clear();
      for (it=v.begin();it!=itend;++it){
	gen tmp=(*it->_VECTptr)[j];
	if (tmp.type==_STRNG && attributs[1].type==_STRNG && *attributs[1]._STRNGptr==nullstr)
	  attributs[1]=gen(*tmp._STRNGptr,contextptr);
	else {
	  if (tmp.is_symb_of_sommet(at_equal))
	    read_attributs(vecteur(1,tmp),attributs,contextptr);
	  else {
	    if (polygone)
	      res.push_back(it->_VECTptr->front()+cst_i*tmp);
	    if (scatter)
	      vres.push_back(symb_pnt_name(it->_VECTptr->front()+cst_i*tmp,attributs[0],string2gen(( (it==v.begin() && !polygone) ?gen2string(attributs[1]):""),false),contextptr));
	  }
	}
      }
      if (polygone)
	vres.push_back(symb_pnt_name(res,attributs[0],attributs[1],contextptr));
    }
#ifndef WIN32
    io_graph(old_iograph,contextptr);
#endif
    if (polygone && !scatter && ncol==2)
      return vres.front();
    return gen(vres,_SEQ__VECT);
  }
  gen _scatterplot(const gen & g,GIAC_CONTEXT){
    return scatterplot(g,false,true,contextptr);
  }
  const char _scatterplot_s []="scatterplot";
  const unary_function_eval __scatterplot(&_scatterplot,_scatterplot_s);
  const unary_function_ptr at_scatterplot (&__scatterplot,0,true);

  const char _nuage_points_s []="nuage_points";
  const unary_function_eval __nuage_points(&_scatterplot,_nuage_points_s);
  const unary_function_ptr at_nuage_points (&__nuage_points,0,true);

  gen _polygonplot(const gen & g,GIAC_CONTEXT){
    return scatterplot(g,true,false,contextptr);
  }
  const char _polygonplot_s []="polygonplot";
  const unary_function_eval __polygonplot(&_polygonplot,_polygonplot_s);
  const unary_function_ptr at_polygonplot (&__polygonplot,0,true);

  const char _ligne_polygonale_s []="ligne_polygonale";
  const unary_function_eval __ligne_polygonale(&_polygonplot,_ligne_polygonale_s);
  const unary_function_ptr at_ligne_polygonale (&__ligne_polygonale,0,true);

  gen _polygonscatterplot(const gen & g,GIAC_CONTEXT){
    return scatterplot(g,true,true,contextptr);
  }
  const char _polygonscatterplot_s []="polygonscatterplot";
  const unary_function_eval __polygonscatterplot(&_polygonscatterplot,_polygonscatterplot_s);
  const unary_function_ptr at_polygonscatterplot (&__polygonscatterplot,0,true);

  const char _ligne_polygonale_pointee_s []="ligne_polygonale_pointee";
  const unary_function_eval __ligne_polygonale_pointee(&_polygonscatterplot,_ligne_polygonale_pointee_s);
  const unary_function_ptr at_ligne_polygonale_pointee (&__ligne_polygonale_pointee,0,true);

  void read_camembert_args(const gen & g,vecteur & vals,vecteur & names,vecteur & attributs,GIAC_CONTEXT){
    if (g.type!=_VECT)
      setsizeerr();
    attributs=vecteur(1,default_color(contextptr) | _FILL_POLYGON);
    int s=read_attributs(*g._VECTptr,attributs,contextptr);
    gen args=(s==1)?g._VECTptr->front():g;
    if (ckmatrix(args)){
      matrice tmp(*args._VECTptr);
      if (tmp.empty())
	setdimerr();
      if (tmp.size()!=2)
	tmp=mtran(tmp);
      int ts=tmp.size();
      if (ts<2)
	setdimerr();
      if (ts>2){
	// draw a camembert for each line
	// [ list_of_class_names camembert1_values camembert2_values etc. ]
	// camembertj_values may begin with a title string
	names=*tmp.front()._VECTptr;
	if (names.size()<2)
	  setdimerr();
	if (names[1].type!=_STRNG)
	  setsizeerr();
	vals=vecteur(tmp.begin()+1,tmp.end());
	return;
      }
      vals=*tmp[1]._VECTptr;
      names=*tmp[0]._VECTptr;
      if (vals.front().type==_STRNG)
	std::swap(vals,names);
      vals=vecteur(1,vals);
      return;
    }
    if (args.type!=_VECT)
      setsizeerr();
    vals=*args._VECTptr;
    names=vecteur(vals.size(),string2gen("",false));
    vals=vecteur(1,vals);
  }

  // list of values or matrix with col1=list of legends, col2=list of values
  gen _diagramme_batons(const gen & g,GIAC_CONTEXT){
    vecteur vals,names,attributs,res;
    read_camembert_args(g,vals,names,attributs,contextptr);
    vecteur attr(gen2vecteur(attributs[0]));
    int ncamemberts=vals.size(),s=vals.front()._VECTptr->size(),t=attr.size();
    if (t==1)
      t=0;
    for (int j=0;j<ncamemberts;j++){
      vecteur & Vals = *vals[j]._VECTptr;
      int i=0;
      gen xy=s*j;
      if (Vals[0].type==_STRNG){
	// add title
	res.push_back(symb_pnt_name(xy+.5-2*cst_i,_POINT_INVISIBLE,Vals[0],contextptr));
	++i;
      }
      for (;i<s;++i){
	gen tmp(makevecteur(xy+i+.4+cst_i*Vals[i],xy+i+.4,xy+i-.4,xy+i-.4+cst_i*Vals[i],xy+i+.4+cst_i*Vals[i]),_LINE__VECT);
	res.push_back(symb_pnt_name(tmp,i<t?attr[i]:((i==7?0:i) | _FILL_POLYGON | _QUADRANT2),names[i],contextptr));
      }
    }
    return res;
  }
  const char _diagramme_batons_s []="bar_plot";
  const unary_function_eval __diagramme_batons(&_diagramme_batons,_diagramme_batons_s);
  const unary_function_ptr at_diagramme_batons (&__diagramme_batons,0,true);

  gen _camembert(const gen & g,GIAC_CONTEXT){
    vecteur vals,names,attributs,res;
    read_camembert_args(g,vals,names,attributs,contextptr);
    vecteur attr(gen2vecteur(attributs[0]));
    int ncamemberts=vals.size(),s=vals.front()._VECTptr->size(),t=attr.size();
    for (int j=0;j<ncamemberts;j++){
      gen xy=5*(j%4)-5*(j/4)*cst_i;
      gen diametre=makevecteur(-1+xy,1+xy);
      gen a(0),da;
      double da100;
      char ss[256];
      vecteur & Vals = *vals[j]._VECTptr;
      gen somme;
      int i=0,pos=0;;
      if (Vals[0].type==_STRNG){
	// add title
	res.push_back(symb_pnt_name(xy-1+2*cst_i,_POINT_INVISIBLE,Vals[0],contextptr));
	++i;
	somme=_plus(vecteur(Vals.begin()+1,Vals.end()),contextptr);
      }
      else
	somme=_plus(Vals,contextptr);
      for (;i<s;++i){
	if (ck_is_strictly_positive(-Vals[i],contextptr))
	  setsizeerr("Negative value encoutered");
	da=2*cst_pi*Vals[i]/somme;
	da100=evalf_double(100*Vals[i]/somme,1,contextptr)._DOUBLE_val;
	if (da100>0){
	  sprintf(ss,"%.4g",da100);
	  if (is_positive(a-cst_pi/2,contextptr))
	    pos=_QUADRANT2;
	  if (is_positive(a-cst_pi,contextptr))
	    pos=_QUADRANT3;
	  if (is_positive(a-3*cst_pi/2,contextptr))
	    pos=_QUADRANT4;
	  gen tmp=symbolic(at_cercle,gen(makevecteur(diametre,a,a+da),_PNT__VECT));
	  res.push_back(symb_pnt_name(tmp,i<t?attr[i]:(i%7 | _FILL_POLYGON | pos),string2gen(gen2string(names[i])+":"+string(ss)+"%",false),contextptr));
	  a=a+da;
	}
      }
    }
    return res;
  }
  const char _camembert_s []="camembert";
  const unary_function_eval __camembert(&_camembert,_camembert_s);
  const unary_function_ptr at_camembert (&__camembert,0,true);

  // Graham scan convex hull
 bool graham_sort_function(const gen & a,const gen & b){
   if (a.type!=_VECT || b.type!=_VECT || a._VECTptr->size()!=3 || b._VECTptr->size()!=3)
     setsizeerr();
   vecteur & v=*a._VECTptr;
   vecteur & w=*b._VECTptr;
   return is_strictly_greater(w[1],v[1],context0) || (v[1]==w[1] && is_strictly_greater(w[2],v[2],context0)) ;
 }

  gen cross_prod(const gen & a,const gen & b,const gen & c,GIAC_CONTEXT){
    gen ab=b-a,ac=c-a;
    gen A(re(ab,contextptr)),B(im(ab,contextptr)),C(re(ac,contextptr)),D(im(ac,contextptr));
    return A*D-B*C;
  }

  gen _convexhull(const gen & g,GIAC_CONTEXT){
    if (g.type!=_VECT)
      setsizeerr(); 
    vecteur l0(*_affixe(g,contextptr)._VECTptr),l;
    int s=l0.size();
    for (int i=0;i<s;++i){
      if (l0[i].type==_VECT)
	l=mergevecteur(l,*l0[i]._VECTptr);
      else
	l.push_back(l0[i]);
    }
    s=l.size();
    if (s<=3)
      return l;
    gen zmin=l[0],zcur;
    gen ymin=im(zmin,contextptr),ycur,xmin=re(zmin,contextptr),xcur;
    for (int j=1;j<s;++j){
      zcur=l[j]; ycur=im(zcur,contextptr); xcur=re(zcur,contextptr);
      if ( is_strictly_greater(ymin,ycur,contextptr) ||
	   (ycur==ymin && is_strictly_greater(xmin,xcur,contextptr)) ){
	zmin=zcur; ymin=ycur; xmin=xcur;
      }
    }
    vecteur ls;
    for (int j=0;j<s;++j){
      zcur=l[j];
      if (zcur!=zmin){
	ls.push_back(makevecteur(zcur,arg(zcur-zmin,contextptr),(zcur-zmin)*conj(zcur-zmin,contextptr)));
      }
    }
    sort(ls.begin(),ls.end(),graham_sort_function);
    vecteur res(makevecteur(zmin,ls[0][0]));
    int ress=2;
    gen o;
    for (int j=1;j<s-1;++j){
      zcur=ls[j][0];
      o=cross_prod(res[ress-2],res[ress-1],zcur,contextptr);
      if (is_zero(o))
	res[ress-1]=zcur;
      else {
	if (is_strictly_positive(o,contextptr)){
	  res.push_back(zcur);
	  ress++;
	}
	else {
	  while (!is_positive(o,contextptr) && ress>2){
	    res.pop_back();
	    ress--;
	    o=cross_prod(res[ress-2],res[ress-1],zcur,contextptr);
	  }
	  res.push_back(zcur);
	  ress++;
	}
      }
    }
    return gen(res,g.subtype);
  }
  const char _convexhull_s []="convexhull";
  const unary_function_eval __convexhull(&_convexhull,_convexhull_s);
  const unary_function_ptr at_convexhull (&__convexhull,0,true);

  // Simplex algorithm solving max c.x where constraints on x are in 
  // canonical form: A*x <= b with b>= 0
  // Variables are added to get [A|I] (x,x_slack) = b
  // 
  // Arguments:
  // m must contain an identity matrix in the n-1 first rows
  // like [A|I|b], to solve for b>=0, A*x<=b (I is for "slack" variables)
  // <variables d'ecarts ajoutees pour transformer <= en egalite >
  // last line (objective function row) [-c|0|0], maximize or minimize c.x
  // if the coefficients of the last row at the columns of the identity 
  // are not zero, step 0 will transform the last row to set them to 0.
  // max_pb is true for maximization and false for min
  // choose_first=true if we choose the first possible entering/outgoing index
  // 
  // Returns:
  // optimum will contain the max (min) value, if not +/-inf
  // bfs contains the coordinates of a solution
  // At the end of the algorithm we have [B^-1*A|B^-1|B^-1*b] for the
  // n-1 first row, and for last row [-c+c_B * B^-1*A | c_B*B^-1|c_B*B^-1*b]
  // where all coeffs of non-basic variables are + (for a max) and other are 0
  // Since the function to maximize + (last row) scalar (x,x_slack_variables)
  // = c_B*B^-1*b, it is not possible to improve c_B*B^-1*b
  // The reason is that the current solution has non-0 components
  // corresponding to the 0 value in the last row 
  // and 0 components corresponding to non-0 positive values in the last row
  // If we move one 0 component of the current solution, it must increase 
  // hence we have to decrease one of the coeff corresponding to non-0
  // positive coeffs in the last row, decreasing the value of the function
  // to maximize.
  // 
  // Not yet implemented: how to reduce any linear programming problem
  // to a feasible canonical matrix m
  // The idea is to add artificial variables (as many as there are equalities)
  // and maximize -sum(artificial variables) starting with all non
  // artificial variables equal to 0
  // If the max is not 0 there is no solution
  // otherwise all artificial variables are set to 0 and we have a
  // basic feasible solution to start with
  //
  // NB: If a coeff of the bfs is 0 we may cycle, using choose_first=true
  // will insure we do not cycle (Bland's rule)
  // another rule might be implemented by keeping somewhere all the
  // visited basis corresponding to the same max value
  matrice simplex_reduce(const matrice & m_orig,vecteur & bfs,gen & optimum,bool max_pb,bool choose_first,GIAC_CONTEXT){
    matrice m(m_orig);
    int nr=m.size();
    int nc=m.front()._VECTptr->size();
    if (nc<nr+1)
      setdimerr();
    // Step 0 set the coefficients of the last row in the idn column to 0
    vecteur lastline(nr-1);
    matrice mt(mtran(m)); 
    for (int i=0;i<nc-1;++i){
      int cur_col=-1;
      vecteur & mti=*mt[i]._VECTptr;
      for (int j=0;j<nr-1;++j){
	if (is_zero(mti[j]))
	  continue;
	if (!is_one(mti[j]) || cur_col>=0){
	  cur_col=-1;
	  break; // not an idn line
	}
	cur_col=j;	    
      }
      if (cur_col>=0)
	lastline[cur_col]=mti[nr-1];
    }
    for (int i=0;i<nr-1;++i){
      if (!is_zero(lastline[i]))
	m[nr-1]=subvecteur(*m[nr-1]._VECTptr,multvecteur(lastline[i],*m[i]._VECTptr));
    }
    for (;;){
      // Step 1: find the most positive (min_pb) or negative (max_pb) 
      // coefficient of the objective function row (last row)
      // or choose the first + or - if choose_first is true
      gen mincoeff=0;
      int J=-1;
      vecteur &last=*m.back()._VECTptr;
      for (int j=0;j<nc-1;++j){
	if (is_strictly_greater((max_pb?mincoeff:last[j]),
				(max_pb?last[j]:mincoeff),contextptr)){
	  J=j;
	  mincoeff=last[j];
	  if (choose_first)
	    break;
	}
      }
      if (J==-1){ // Find bfs and optimum
	optimum=m[nr-1][nc-1];
	bfs=vecteur(nc-1);
	// Push back 0 or m[i][nc-1] if column is an identity column
	matrice mt=mtran(m);
	for (int i=0;i<nc-1;++i){
	  int cur_col=-1;
	  if (is_zero(mt[i][nr-1])){
	    for (int j=0;j<nr-1;++j){
	      if (is_zero(mt[i][j]))
		continue;
	      if (cur_col>=0){
		cur_col=-1;
		break; // not an idn line
	      }
	      cur_col=j;	    
	    }
	  }
	  if (cur_col>=0)
	    bfs[i]=mt[nc-1][cur_col];
	}
	return m;
      }
      int I=-1;
      mincoeff=plus_inf;
      gen ratio;
      // We will move the J-th variable from 0 to something positive
      // We have to find which variables govern how much positive xJ can be
      // -> find the smallest positive ratio 
      // and choose the 1st one if the smallest ratio is reached several times
      for (int i=0;i<nr-1;++i){
	gen m1(m[i][J]);
	if (is_strictly_positive(m1,contextptr) && 
	    is_strictly_greater(mincoeff,ratio=m[i][nc-1]/m1,contextptr)){
	  I=i;
	  mincoeff=ratio;
	}
      }
      if (I==-1){ // The function is not bounded since xJ can grow to +inf
	optimum=max_pb?plus_inf:minus_inf;
	return m;
      }
      if (is_zero(mincoeff)) // Bland's rule
	choose_first=true;
      // Pivot found, line I, column J, reduce matrix (Gauss-like)
      m[I]=divvecteur(*m[I]._VECTptr,m[I][J]);
      vecteur & pivot_v = *m[I]._VECTptr;
      gen a;
      for (int i=0;i<nr;++i){
	if (i==I)
	  continue;
	vecteur & v=*m[i]._VECTptr;
	a=v[J];
	for (int j=0;j<nc;++j){
	  v[j]=v[j]-a*pivot_v[j];
	}
      }
    }
  }

  // solve max(c.x) under Ax<=b, returns optimum value, solution x0
  // and reduced matrix
  gen _simplex_reduce(const gen & g,GIAC_CONTEXT){
    matrice m;
    vecteur v; 
    gen optimum;
    if (g.type ==_VECT && g.subtype==_SEQ__VECT && g._VECTptr->size()==3){
      vecteur & gv =*g._VECTptr;
      if (gv[0].type!=_VECT || gv[1].type!=_VECT || gv[2].type!=_VECT)
	settypeerr();
      m=*gv[0]._VECTptr;
      int add=m.size();
      m=mtran(m);
      m=mergevecteur(m,midn(add));
      m.push_back(gv[1]);
      if (!ckmatrix(m))
	setdimerr();
      m=mtran(m);
      m.push_back(mergevecteur(*(-gv[2])._VECTptr,vecteur(add+1,0)));
      if (!ckmatrix(m))
	setdimerr();
    }
    else {
      if (!ckmatrix(g))
	setsizeerr(); 
      m=*g._VECTptr;
    }
    m=simplex_reduce(m,v,optimum,true,false,contextptr);
    return gen(makevecteur(optimum,v,m),_SEQ__VECT);
  }
  const char _simplex_reduce_s []="simplex_reduce";
  const unary_function_eval __simplex_reduce(&_simplex_reduce,_simplex_reduce_s);
  const unary_function_ptr at_simplex_reduce (&__simplex_reduce,0,true);

  // natural_spline([x0,...,xn],[y0,...,yn],x,d)
  // -> spline of degree d, in C^{d-1}, with values yk at xk
  // and initial/final derivatives = 0 from order 1 to (d-1)/2, 
  // d-1 conditions
  // returns a list of n polynomials with respect to x
  // to get the value of the spline, find the right interval hence polynomial
  // and call horner(poly,value-xi)
  // x and d are optionnal, if not precised d is 3
  gen _spline(const gen & g,GIAC_CONTEXT){
    if (g.type!=_VECT || g._VECTptr->size()<2)
      setsizeerr();
    vecteur w(*g._VECTptr);
    if (w.size()<3)
      w.push_back(vx_var);
    if (w.size()<4)
      w.push_back(3);
    gen X(w[0]), Y(w[1]), xvar(w[2]), dg(w[3]);
    if (dg.type!=_INT_ || dg.val<1 || X.type!=_VECT || Y.type!=_VECT)
      settypeerr();
    vecteur &x=*X._VECTptr;
    vecteur &y=*Y._VECTptr;
    int n=x.size()-1;
    if (n<1 || y.size()!=unsigned(n+1))
      setdimerr();
    int d(dg.val);
    // create n unknowns, the values of the highest derivative coeff
    // at x0, ..., xn-1
    // and (d-1)/2 unknowns, the values of diff(spline,x$k) for k=1 to (d-1)/2
    vecteur z(n),f((d-1)/2),pol;
    for (int i=0;i<n;++i){
      z[i]=identificateur(" z"+print_INT_(i));
    }
    for (int i=0;i<(d-1)/2;++i){
      f[i]=identificateur(" f"+print_INT_(i));
    }
    // create vector of linear equations to solve for
    vecteur lineq;
    // create initial taylor polynomial form of the poly
    vecteur v(d+1);
    v[0]=z[0]; // z[0]*(x-x0)^d + f_(d-1)/2 * (x-x0)^{(d-1)/2} + ...+f_0*(x-x_0) + y0
    for (int i=1;i<=(d-1)/2;++i)
      v[d-i]=f[i-1];
    v[d]=y[0]; // all conditions at x0 are solved
    pol.push_back(v);
    for (int i=0;i<n;++i){   
      // move from xi to xi+1
      v=*ratnormal(taylor(v,x[i+1]-x[i]))._VECTptr;
      lineq.push_back(v[d]-y[i+1]);
      // new v changes by the first coeff of v only
      v[0]=(i==n-1)?0:z[i+1];
      pol.push_back(v);
    }
    // add conditions at x[n]
    for (int i=1;i<=(d-1)/2;++i){
      lineq.push_back(v[i]);
    }
    vecteur inconnu(mergevecteur(z,f));
    vecteur zf=linsolve(lineq,inconnu,contextptr);
    pol.pop_back();
    pol=*ratnormal(subst(pol,inconnu,zf,false,contextptr))._VECTptr;
    for (int i=0;i<n;++i){
      if (pol[i].type==_VECT)
	pol[i]=symb_horner(*pol[i]._VECTptr,xvar-x[i]);
    }
    return pol;
  }
  const char _spline_s []="spline";
  const unary_function_eval __spline(&_spline,_spline_s);
  const unary_function_ptr at_spline (&__spline,0,true);

  extern const unary_function_ptr at_bitand;
  extern const unary_function_ptr at_bitor;
  extern const unary_function_ptr at_bitxor;
  extern const unary_function_ptr at_hamdist;
  gen giac_bitand(const gen & a,const gen & b){
    register unsigned t=(a.type<< _DECALAGE) | b.type;
    if (!t)
      return( a.val & b.val);
    register ref_mpz_t * e;
    switch ( t ) {
    case _ZINT__ZINT:
      e = new ref_mpz_t;
      mpz_and(e->z,*a._ZINTptr,*b._ZINTptr);
      return e;
    case _INT___ZINT: 
      e = new ref_mpz_t;
      mpz_set_ui(e->z,a.val);
      mpz_and(e->z,e->z,*b._ZINTptr);
      return(e);
    case _ZINT__INT_:
      e = new ref_mpz_t;
      mpz_set_ui(e->z,b.val);
      mpz_and(e->z,*a._ZINTptr,e->z);
      return(e);
    }
    return symbolic(at_bitand,gen(makevecteur(a,b),_SEQ__VECT));
  }
  gen giac_bitor(const gen & a,const gen & b){
    register unsigned t=(a.type<< _DECALAGE) | b.type;
    if (!t)
      return( a.val | b.val);
    register ref_mpz_t * e;
    switch ( t ) {
    case _ZINT__ZINT:
      e = new ref_mpz_t;
      mpz_ior(e->z,*a._ZINTptr,*b._ZINTptr);
      return(e);
    case _INT___ZINT: 
      e = new ref_mpz_t;
      mpz_set_ui(e->z,a.val);
      mpz_ior(e->z,e->z,*b._ZINTptr);
      return(e);
    case _ZINT__INT_:
      e = new ref_mpz_t;
      mpz_set_ui(e->z,b.val);
      mpz_ior(e->z,*a._ZINTptr,e->z);
      return(e);
    }
    return symbolic(at_bitor,gen(makevecteur(a,b),_SEQ__VECT));
  }
  gen giac_bitxor(const gen & a,const gen & b){
    register unsigned t=(a.type<< _DECALAGE) | b.type;
    if (!t)
      return( a.val ^ b.val);
    register ref_mpz_t * e;
    switch ( t ) {
    case _ZINT__ZINT:
      e = new ref_mpz_t;
      mpz_xor(e->z,*a._ZINTptr,*b._ZINTptr);
      return(e);
    case _INT___ZINT: 
      e = new ref_mpz_t;
      mpz_set_ui(e->z,a.val);
      mpz_xor(e->z,e->z,*b._ZINTptr);
      return(e);
    case _ZINT__INT_:
      e = new ref_mpz_t;
      mpz_set_ui(e->z,b.val);
      mpz_xor(e->z,*a._ZINTptr,e->z);
      return(e);
    }
    return symbolic(at_bitxor,gen(makevecteur(a,b),_SEQ__VECT));
  }

  gen giac_hamdist(const gen & a,const gen & b){
    unsigned long t=(a.type<< _DECALAGE) | b.type;
    ref_mpz_t *  e = new ref_mpz_t;
    switch ( t ) {
    case 0:
      mpz_set_ui(e->z,a.val ^ b.val);
      t = mpz_popcount(e->z);
      break;
    case _ZINT__ZINT:
      t=mpz_hamdist(*a._ZINTptr,*b._ZINTptr);
      break;
    case _INT___ZINT: 
      mpz_set_ui(e->z,a.val);
      t=mpz_hamdist(e->z,*b._ZINTptr);
      break;
    case _ZINT__INT_:
      mpz_set_ui(e->z,b.val);
      t=mpz_hamdist(*a._ZINTptr,e->z);
      break;
    default:
      delete e;
      return symbolic(at_hamdist,gen(makevecteur(a,b),_SEQ__VECT));
    }
    delete e;
    return longlong(t);
  }

  gen binop(const gen & g,gen (* f) (const gen &, const gen &)){
    if (g.type!=_VECT && g._VECTptr->empty())
      setsizeerr();
    const_iterateur it=g._VECTptr->begin(),itend=g._VECTptr->end();
    gen res=*it;
    for (++it;it!=itend;++it){
      res=apply(res,*it,f);
    }
    return res;
  }
  gen _bitand(const gen & g,GIAC_CONTEXT){
    return binop(g,giac_bitand);
  }
  const char _bitand_s []="bitand";
  const unary_function_eval __bitand(&_bitand,_bitand_s);
  const unary_function_ptr at_bitand (&__bitand,0,true);

  gen _bitor(const gen & g,GIAC_CONTEXT){
    return binop(g,giac_bitor);
  }
  const char _bitor_s []="bitor";
  const unary_function_eval __bitor(&_bitor,_bitor_s);
  const unary_function_ptr at_bitor (&__bitor,0,true);

  gen _bitxor(const gen & g,GIAC_CONTEXT){
    return binop(g,giac_bitxor);
  }
  const char _bitxor_s []="bitxor";
  const unary_function_eval __bitxor(&_bitxor,_bitxor_s);
  const unary_function_ptr at_bitxor (&__bitxor,0,true);

  gen _hamdist(const gen & g,GIAC_CONTEXT){
    return binop(g,giac_hamdist);
  }
  const char _hamdist_s []="hamdist";
  const unary_function_eval __hamdist(&_hamdist,_hamdist_s);
  const unary_function_ptr at_hamdist (&__hamdist,0,true);


  // ploarea(polygone), plotarea(f(x),x=a..b), plotarea(f(x),x=a..b,n,method)
  // method=trapeze,point_milieu,rectangle_gauche,rectangle_droit
  gen _plotarea(const gen & g,GIAC_CONTEXT){
    vecteur v(gen2vecteur(g));
    vecteur attributs(default_color(contextptr));
    int s=read_attributs(v,attributs,contextptr);
    if (!s)
      setsizeerr();
    if (attributs.size()<2)
      attributs.push_back(0);
    if (attributs[0].type==_INT_)
      attributs[0].val= attributs[0].val | _FILL_POLYGON;
    v[0]=remove_at_pnt(v[0]);
    if (v[0].type==_VECT){
      attributs[1]=_aire(v[0],contextptr);
      return pnt_attrib(v[0],attributs,contextptr);
    }
    if (s>=2 && v[0].type!=_VECT){
      gen tmp(v[1]),a,b,x(vx_var);
      if (tmp.is_symb_of_sommet(at_equal) && tmp._SYMBptr->feuille.type==_VECT && tmp._SYMBptr->feuille._VECTptr->size()==2){
	x=tmp._SYMBptr->feuille[0];
	tmp=tmp._SYMBptr->feuille[1];
      }
      if (tmp.is_symb_of_sommet(at_interval) && tmp._SYMBptr->feuille.type==_VECT && tmp._SYMBptr->feuille._VECTptr->size()==2){
	a=tmp._SYMBptr->feuille[0];
	b=tmp._SYMBptr->feuille[1];
      }
      else
	setsizeerr("plotarea(f(x),x=a..b[,n,method])");
      if (s>=2){
	gen graph=funcplotfunc(gen(vecteur(v.begin(),v.begin()+s),_SEQ__VECT),false,contextptr); // must be a graph of fcn
	// extract polygon
	gen graphe=remove_at_pnt(graph);
	if (graphe.is_symb_of_sommet(at_curve) && graphe._SYMBptr->feuille.type==_VECT){
	  vecteur & graphev=*graphe._SYMBptr->feuille._VECTptr;
	  if (graphev.size()>1){
	    gen polyg=graphev[1];
	    if (polyg.type==_VECT){
	      if (s==2){
		// add verticals and horizontal
		vecteur res(*polyg._VECTptr);
		res.insert(res.begin(),a);
		res.insert(res.begin(),b);
		res.push_back(b);
		int nd=decimal_digits(contextptr);
		decimal_digits(3,contextptr);
		attributs[1]=string2gen(_romberg(gen(makevecteur(v[0],v[1]),_SEQ__VECT),contextptr).print(contextptr),false);
		decimal_digits(nd,contextptr);
		return pnt_attrib(gen(res,_GROUP__VECT),attributs,contextptr);
	      } // end s==2
	      if (s>=3)
		v[2]=_floor(v[2],contextptr);
	      if (s>=3 && v[2].type==_INT_){
		int n=v[2].val;
		if (n<1)
		  setsizeerr();
		vecteur res;
		res.push_back(b);
		res.push_back(a);
		gen dx=(b-a)/n,x0=a,xf=x0,fxf,f=v[0],A;
		int method=_TRAPEZE;
		if (s>=4 && v[3].type==_INT_)
		  method = v[3].val;
		if (method==_RECTANGLE_DROIT || method==_RECTANGLE_GAUCHE || method==_POINT_MILIEU){
		  if (method==_RECTANGLE_DROIT)
		    xf=x0+dx;
		  if (method==_POINT_MILIEU)
		    xf=x0+dx/2;
		  for (int i=0;i<n;++i){
		    fxf=evalf(quotesubst(f,x,xf,contextptr),1,contextptr);
		    A=A+dx*fxf;
		    res.push_back(x0+fxf*cst_i);
		    x0=x0+dx;
		    xf=xf+dx;
		    res.push_back(x0+fxf*cst_i);
		  }
		}
		if (method==_TRAPEZE){
		  fxf=evalf(quotesubst(f,x,xf,contextptr),1,contextptr);
		  A=dx*fxf/2;
		  res.push_back(xf+fxf*cst_i);
		  xf=x0+dx;
		  for (int i=0;i<n-1;++i){
		    fxf=evalf(quotesubst(f,x,xf,contextptr),1,contextptr);
		    A=A+dx*fxf;
		    res.push_back(xf+fxf*cst_i);
		    x0=x0+dx;
		    xf=xf+dx;
		  }
		  fxf=evalf(quotesubst(f,x,b,contextptr),1,contextptr);
		  A=A+dx*fxf/2;
		  res.push_back(b+fxf*cst_i);
		}
		res.push_back(b);
		int nd=decimal_digits(contextptr);
		decimal_digits(3,contextptr);
		attributs[1]=string2gen(A.print(contextptr),false);
		decimal_digits(nd,contextptr);
		return gen(makevecteur(pnt_attrib(res,attributs,contextptr),graph),_SEQ__VECT);
	      } // end if (s>=3)
	    } // end polyg.type==_VECT
	  }
	}
      } // end s>=2
    }
    setsizeerr("");
    return undef;
  }
  const char _plotarea_s []="plotarea";
  const unary_function_eval __plotarea(&_plotarea,_plotarea_s);
  const unary_function_ptr at_plotarea (&__plotarea,0,true);

  const char _areaplot_s []="areaplot";
  const unary_function_eval __areaplot(&_plotarea,_areaplot_s);
  const unary_function_ptr at_areaplot (&__areaplot,0,true);

  gen _add_language(const gen & args,GIAC_CONTEXT){
    if (args.type==_INT_){
      add_language(args.val);
      return 1;
    }
    if (args.type==_STRNG){
      string s=*args._STRNGptr;
      s=s.substr(0,2);
      int i=string2lang(s);
      if (i){
	add_language(i);
	return 1;
      }
    }
    return 0;
  }
  const char _add_language_s []="add_language";
  const unary_function_eval __add_language(&_add_language,_add_language_s);
  const unary_function_ptr at_add_language (&__add_language,0,true);

  gen _remove_language(const gen & args,GIAC_CONTEXT){
    if (args.type==_INT_){
      remove_language(args.val);
      return 1;
    }
    if (args.type==_STRNG){
      string s=*args._STRNGptr;
      s=s.substr(0,2);
      int i=string2lang(s);
      if (i){
	remove_language(i);
	return 1;
      }
    }
    return 0;
  }
  const char _remove_language_s []="remove_language";
  const unary_function_eval __remove_language(&_remove_language,_remove_language_s);
  const unary_function_ptr at_remove_language (&__remove_language,0,true);

  gen _show_language(const gen & args,GIAC_CONTEXT){
    return vector_int_2_vecteur(lexer_localization_vector,contextptr);
  }
  const char _show_language_s []="show_language";
  const unary_function_eval __show_language(&_show_language,_show_language_s);
  const unary_function_ptr at_show_language (&__show_language,0,true);

  gen _os_version(const gen & args,GIAC_CONTEXT){
#ifdef WIN32
    return string2gen("win",false);
#else
#ifdef __APPLE__
    return string2gen("macos",false);
#else
    return string2gen("unix",false);
#endif
#endif
  }
  const char _os_version_s []="os_version";
  const unary_function_eval __os_version(&_os_version,_os_version_s);
  const unary_function_ptr at_os_version (&__os_version,0,true);

#ifndef NO_NAMESPACE_GIAC
} // namespace giac
#endif // ndef NO_NAMESPACE_GIAC
