// -*- mode:C++ ; compile-command: "g++-3.4 -I.. -g -c rpn.cc  -DIN_GIAC -DHAVE_CONFIG_H" -*-
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "first.h"
/*
 *  Copyright (C) 2001,7 B. Parisse, Institut Fourier, 38402 St Martin d'Heres
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
using namespace std;
#include "rpn.h"
#include "symbolic.h"
#include "unary.h"
#include <algorithm>
#include "prog.h"
#include "usual.h"
#include "identificateur.h"
#include <stdio.h>
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#include <dirent.h>
#include <pwd.h>
#endif
#include "input_lexer.h"
#include "plot.h"
#include "tex.h"
#include "ti89.h"
#include "maple.h"
#include "misc.h"
#include "permu.h"
#include "intg.h"
#include "derive.h"
#include "sym2poly.h"
#include "input_parser.h"
#include "solve.h"
#include "subst.h"

#ifndef NO_NAMESPACE_GIAC
namespace giac {
#endif // ndef NO_NAMESPACE_GIAC
  bool rpn_mode=false;

  string enmajuscule(const string & s){
    string res;
    string::const_iterator it=s.begin(),itend=s.end();
    for (;it!=itend;++it){
      if ((*it>='a') && (*it<='z'))
        res += *it-32;
      else
        res += *it;
    }
    return res;
  }

  string printasconstant(const gen & feuille,const string & sommetstr,GIAC_CONTEXT){
      return sommetstr;
  }
  gen symb_rpn(const gen & args){
    return symbolic(at_rpn,args);
  }
  gen _rpn(const gen & args,GIAC_CONTEXT){
      rpn_mode=true;
      return symb_rpn(args);
  }
  const char _rpn_s []="rpn";
  const unary_function_eval __rpn(&_rpn,_rpn_s,&printasconstant);
  const unary_function_ptr at_rpn (&__rpn);

  gen symb_alg(const gen & args){
    return symbolic(at_alg,args);
  }
  gen _alg(const gen & args,GIAC_CONTEXT){
      rpn_mode=false;
      return symb_alg(args);
  }
  const char _alg_s []="alg";
  const unary_function_eval __alg(&_alg,_alg_s,&printasconstant);
  const unary_function_ptr at_alg (&__alg);

  void roll(int i,vecteur & v){
    if (i<2)
      return;
    iterateur it=v.begin(),itend=v.end();
    if (itend-it<i)
      return;
    it=itend-i;
    gen save=*it;
    for (;;){
      ++it;
      if (it==itend)
	break;
      *(it-1)=*it;
    }
    *(it-1)=save;
  }

  void ROLL(int i,GIAC_CONTEXT){
    roll(i,history_in(contextptr));
    roll(i,history_out(contextptr));
  }

  gen symb_ROLL(const gen & args){
    return symbolic(at_ROLL,args);
  }
  gen _ROLL(const gen & args,GIAC_CONTEXT){
    if (args._VECTptr->empty())
      return args;
    gen e=args._VECTptr->back();
    args._VECTptr->pop_back();
    if (e.type==_INT_)
      roll(e.val,*args._VECTptr);
    if (e.type==_DOUBLE_)
      roll(int(e._DOUBLE_val),*args._VECTptr);
    return args;
  }
  const char _ROLL_s []="ROLL";
  const unary_function_eval __ROLL(&_ROLL,_ROLL_s);
  const unary_function_ptr at_ROLL (&__ROLL);

  void rolld(int i,vecteur & v){
    if (i<2)
      return;
    iterateur it=v.begin(),itend=v.end();
    if (itend-it<i)
      return;
    it=itend-i;
    --itend;
    gen save=*itend;
    for (;it!=itend;){
      --itend;
      *(itend+1)=*itend;
    }
    *it=save;
  }

  void ROLLD(int i,GIAC_CONTEXT){
    rolld(i,history_in(contextptr));
    rolld(i,history_out(contextptr));
  }
  gen symb_ROLLD(const gen & args){
    return symbolic(at_ROLLD,args);
  }
  gen _ROLLD(const gen & args,GIAC_CONTEXT){
    if (args._VECTptr->empty())
      return args;
    gen e=args._VECTptr->back();
    args._VECTptr->pop_back();
    if (e.type==_INT_)
      rolld(e.val,*args._VECTptr);
    if (e.type==_DOUBLE_)
      rolld(int(e._DOUBLE_val),*args._VECTptr);
    return args;
  }
  const char _ROLLD_s []="ROLLD";
  const unary_function_eval __ROLLD(&_ROLLD,_ROLLD_s);
  const unary_function_ptr at_ROLLD (&__ROLLD);

  void stack_swap(vecteur & v){
    iterateur it=v.begin(),itend=v.end();
    int s=itend-it;
    if (s<2)
      return;
    --itend;
    gen tmp=*itend;
    *itend=*(itend-1);
    *(itend-1)=tmp;
  }

  void SWAP(GIAC_CONTEXT){
    stack_swap(history_in(contextptr));
    stack_swap(history_out(contextptr));
  }

  gen symb_SWAP(const gen & args){
    return symbolic(at_SWAP,args);
  }
  gen _SWAP(const gen & args,GIAC_CONTEXT){
    stack_swap(*args._VECTptr);
    return args;
  }
  const char _SWAP_s []="SWAP";
  const unary_function_eval __SWAP(&_SWAP,_SWAP_s);
  const unary_function_ptr at_SWAP (&__SWAP);

  void dup(vecteur & v){
    if (!v.empty())
      v.push_back(v.back());
  }
  gen symb_DUP(const gen & args){
    return symbolic(at_DUP,args);
  }
  gen _DUP(const gen & args,GIAC_CONTEXT){
    dup(*args._VECTptr);
    return args;
  }
  const char _DUP_s []="DUP";
  const unary_function_eval __DUP(&_DUP,_DUP_s);
  const unary_function_ptr at_DUP (&__DUP);

  void over(vecteur & v){
    int s=v.size();
    if (s>=2)
      v.push_back(v[s-2]);
  }
  gen symb_OVER(const gen & args){
    return symbolic(at_OVER,args);
  }
  gen _OVER(const gen & args,GIAC_CONTEXT){
    over(*args._VECTptr);
    return args;
  }
  const char _OVER_s []="OVER";
  const unary_function_eval __OVER(&_OVER,_OVER_s);
  const unary_function_ptr at_OVER (&__OVER);

  void pick(int i,vecteur & v){
    int s=v.size();
    if ((i>=1) && (s>=i))
      v.push_back(v[s-i]);
  }
  gen symb_PICK(const gen & args){
    return symbolic(at_PICK,args);
  }
  gen _PICK(const gen & args,GIAC_CONTEXT){
    if (args._VECTptr->empty())
      return args;
    gen e=args._VECTptr->back();
    args._VECTptr->pop_back();
    if (e.type==_INT_)
      pick(e.val,*args._VECTptr);
    if (e.type==_DOUBLE_)
      pick(int(e._DOUBLE_val),*args._VECTptr);
    return args;
  }
  const char _PICK_s []="PICK";
  const unary_function_eval __PICK(&_PICK,_PICK_s);
  const unary_function_ptr at_PICK (&__PICK);

  void drop(vecteur & v){
    if (v.empty())
      return;
    v.pop_back();
  }
  gen symb_DROP(const gen & args){
    return symbolic(at_DROP,args);
  }
  gen _DROP(const gen & args,GIAC_CONTEXT){
    drop(*args._VECTptr);
    return args;
  }
  const char _DROP_s []="DROP";
  const unary_function_eval __DROP(&_DROP,_DROP_s);
  const unary_function_ptr at_DROP (&__DROP);

  string printasNOP(const gen & feuille,const string & sommetstr,GIAC_CONTEXT){
    return sommetstr;
  }
  gen symb_NOP(const gen & args){
    return vecteur(1,symbolic(at_NOP,args));
  }
  gen _NOP(const gen & args,GIAC_CONTEXT){
    return args;
  }
  const char _NOP_s []="NOP";
  const unary_function_eval __NOP(&_NOP,_NOP_s,&printasNOP);
  const unary_function_ptr at_NOP (&__NOP);

  string printasIFTE(const gen & feuille,const string & sommetstr,GIAC_CONTEXT){
    const_iterateur it=feuille._VECTptr->begin();
    string res("IF " + printinner_VECT(*it->_VECTptr,_RPN_FUNC__VECT,contextptr));
    res += " THEN ";
    ++it;
    res += printinner_VECT(*it->_VECTptr,_RPN_FUNC__VECT,contextptr) + " ELSE ";
    ++it;
    return res + printinner_VECT(*it->_VECTptr,_RPN_FUNC__VECT,contextptr)+ " END";
  }
  gen symb_IFTE(const gen & args){
    return symbolic(at_IFTE,args);
  }
  gen _IFTE(const gen & args,const context * contextptr){
    if (args._VECTptr->size()<3)
      return args;
    gen no=args._VECTptr->back();
    args._VECTptr->pop_back();
    gen yes=args._VECTptr->back();
    args._VECTptr->pop_back();
    gen e=args._VECTptr->back();
    args._VECTptr->pop_back();
    if (e.type==_VECT){
      rpn_eval(e,*args._VECTptr,contextptr);
      if (args._VECTptr->empty())
	return args;
      e=args._VECTptr->back();
      args._VECTptr->pop_back();
    }
    if (is_zero(e))
      return rpn_eval(no,*args._VECTptr,contextptr);
    else
      return rpn_eval(yes,*args._VECTptr,contextptr);
  }
  const char _IFTE_s []="IFTE";
  const unary_function_eval __IFTE(&_IFTE,_IFTE_s,&printasIFTE);
  const unary_function_ptr at_IFTE (&__IFTE);

  string printasRPN_LOCAL(const gen & feuille,const string & sommetstr,GIAC_CONTEXT){
    string s("-> ");
    s += printinner_VECT(*feuille._VECTptr->front()._VECTptr,_RPN_FUNC__VECT,contextptr);
    gen e= feuille._VECTptr->back();
    if ( (e.type==_VECT) && (e.subtype==_RPN_FUNC__VECT))
      return s + " " +e.print(contextptr);
    else { 
      if ( (e.type==_SYMB) && (e._SYMBptr->sommet==at_quote))
	return s + " '"+e._SYMBptr->feuille.print(contextptr)+"'";
      else
	return s + " '"+e.print(contextptr)+"'";
    }
  }
  gen symb_RPN_LOCAL(const gen & a,const gen & b){
    return symbolic(at_RPN_LOCAL,makevecteur(a,b));
  }
  gen _RPN_LOCAL(const gen & args,const context * contextptr) {
    // stack level 2=symbolic names, level 1=program
    if (args.type!=_VECT)
      return symbolic(at_RPN_LOCAL,args);
    int s=args._VECTptr->size();
    if (s<3)
      toofewargs("RPN_LOCAL must have at least 3 args");
    gen prog=args._VECTptr->back();
    args._VECTptr->pop_back();
    vecteur names=*(args._VECTptr->back()._VECTptr); // must be a vector
    args._VECTptr->pop_back();
    // get values from stack
    int nvars=names.size();
    if (s-2<nvars)
      toofewargs("RPN_LOCAL");
    vecteur values(names);
    for (int j=nvars-1;j>=0;--j){
      values[j]=args._VECTptr->back();
      args._VECTptr->pop_back();
    }
    // Initialization
    context * newcontextptr = (context *) contextptr;
    int protect=bind(values,names,newcontextptr);
    vecteur res;
    if ((prog.type==_SYMB) && (prog._SYMBptr->sommet==at_quote)){
      args._VECTptr->push_back(prog._SYMBptr->feuille.eval(eval_level(contextptr),newcontextptr));
      res=*args._VECTptr;
    }
    else
      res=rpn_eval(prog,*args._VECTptr,newcontextptr);
    leave(protect,names,newcontextptr);
    return gen(res,_RPN_STACK__VECT);
  }
  const char _RPN_LOCAL_s []="RPN_LOCAL";
  const unary_function_eval __RPN_LOCAL(&_RPN_LOCAL,_RPN_LOCAL_s,&printasRPN_LOCAL);
  const unary_function_ptr at_RPN_LOCAL (&__RPN_LOCAL);

  string printasRPN_FOR(const gen & feuille,const string & sommetstr,GIAC_CONTEXT){
    if ( (feuille.type!=_VECT) && (feuille._VECTptr->size()!=2))
      return "Invalid_RPN_FOR";
    string s;
    gen controle=feuille._VECTptr->front();
    gen prog=feuille._VECTptr->back();
    bool is_start= (controle[0].print(contextptr)==" j");
    if (is_start)
      s="START ";
    else
      s="FOR "+controle[0].print(contextptr)+" ";
    s += printinner_VECT(*prog._VECTptr,_RPN_FUNC__VECT,contextptr);
    if (is_one(controle[1]))
      s+=" NEXT";
    else
      s+= " "+controle[1].print(contextptr)+" STEP";
    return s;
  }
  gen symb_RPN_FOR(const gen & a,const gen & b){
    return symbolic(at_RPN_FOR,makevecteur(a,b));
  }
  gen _RPN_FOR(const gen & args,const context * contextptr) {
    // stack level 4=init
    // stack level 3=end
    // stack level 2=[init_var,step]
    // level 1=program to execute, 
    if (args.type!=_VECT)
      return symbolic(at_RPN_FOR,args);
    int s=args._VECTptr->size();
    if (s<4)
      toofewargs("RPN_FOR must have at least 4 args");
    gen prog=args._VECTptr->back();
    args._VECTptr->pop_back();
    vecteur controle=*(args._VECTptr->back()._VECTptr); // it must be a vector
    args._VECTptr->pop_back();
    gen fin=args._VECTptr->back();
    args._VECTptr->pop_back();
    gen debut=args._VECTptr->back();
    args._VECTptr->pop_back();
    // Initialization
    vecteur names(1,controle[0]);
    gen test=inferieur_egal(controle[0],fin,contextptr);
    context * newcontextptr = (context *) contextptr;
    int protect=bind(vecteur(1,debut),names,newcontextptr);
    vecteur res;
    for (;!is_zero(test.eval(eval_level(newcontextptr),newcontextptr).evalf(eval_level(contextptr),newcontextptr));sto(eval(controle[0]+controle[1],eval_level(newcontextptr),newcontextptr),controle[0],newcontextptr)){
      res=rpn_eval(prog,*args._VECTptr,newcontextptr);
    }
    leave(protect,names,newcontextptr);
    return gen(res,_RPN_STACK__VECT);
  }
  const char _RPN_FOR_s []="RPN_FOR";
  const unary_function_eval __RPN_FOR(&_RPN_FOR,_RPN_FOR_s,&printasRPN_FOR);
  const unary_function_ptr at_RPN_FOR (&__RPN_FOR);

  string printasRPN_WHILE(const gen & feuille,const string & sommetstr,GIAC_CONTEXT){
    if ( (feuille.type!=_VECT) && (feuille._VECTptr->size()!=2))
      return "Invalid_RPN_WHILE";
    return "WHILE "+ printinner_VECT(*feuille._VECTptr->front()._VECTptr,_RPN_FUNC__VECT,contextptr) + " REPEAT "+printinner_VECT(*feuille._VECTptr->back()._VECTptr,_RPN_FUNC__VECT,contextptr)+ " END ";
  }
  gen symb_RPN_WHILE(const gen & a,const gen & b){
    return symbolic(at_RPN_WHILE,makevecteur(a,b));
  }
  gen _RPN_WHILE(const gen & args,const context * contextptr) {
    // stack level 2=condition
    // level 1=program to execute 
    if (args.type!=_VECT)
      return symbolic(at_RPN_WHILE,args);
    int s=args._VECTptr->size();
    if (s<2)
      toofewargs("RPN_WHILE must have at least 2 args");
    gen prog=args._VECTptr->back();
    args._VECTptr->pop_back();
    gen controle=args._VECTptr->back();
    args._VECTptr->pop_back();
    vecteur res;
    for (;;){
      res=rpn_eval(controle,*args._VECTptr,contextptr);
      if (args._VECTptr->empty())
	toofewargs("WHILE");
      gen tmp=args._VECTptr->back();
      args._VECTptr->pop_back();
      if (is_zero(tmp.eval(1,contextptr).evalf(1,contextptr)))
	break;
      res=rpn_eval(prog,*args._VECTptr,contextptr);
    }
    return gen(res,_RPN_STACK__VECT);
  }
  const char _RPN_WHILE_s []="RPN_WHILE";
  const unary_function_eval __RPN_WHILE(&_RPN_WHILE,_RPN_WHILE_s,&printasRPN_WHILE);
  const unary_function_ptr at_RPN_WHILE (&__RPN_WHILE);

  string printasRPN_CASE(const gen & feuille,const string & sommetstr,GIAC_CONTEXT){
    if (feuille.type!=_VECT)
      return "Invalid_RPN_CASE";
    vecteur v=*feuille._VECTptr;
    if ((v.size()!=1) ||(v.front().type!=_VECT))
      return "Invalid_RPN_CASE";
    string res("CASE ");
    const_iterateur it=v.front()._VECTptr->begin(),itend=v.front()._VECTptr->end();
    for (;it!=itend;){
      res += printinner_VECT(*it->_VECTptr,_RPN_FUNC__VECT,contextptr);
      ++it;
      if (it==itend)
	break;
      res += " THEN " + printinner_VECT(*it->_VECTptr,_RPN_FUNC__VECT,contextptr) + " END ";
      ++it;
    }
    return res+" END ";
  }
  gen symb_RPN_CASE(const gen & a){
    return symbolic(at_RPN_CASE,vecteur(1,a));
  }
  gen _RPN_CASE(const gen & args,const context * contextptr) {
    // level 1=[case1, prg1, case2,prg2,..., [default]]
    if (args.type!=_VECT)
      return symbolic(at_RPN_CASE,args);
    int s=args._VECTptr->size();
    if (s<1)
      toofewargs("RPN_CASE must have at least 1 arg");
    vecteur controle=*args._VECTptr->back()._VECTptr;
    args._VECTptr->pop_back();
    const_iterateur it=controle.begin(),itend=controle.end();
    vecteur res;
    for (;it!=itend;){
      res=rpn_eval(*it,*args._VECTptr,contextptr);
      if (args._VECTptr->empty())
	toofewargs("CASE");
      ++it;
      if (it==itend) // default of the case struct
	break;
      gen test=args._VECTptr->back();
      args._VECTptr->pop_back();
      if (!(is_zero(test.eval(1,contextptr).evalf(1,contextptr)))){
	res=rpn_eval(*it,*args._VECTptr,contextptr);
	break;
      }
      ++it;
    }
    return gen(*args._VECTptr,_RPN_STACK__VECT);
  }
  const char _RPN_CASE_s []="RPN_CASE";
  const unary_function_eval __RPN_CASE(&_RPN_CASE,_RPN_CASE_s,&printasRPN_CASE);
  const unary_function_ptr at_RPN_CASE (&__RPN_CASE);

  gen symb_RCL(const gen & a){
    return symbolic(at_RCL,a);
  }
  gen _RCL(const gen & args,const context * contextptr) {
    // stack level 2=condition
    // level 1=program to execute 
    if (args.type!=_IDNT)
      return symbolic(at_RCL,args);
    return args._IDNTptr->eval(1,args,contextptr);
  }
  const char _RCL_s []="RCL";
  const unary_function_eval __RCL(&_RCL,_RCL_s);
  const unary_function_ptr at_RCL (&__RCL);

  gen symb_VARS(const gen & a){
    return symbolic(at_VARS,a);
  }
#if defined(__APPLE__) // || defined(__FreeBSD__)
  static int int_one (struct dirent *unused){
#else
  static int int_one (const struct dirent *unused){
#endif
    return 1;
  }
  gen _VARS(const gen & args,const context * contextptr) {
    vecteur res;
    if (contextptr){
      if (contextptr->globalcontextptr && contextptr->globalcontextptr->tabptr){
	sym_tab::const_iterator it=contextptr->globalcontextptr->tabptr->begin(),itend=contextptr->globalcontextptr->tabptr->end();
	for (;it!=itend;++it)
	  res.push_back(identificateur(it->first));
      }
      return res;
    }
    if (!variables_are_files(contextptr)){
      sym_tab::const_iterator it=syms().begin(),itend=syms().end();
      for (;it!=itend;++it){
	gen id=it->second;
	if (id.type==_IDNT && id._IDNTptr->value){
	  res.push_back(id);
	}
      }
      if (is_one(args) && current_folder_name.type==_IDNT && current_folder_name._IDNTptr->value){ // add variables of the current folder
	gen & tmp=*current_folder_name._IDNTptr->value;
	if (tmp.type==_VECT){
	  vecteur v=*current_folder_name._IDNTptr->value->_VECTptr;
	  iterateur it=v.begin(),itend=v.end();
	  for (;it!=itend;++it){
	    if (it->type!=_VECT || it->_VECTptr->size()!=2)
	      continue;
	    vecteur & w=*it->_VECTptr;
	    res.push_back(w[0]);
	  }
	}
      }
      return res;
    }
#ifdef VISUALC
    return undef;
#else
    struct dirent **eps;
    int n;
    n = scandir ("./", &eps, int_one, alphasort);
    if (n >= 0){
      string s;
      int cnt;
      for (cnt = 0; cnt < n; ++cnt){
	s=string(eps[cnt]->d_name);
	unsigned k=0;
	for (;k<s.size();++k){
	  if (!isalphan(s[k]))
	    break;
	}
	if ( (k==s.size()-4) && (s[k]=='.') && (s.substr(k+1,3)=="cas") )
	  res.push_back(identificateur(s.substr(0,k)));
      }
      if ( rpn_mode && (args.type==_VECT) && (args.subtype==_RPN_STACK__VECT) ){
	args._VECTptr->push_back(res);
	return args;
      }
      return res;
    }
    else
      settypeerr ("Couldn't open the directory");
    return 0;
#endif
  }

  const char _VARS_s []="VARS";
  const unary_function_eval __VARS(&_VARS,_VARS_s);
  const unary_function_ptr at_VARS (&__VARS);

  gen symb_purge(const gen & a){
    return symbolic(at_purge,a);
  }
  gen _purge(const gen & args,const context * contextptr) {
    if (rpn_mode && (args.type==_VECT)){
      if (!args._VECTptr->size())
	toofewargs("purge");
      gen apurger=args._VECTptr->back();
      _purge(apurger,contextptr);
      args._VECTptr->pop_back();
      return gen(*args._VECTptr,_RPN_STACK__VECT);
    }
    if (args.type==_VECT)
      return apply(args,contextptr,giac::_purge);
    if (args.is_symb_of_sommet(at_at)){
      gen & f = args._SYMBptr->feuille;
      if (f.type==_VECT && f._VECTptr->size()==2){
	gen m = eval(f._VECTptr->front(),eval_level(contextptr),contextptr);
	gen indice=eval(f._VECTptr->back(),eval_level(contextptr),contextptr);
	if (m.type==_MAP){
	  gen_map::iterator it=m._MAPptr->find(indice),itend=m._MAPptr->end();
	  if (it==itend)
	    setsizeerr("Bad index"+indice.print(contextptr));
	  m._MAPptr->erase(it);
	  return 1;
	}
      }
    }
    if (args.type!=_IDNT)
      return symbolic(at_purge,args);
    // REMOVED! args.eval(eval_level(contextptr),contextptr); 
    if (contextptr){
      if (contextptr->globalcontextptr!=contextptr){ 
	// purge a local variable = set it to assume(DOM_SYMBOLIC)
	gen a2(_SYMB);
	a2.subtype=1;
	return sto(gen(makevecteur(a2),_ASSUME__VECT),args,contextptr);
      }
      // purge a global variable
      sym_tab::iterator it=contextptr->tabptr->find(*args._IDNTptr->name),itend=contextptr->tabptr->end();
      if (it==itend)
	return string2gen("No such variable "+args.print(contextptr),false);
      gen res=it->second;
      if (it->second.type==_POINTER_ && it->second.subtype==_THREAD_POINTER)
	settypeerr(args.print(contextptr)+" is locked by thread "+it->second.print(contextptr));
      contextptr->tabptr->erase(it);
      return res;
    }
    if (current_folder_name.type==_IDNT && current_folder_name._IDNTptr->value && current_folder_name._IDNTptr->value->type==_VECT){
      vecteur v=*current_folder_name._IDNTptr->value->_VECTptr;
      iterateur it=v.begin(),itend=v.end();
      gen val;
      for (;it!=itend;++it){
	if (it->type!=_VECT || it->_VECTptr->size()!=2)
	  continue;
	vecteur & w=*it->_VECTptr;
	if (w[0]==args){
	  val=w[1];
	  break;
	}
      }
      if (it!=itend){
	v.erase(it);
	gen res=gen(v,_FOLDER__VECT);
	*current_folder_name._IDNTptr->value=res;
	if (!child_id && signal_store)
	  _signal(symb_quote(symbolic(at_sto,makevecteur(res,current_folder_name))),contextptr);
	return val;
      }
    }
    if (args._IDNTptr->value){
      if (variables_are_files(contextptr))
	unlink((*args._IDNTptr->name+cas_suffixe).c_str());
      gen res=*args._IDNTptr->value;
      if (res.type==_VECT && res.subtype==_FOLDER__VECT){
	if (res._VECTptr->size()!=1)
	  setsizeerr("Non-empty folder");
      }
      delete args._IDNTptr->value;
      args._IDNTptr->value=0;
      if (!child_id && signal_store)
	_signal(symb_quote(symb_purge(args)),contextptr);
      return res;
    }
    else
      return string2gen(args.print(contextptr)+" not assigned",false);
  }
  const char _purge_s []="purge";
  const unary_function_eval __purge(&_purge,_purge_s);
  const unary_function_ptr at_purge (&__purge,_QUOTE_ARGUMENTS,0);

  string printasRPN_UNTIL(const gen & feuille,const string & sommetstr,GIAC_CONTEXT){
    if ( (feuille.type!=_VECT) && (feuille._VECTptr->size()!=2))
      return "Invalid_RPN_UNTIL";
    return "DO "+ printinner_VECT(*feuille._VECTptr->front()._VECTptr,_RPN_FUNC__VECT,contextptr) + " UNTIL "+printinner_VECT(*feuille._VECTptr->back()._VECTptr,_RPN_FUNC__VECT,contextptr)+ " END ";
  }
  gen symb_RPN_UNTIL(const gen & a,const gen & b){
    return symbolic(at_RPN_UNTIL,makevecteur(a,b));
  }
  gen _RPN_UNTIL(const gen & args,const context * contextptr) {
    // stack level 2=program
    // level 1=condition 
    if (args.type!=_VECT)
      return symbolic(at_RPN_UNTIL,args);
    int s=args._VECTptr->size();
    if (s<2)
      toofewargs("RPN_UNTIL must have at least 2 args");
    gen controle=args._VECTptr->back();
    args._VECTptr->pop_back();
    gen prog=args._VECTptr->back();
    args._VECTptr->pop_back();
    vecteur res;
    for (;;){
      res=rpn_eval(prog,*args._VECTptr,contextptr);
      res=rpn_eval(controle,*args._VECTptr,contextptr);
      if (args._VECTptr->empty())
	toofewargs("UNTIL");
      gen tmp=args._VECTptr->back();
      args._VECTptr->pop_back();
      if (!is_zero(tmp.eval(eval_level(contextptr),contextptr).evalf(eval_level(contextptr),contextptr)))
	break;
    }
    return gen(res,_RPN_STACK__VECT);
  }
  const char _RPN_UNTIL_s []="RPN_UNTIL";
  const unary_function_eval __RPN_UNTIL(&_RPN_UNTIL,_RPN_UNTIL_s,&printasRPN_UNTIL);
  const unary_function_ptr at_RPN_UNTIL (&__RPN_UNTIL);

  // RPN evaluation loop (_VECTEVAL), no return stack currently
  vecteur rpn_eval(const vecteur & prog,vecteur & pile,const context * contextptr){
    const_iterateur it=prog.begin(),itend=prog.end();
    for (;it!=itend;++it){
      if (it->type==_FUNC){
	// test nargs with subtype
	int nargs=it->subtype;
	if (nargs>signed(pile.size()))
	  toofewargs(it->print(contextptr)+": stack "+gen(pile).print(contextptr));
	if (nargs==1){
	  pile.back()=(*it->_FUNCptr)(pile.back(),contextptr);
	}
	else {
	  if (nargs){
	    vecteur v(nargs);
	    for (int k=nargs-1;k>=0;--k){
	      v[k]=pile.back();
	      pile.pop_back();
	    }
	    pile.push_back((*it->_FUNCptr)(v,contextptr));
	  }
	  else {
	    gen res;
	    if (*it->_FUNCptr==at_eval){ // eval stack level 1
	      if (pile.empty())
		toofewargs("EVAL");
	      res=pile.back();
	      pile.pop_back();
	      if ( (res.type==_SYMB) && (res._SYMBptr->sommet==at_rpn_prog))
		res=res._SYMBptr->feuille;
	      res=rpn_eval(res,pile,contextptr);
	    }
	    else
	      res=(*it->_FUNCptr)(pile,contextptr);
	    if ( (res.type==_VECT) && (res.subtype=_RPN_STACK__VECT) )
	      pile= *res._VECTptr;
	    else
	      pile= vecteur(1,res);
	  }
	}
      }
      else {
	// test for special symbolic (control struct)
	const unary_function_ptr control_op[]={at_RPN_LOCAL,at_RPN_FOR,at_IFTE,at_RPN_CASE,at_RPN_WHILE,at_RPN_UNTIL,0};
	if ( (it->type==_SYMB) && equalposcomp(control_op,it->_SYMBptr->sommet)){
	  // push args of it to the stack and call sommet on the stack
	  if (it->_SYMBptr->feuille.type!=_VECT) // should not happen!
	    pile.push_back(it->_SYMBptr->feuille);
	  else
	    pile=mergevecteur(pile,*it->_SYMBptr->feuille._VECTptr);
	  gen res=it->_SYMBptr->sommet(pile,contextptr);
	  if ( (res.type==_VECT) && (res.subtype=_RPN_STACK__VECT) )
	    pile= *res._VECTptr;
	  else
	    pile= vecteur(1,res);	  
	}
	else {
	  if ( (it->type!=_VECT) 
	       // || (it->subtype==_RPN_FUNC__VECT) 
	       ){
	    gen res=it->eval(1,contextptr);
	    if ( (res.type==_VECT) && (res.subtype==_RPN_STACK__VECT))
	      pile=*res._VECTptr;
	    else
	      pile.push_back(res);
	  }
	  else
	    pile.push_back(*it);
	}
      }
    }
    return pile;
  }

  vecteur rpn_eval(const gen & prog,vecteur & pile,const context * contextptr){
    if (prog.type!=_VECT)
      return rpn_eval(vecteur(1,prog),pile,contextptr);
    else
      return rpn_eval(*prog._VECTptr,pile,contextptr);
  }

  string printasrpn_prog(const gen & feuille,const string & sommetstr,GIAC_CONTEXT){
    if (feuille.type!=_VECT)
      return "<< "+feuille.print(contextptr)+" >>";
    return "<< "+printinner_VECT(*feuille._VECTptr,_RPN_FUNC__VECT,contextptr)+" >>";
  }
  gen symb_rpn_prog(const gen & args){
    return symbolic(at_rpn_prog,args);
  }
  gen _rpn_prog(const gen & args,const context * contextptr){
    if (!rpn_mode || (args.type!=_VECT))
      return symbolic(at_rpn_prog,args);
    vecteur pile(history_out(contextptr));
    *logptr(contextptr) << pile << " " << args << endl;
    return gen(rpn_eval(*args._VECTptr,pile,contextptr),_RPN_STACK__VECT);
  }
  const char _rpn_prog_s []="rpn_prog";
  const unary_function_eval __rpn_prog(&_rpn_prog,_rpn_prog_s,&printasrpn_prog);
  const unary_function_ptr at_rpn_prog (&__rpn_prog,_QUOTE_ARGUMENTS,0);

  string texprintasdivision(const gen & feuille,const string & s,GIAC_CONTEXT){
    if (feuille.type!=_VECT || feuille._VECTptr->size()!=2)
      return "invalid /";
    return "\\frac{"+gen2tex(feuille._VECTptr->front(),contextptr)+"}{"+gen2tex(feuille._VECTptr->back(),contextptr)+"}";
  }
  gen symb_division(const gen & a,const gen & b){
    return symbolic(at_division,makevecteur(a,b));
  }
  gen symb_division(const gen & args){
    return symbolic(at_division,args);
  }
  gen _division(const gen & args,GIAC_CONTEXT){
    if ( (args.type!=_VECT) || (args._VECTptr->size()!=2) )
      return symb_division(args);
    gen a=args._VECTptr->front(),b=args._VECTptr->back();
    if (a.is_approx()){
      gen b1;
      if (has_evalf(b,b1,1,contextptr) && b.type!=b1.type)
	return rdiv(a,b1);
    }
    if (b.is_approx()){
      gen a1;
      if (has_evalf(a,a1,1,contextptr) && a.type!=a1.type)
	return rdiv(a1,b);
    }
    return rdiv(a,b);
  }
  const char _division_s []="/";
  const unary_function_eval __division(&_division,_division_s,&printsommetasoperator,&texprintasdivision);
  const unary_function_ptr at_division (&__division);

  gen symb_binary_minus(const gen & a,const gen & b){
    return symbolic(at_binary_minus,makevecteur(a,b));
  }
  gen symb_binary_minus(const gen & args){
    return symbolic(at_binary_minus,args);
  }
  gen _binary_minus(const gen & args,GIAC_CONTEXT){
    if ( (args.type!=_VECT) || (args._VECTptr->size()!=2) )
      return symb_binary_minus(args);
    return args._VECTptr->front()-args._VECTptr->back();
  }
  const string _binary_minus_s("-");
  const unary_function_eval __binary_minus(&_binary_minus,_binary_minus_s,&printsommetasoperator,&texprintsommetasoperator);
  const unary_function_ptr at_binary_minus (&__binary_minus);

  vecteur tab2vecteur(gen tab[]){
    vecteur res;
    for (;!is_zero(*tab);++tab)
      res.push_back(*tab);
    return res;
  }

  // hp38 compatibility
  // model
  gen _hp38(const gen & args,GIAC_CONTEXT){
    if ( (args.type!=_VECT) || (args._VECTptr->size()!=2) )
      setsizeerr();
    return undef;
  }
  const string _hp38_s("hp38");
  const unary_function_eval __hp38(&_hp38,_hp38_s);
  const unary_function_ptr at_hp38 (&__hp38,0,true);

  // real/complex
  gen _ABS(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT)
      return abs(args,contextptr);
    vecteur v;
    aplatir(*args._VECTptr,v);
    return _l2norm(v,contextptr);
  }
  const string _ABS_s("ABS");
  const unary_function_eval __ABS(&_ABS,_ABS_s);
  const unary_function_ptr at_ABS (&__ABS,0,true);

  const string _ARG_s("ARG");
  const unary_function_eval __ARG(&giac::arg,_ARG_s);
  const unary_function_ptr at_ARG (&__ARG,0,true);

  const string _CONJ_s("CONJ");
  const unary_function_eval __CONJ(&giac::conj,_CONJ_s);
  const unary_function_ptr at_CONJ (&__CONJ,0,true);

  const string _RE_s("RE");
  const unary_function_eval __RE(&giac::re,_RE_s);
  const unary_function_ptr at_RE (&__RE,0,true);

  const string _IM_s("IM");
  const unary_function_eval __IM(&giac::im,_IM_s);
  const unary_function_ptr at_IM (&__IM,0,true);

  const string _FLOOR_s("FLOOR");
  const unary_function_eval __FLOOR(&giac::_floor,_FLOOR_s);
  const unary_function_ptr at_FLOOR (&__FLOOR,0,true);

  const string _CEILING_s("CEILING");
  const unary_function_eval __CEILING(&giac::_ceil,_CEILING_s);
  const unary_function_ptr at_CEILING (&__CEILING,0,true);

  gen fPart(const gen & g,GIAC_CONTEXT);
  const string _FRAC_s("FRAC");
  const unary_function_eval __FRAC(&giac::fPart,_FRAC_s);
  const unary_function_ptr at_FRAC (&__FRAC,0,true);

  const string _MAX_s("MAX");
  const unary_function_eval __MAX(&giac::_max,_MAX_s);
  const unary_function_ptr at_MAX (&__MAX,0,true);

  const string _MIN_s("MIN");
  const unary_function_eval __MIN(&giac::_min,_MIN_s);
  const unary_function_ptr at_MIN (&__MIN,0,true);

  gen _MODULO(const gen & args,GIAC_CONTEXT){
    if (args.type==_VECT && args._VECTptr->size()==2){
      gen a=args._VECTptr->front(),b=args._VECTptr->back();
      if (is_zero(b))
	return a;
      return a-b*_floor(a/b,contextptr);
    }
    settypeerr();
    return undef;
  }
  const string _MOD_s("MOD");
  const unary_function_eval __MOD(&giac::_MODULO,_MOD_s);
  const unary_function_ptr at_MOD (&__MOD,0,T_DIV);

  const string _ROUND_s("ROUND");
  const unary_function_eval __ROUND(&giac::_round,_ROUND_s); 
  const unary_function_ptr at_ROUND (&__ROUND,0,true);

  gen _trunc(const gen & args,GIAC_CONTEXT);
  const string _TRUNCATE_s("TRUNCATE");
  const unary_function_eval __TRUNCATE(&giac::_trunc,_TRUNCATE_s); 
  const unary_function_ptr at_TRUNCATE (&__TRUNCATE,0,true);

  const string _QUOTE_s("QUOTE");
  const unary_function_eval __QUOTE(&giac::quote,_QUOTE_s); 
  const unary_function_ptr at_QUOTE (&__QUOTE,_QUOTE_ARGUMENTS,true);

  // Note that 0 is necessary otherwise conflict on freeBSD
  const string _SIGN_s("SIGN");
  const unary_function_eval __SIGN0(&giac::sign,_SIGN_s);
  const unary_function_ptr at_SIGN (&__SIGN0,0,true);
  
  // proba
  const string _COMB_s("COMB");
  const unary_function_eval __COMB(&giac::_comb,_COMB_s); 
  const unary_function_ptr at_COMB (&__COMB,0,true);  

  const string _PERM_s("PERM");
  const unary_function_eval __PERM(&giac::_perm,_PERM_s); 
  const unary_function_ptr at_PERM (&__PERM,0,true);  

  gen _RANDOM(const gen & g,GIAC_CONTEXT){
    return double(giac_rand(contextptr))/RAND_MAX;
  }
  const string _RANDOM_s("RANDOM");
  const unary_function_eval __RANDOM(&giac::_RANDOM,_RANDOM_s); 
  const unary_function_ptr at_RANDOM (&__RANDOM,0,T_RETURN);  

  const string _RANDSEED_s("RANDSEED");
  const unary_function_eval __RANDSEED(&giac::_srand,_RANDSEED_s); 
  const unary_function_ptr at_RANDSEED (&__RANDSEED,0,T_RETURN);  

  gen _MAXREAL(const gen & g,GIAC_CONTEXT){
    return giac_float(9.99999999e49);
  }
  const string _MAXREAL_s("MAXREAL");
  const unary_function_eval __MAXREAL(&giac::_MAXREAL,_MAXREAL_s); 
  const unary_function_ptr at_MAXREAL (&__MAXREAL,0,T_RETURN);  

  gen _MINREAL(const gen & g,GIAC_CONTEXT){
    return giac_float(1e-49);
  }
  const string _MINREAL_s("MINREAL");
  const unary_function_eval __MINREAL(&giac::_MINREAL,_MINREAL_s); 
  const unary_function_ptr at_MINREAL (&__MINREAL,0,T_RETURN);  

  // transcendent
  const string _EXP_s("EXP");
  const unary_function_eval __EXP(&giac::exp,_EXP_s);
  const unary_function_ptr at_EXP (&__EXP,0,true);

  gen _EXPM1(const gen & g,GIAC_CONTEXT){
    return exp(g,contextptr)-1;
  }
  const string _EXPM1_s("EXPM1");
  const unary_function_eval __EXPM1(&giac::_EXPM1,_EXPM1_s);
  const unary_function_ptr at_EXPM1 (&__EXPM1,0,true);

  gen _LNP1(const gen & g,GIAC_CONTEXT){
    return ln(g+1,contextptr);
  }
  const string _LNP1_s("LNP1");
  const unary_function_eval __LNP1(&giac::_LNP1,_LNP1_s);
  const unary_function_ptr at_LNP1 (&__LNP1,0,true);

  const string _LN_s("LN");
  const unary_function_eval __LN(&giac::ln,_LN_s);
  const unary_function_ptr at_LN (&__LN,0,true);

  const string _LOG_s("LOG");
  const unary_function_eval __LOG(&giac::log10,_LOG_s);
  const unary_function_ptr at_LOG (&__LOG,0,true);

  const string _ALOG_s("ALOG");
  const unary_function_eval __ALOG(&giac::alog10,_ALOG_s);
  const unary_function_ptr at_ALOG (&__ALOG,0,true);

  const string _SIN_s("SIN");
  const unary_function_eval __SIN(&giac::sin,_SIN_s);
  const unary_function_ptr at_SIN (&__SIN,0,true);

  const string _COS_s("COS");
  const unary_function_eval __COS(&giac::cos,_COS_s);
  const unary_function_ptr at_COS (&__COS,0,true);

  const string _TAN_s("TAN");
  const unary_function_eval __TAN(&giac::tan,_TAN_s);
  const unary_function_ptr at_TAN (&__TAN,0,true);

  const string _ASIN_s("ASIN");
  const unary_function_eval __ASIN(&giac::asin,_ASIN_s);
  const unary_function_ptr at_ASIN (&__ASIN,0,true);

  const string _ACOS_s("ACOS");
  const unary_function_eval __ACOS(&giac::acos,_ACOS_s);
  const unary_function_ptr at_ACOS (&__ACOS,0,true);

  const string _ATAN_s("ATAN");
  const unary_function_eval __ATAN(&giac::atan,_ATAN_s);
  const unary_function_ptr at_ATAN (&__ATAN,0,true);

  const string _SINH_s("SINH");
  const unary_function_eval __SINH(&giac::sinh,_SINH_s);
  const unary_function_ptr at_SINH (&__SINH,0,true);

  const string _COSH_s("COSH");
  const unary_function_eval __COSH(&giac::cosh,_COSH_s);
  const unary_function_ptr at_COSH (&__COSH,0,true);

  const string _TANH_s("TANH");
  const unary_function_eval __TANH(&giac::tanh,_TANH_s);
  const unary_function_ptr at_TANH (&__TANH,0,true);

  const string _ASINH_s("ASINH");
  const unary_function_eval __ASINH(&giac::asinh,_ASINH_s);
  const unary_function_ptr at_ASINH (&__ASINH,0,true);

  const string _ACOSH_s("ACOSH");
  const unary_function_eval __ACOSH(&giac::acosh,_ACOSH_s);
  const unary_function_ptr at_ACOSH (&__ACOSH,0,true);

  const string _ATANH_s("ATANH");
  const unary_function_eval __ATANH(&giac::atanh,_ATANH_s);
  const unary_function_ptr at_ATANH (&__ATANH,0,true);

  /* matrices */
  gen _ADDROW(const gen & args,GIAC_CONTEXT){
    vecteur v(gen2vecteur(args));
    if (!v.empty() && v[0].type==_IDNT){
      gen v0=v[0];
      gen g=eval(args,eval_level(contextptr),contextptr);
      if (ckmatrix(g[0]))
	return sto(_ADDROW(g,contextptr),v0,contextptr);
    }
    v[2]=_floor(v[2],contextptr);
    if (v.size()!=3 || !ckmatrix(v[0]) || v[1].type!=_VECT || v[2].type!=_INT_)
      settypeerr();
    vecteur & w = *v[0]._VECTptr;
    if (w.front()._VECTptr->size()!=v[1]._VECTptr->size())
      setdimerr();
    int s=w.size();
    int l2=v[2].val-(xcas_mode(contextptr)!=0);
    if (l2<0 || l2>s)
      setdimerr();    
    matrice res(w.begin(),w.begin()+l2);
    res.push_back(v[1]);
    for (int i=l2;i<s;++i)
      res.push_back(w[i]);
    return res;
  }
  const string _ADDROW_s("ADDROW");
  const unary_function_eval __ADDROW(&giac::_ADDROW,_ADDROW_s);
  const unary_function_ptr at_ADDROW (&__ADDROW,_QUOTE_ARGUMENTS,true);

  gen _ADDCOL(const gen & args,GIAC_CONTEXT){
    vecteur v(gen2vecteur(args));
    if (!v.empty() && v[0].type==_IDNT){
      gen v0=v[0];
      gen g=eval(args,eval_level(contextptr),contextptr);
      if (ckmatrix(g[0]))
	return sto(_ADDCOL(g,contextptr),v0,contextptr);
    }
    if (v.size()!=3 || !ckmatrix(v[0]))
      setsizeerr();
    matrice m;
    mtran(*v[0]._VECTptr,m);
    gen res=_ADDROW(makevecteur(m,v[1],v[2]),contextptr);
    if (res.type==_VECT){
      mtran(*res._VECTptr,m);
      res=m;
    }
    return res;
  }
  const string _ADDCOL_s("ADDCOL");
  const unary_function_eval __ADDCOL(&giac::_ADDCOL,_ADDCOL_s);
  const unary_function_ptr at_ADDCOL (&__ADDCOL,_QUOTE_ARGUMENTS,true);

  gen _SCALE(const gen & g,GIAC_CONTEXT){
    if (g.type!=_VECT || g._VECTptr->size()!=3)
      setsizeerr();
    vecteur v =*g._VECTptr;
    swapgen(v[0],v[1]);
    return _mRow(gen(v,_SEQ__VECT),contextptr);
  }
  const string _SCALE_s("SCALE");
  const unary_function_eval __SCALE(&giac::_SCALE,_SCALE_s);
  const unary_function_ptr at_SCALE (&__SCALE,_QUOTE_ARGUMENTS,true);

  gen _SCALEADD(const gen & g,GIAC_CONTEXT){
    if (g.type!=_VECT || g._VECTptr->size()!=4)
      setsizeerr();
    vecteur v =*g._VECTptr;
    swapgen(v[0],v[1]);
    return _mRowAdd(gen(v,_SEQ__VECT),contextptr);
  }
  const string _SCALEADD_s("SCALEADD");
  const unary_function_eval __SCALEADD(&giac::_SCALEADD,_SCALEADD_s);
  const unary_function_ptr at_SCALEADD (&__SCALEADD,_QUOTE_ARGUMENTS,true);

  gen _SWAPCOL(const gen & args,GIAC_CONTEXT){
    vecteur v(gen2vecteur(args));
    if (!v.empty() && v[0].type==_IDNT){
      gen v0=v[0];
      gen g=eval(args,eval_level(contextptr),contextptr);
      if (ckmatrix(g[0]))
	return sto(_SWAPCOL(g,contextptr),v0,contextptr);
    }
    if (v.size()!=3 || !ckmatrix(v[0]))
      setsizeerr();
    matrice m;
    mtran(*v[0]._VECTptr,m);
    gen res=_rowSwap(makevecteur(m,v[1],v[2]),contextptr);
    if (res.type==_VECT){
      mtran(*res._VECTptr,m);
      res=m;
    }
    return res;
  }
  const string _SWAPCOL_s("SWAPCOL");
  const unary_function_eval __SWAPCOL(&giac::_SWAPCOL,_SWAPCOL_s);
  const unary_function_ptr at_SWAPCOL (&__SWAPCOL,_QUOTE_ARGUMENTS,true);

  const string _DELCOL_s("DELCOL");
  const unary_function_eval __DELCOL(&giac::_delcols,_DELCOL_s);
  const unary_function_ptr at_DELCOL (&__DELCOL,_QUOTE_ARGUMENTS,true);

  const string _DELROW_s("DELROW");
  const unary_function_eval __DELROW(&giac::_delrows,_DELROW_s);
  const unary_function_ptr at_DELROW (&__DELROW,_QUOTE_ARGUMENTS,true);

  const string _SWAPROW_s("SWAPROW");
  const unary_function_eval __SWAPROW(&giac::_rowSwap,_SWAPROW_s);
  const unary_function_ptr at_SWAPROW (&__SWAPROW,_QUOTE_ARGUMENTS,true);

  gen _SUB(const gen & args,GIAC_CONTEXT){
    vecteur v(gen2vecteur(args));
    gen v0=v[0];
    v=*eval(v,eval_level(contextptr),contextptr)._VECTptr;
    if (v.size()!=4 || v0.type!=_IDNT)
      return _mid(v,contextptr);
    v[2]=_floor(v[2],contextptr);
    v[3]=_floor(v[3],contextptr);
    if (ckmatrix(v[1]))
      return sto(_subMat(gen(makevecteur(v[1],v[2],v[3]),_SEQ__VECT),contextptr),v0,contextptr);
    if (v[1].type==_VECT && v[2].type==_INT_ && v[3].type==_INT_){
      vecteur & w =*v[1]._VECTptr;
      int v2=v[2].val+(xcas_mode(contextptr)==0)-1, v3=v[3].val+(xcas_mode(contextptr)==0)-1;
      if (v2<=v3 && v2>=0 && v3<int(w.size()))
	return sto(vecteur(w.begin()+v2,w.begin()+v3+1),v0,contextptr);
    }
    return undef;
  }
  const string _SUB_s("SUB");
  const unary_function_eval __SUB(&giac::_SUB,_SUB_s);
  const unary_function_ptr at_SUB (&__SUB,_QUOTE_ARGUMENTS,true);

  gen _RANDMAT(const gen & args,GIAC_CONTEXT){
    vecteur v(gen2vecteur(args));
    int s=v.size();
    gen name=v[0];
    if (s<2 || name.type!=_IDNT)
      setsizeerr();
    v=*eval(v,eval_level(contextptr),contextptr)._VECTptr;
    v[1]=_floor(v[1],contextptr);
    if (v[1].type!=_INT_)
      settypeerr();
    if (s==2)
      return sto(vranm(v[1].val,0,contextptr),name,contextptr);
    v[2]=_floor(v[2],contextptr);
    if (v[2].type!=_INT_)
      settypeerr();
    return sto(mranm(v[1].val,v[2].val,0,contextptr),name,contextptr);    
  }
  const string _RANDMAT_s("RANDMAT");
  const unary_function_eval __RANDMAT(&giac::_RANDMAT,_RANDMAT_s);
  const unary_function_ptr at_RANDMAT (&__RANDMAT,_QUOTE_ARGUMENTS,true);

  gen _REDIM(const gen & args,GIAC_CONTEXT){
    vecteur v(gen2vecteur(args));
    int s=v.size();
    gen name=v[0];
    if (s<2 || name.type!=_IDNT)
      setsizeerr();
    v=*eval(v,eval_level(contextptr),contextptr)._VECTptr;
    if (v[0].type!=_VECT)
      settypeerr();
    vecteur w=*v[0]._VECTptr,argv;
    if (ckmatrix(v[0]))
      aplatir(*v[0]._VECTptr,w);
    v[1]=_floor(v[1],contextptr);
    argv=gen2vecteur(v[1]);
    argv.push_back(w);
    return sto(_matrix(gen(argv,_SEQ__VECT),contextptr),name,contextptr);
  }
  const string _REDIM_s("REDIM");
  const unary_function_eval __REDIM(&giac::_REDIM,_REDIM_s);
  const unary_function_ptr at_REDIM (&__REDIM,_QUOTE_ARGUMENTS,true);

  gen _REPLACE(const gen & args,GIAC_CONTEXT){
    vecteur v(gen2vecteur(args));
    int s=v.size();
    gen name=v[0];
    if (s!=3 || name.type!=_IDNT)
      setsizeerr();
    v=*eval(v,eval_level(contextptr),contextptr)._VECTptr;
    if (v[0].type!=_VECT || v[2].type!=_VECT)
      settypeerr();
    v[1]=_floor(v[1],contextptr);
    int pos,l,c=0;
    vecteur w0=*v[0]._VECTptr,w2=*v[2]._VECTptr,argv;
    if (ckmatrix(v[0])){
      mdims(w0,l,c);
      aplatir(*v[0]._VECTptr,w0);
    }
    else
      l=w0.size();
    if (v[1].type==_VECT){
      vecteur & v1=*v[1]._VECTptr;
      if (v1.size()!=2 || v1.front().type!=_INT_ || v1.back().type!=_INT_)
	settypeerr();
      pos=v1.front().val*c+v1.back().val;
    }
    else {
      if (v[1].type!=_INT_)
	settypeerr();
      pos=v[1].val;
    }
    if (ckmatrix(v[1]))
      aplatir(*v[1]._VECTptr,w2);
    int w0s=w0.size(),w2s=w2.size(),i=pos,j=0;
    for (;i<w0s && j<w2s;++j,++i){
      w0[i]=w2[j];
    }
    if (!c) return sto(w0,name,contextptr);
    return sto(_matrix(gen(makevecteur(l,c,w0),_SEQ__VECT),contextptr),name,contextptr);
  }
  const string _REPLACE_s("REPLACE");
  const unary_function_eval __REPLACE(&giac::_REPLACE,_REPLACE_s);
  const unary_function_ptr at_REPLACE (&__REPLACE,_QUOTE_ARGUMENTS,true);

  const string _COLNORM_s("COLNORM");
  const unary_function_eval __COLNORM(&giac::_colNorm,_COLNORM_s);
  const unary_function_ptr at_COLNORM (&__COLNORM,0,true);

  const string _ROWNORM_s("ROWNORM");
  const unary_function_eval __ROWNORM(&giac::_rowNorm,_ROWNORM_s);
  const unary_function_ptr at_ROWNORM (&__ROWNORM,0,true);

  const string _CROSS_s("CROSS");
  const unary_function_eval __CROSS(&giac::_cross,_CROSS_s);
  const unary_function_ptr at_CROSS (&__CROSS,0,true);

  const string _DET_s("DET");
  const unary_function_eval __DET(&giac::_det,_DET_s);
  const unary_function_ptr at_DET (&__DET,0,true);

  const string _DOT_s("DOT");
  const unary_function_eval __DOT(&giac::_dotprod,_DOT_s);
  const unary_function_ptr at_DOT (&__DOT,0,true);

  gen _EIGENVAL(const gen & args,GIAC_CONTEXT){
    return _diag(_egvl(args,contextptr),contextptr);
  }
  const string _EIGENVAL_s("EIGENVAL");
  const unary_function_eval __EIGENVAL(&giac::_EIGENVAL,_EIGENVAL_s);
  const unary_function_ptr at_EIGENVAL (&__EIGENVAL,0,true);

  const string _EIGENVV_s("EIGENVV");
  const unary_function_eval __EIGENVV(&giac::_jordan,_EIGENVV_s);
  const unary_function_ptr at_EIGENVV (&__EIGENVV,0,true);

  const string _IDENMAT_s("IDENMAT");
  const unary_function_eval __IDENMAT(&giac::_idn,_IDENMAT_s);
  const unary_function_ptr at_IDENMAT (&__IDENMAT,0,true);

  const string _INVERSE_s("INVERSE");
  const unary_function_eval __INVERSE(&giac::inv,_INVERSE_s);
  const unary_function_ptr at_INVERSE (&__INVERSE,0,true);

  const string _TRACE_s("TRACE");
  const unary_function_eval __TRACE(&giac::ckmtrace,_TRACE_s);
  const unary_function_ptr at_TRACE (&__TRACE,0,true);

  const string _TRN_s("TRN");
  const unary_function_eval __TRN(&giac::_trn,_TRN_s);
  const unary_function_ptr at_TRN (&__TRN,0,true);

  const string _RANK_s("RANK");
  const unary_function_eval __RANK(&giac::_rank,_RANK_s);
  const unary_function_ptr at_RANK (&__RANK,0,true);

  gen _SIZE(const gen& args,GIAC_CONTEXT){
    if (ckmatrix(args))
      return _dim(args,contextptr);
    else
      return _size(args,contextptr);
  }
  const string _SIZE_s("SIZE");
  const unary_function_eval __SIZE(&giac::_SIZE,_SIZE_s);
  const unary_function_ptr at_SIZE (&__SIZE,0,true);

  const string _SORT_s("SORT");
  const unary_function_eval __SORT(&giac::_sort,_SORT_s);
  const unary_function_ptr at_SORT (&__SORT,0,true);

  const string _DELTALIST_s("ΔLIST");
  const unary_function_eval __DELTALIST(&giac::_deltalist,_DELTALIST_s);
  const unary_function_ptr at_DELTALIST (&__DELTALIST,0,true);

  const string _CONCAT1_s0("CONCAT");
  const unary_function_eval __CONCAT10(&giac::_concat,_CONCAT1_s0);
  const unary_function_ptr at_CONCAT10 (&__CONCAT10,0,true);

  const string _PILIST_s("ΠLIST");
  const unary_function_eval __PILIST(&giac::_product,_PILIST_s);
  const unary_function_ptr at_PILIST (&__PILIST,0,true);

  const string _SIGMALIST_s("ΣLIST");
  const unary_function_eval __SIGMALIST(&giac::_sum,_SIGMALIST_s);
  const unary_function_ptr at_SIGMALIST (&__SIGMALIST,0,true);

  const string _REVERSE_s("REVERSE");
  const unary_function_eval __REVERSE(&giac::_revlist,_REVERSE_s);
  const unary_function_ptr at_REVERSE (&__REVERSE,0,true);

  const string _POS_s("POS");
  const unary_function_eval __POS(&giac::_contains,_POS_s);
  const unary_function_ptr at_POS (&__POS,0,true);

  const string _MAKELIST_s("MAKELIST");
  const unary_function_eval __MAKELIST(&giac::_seq,_MAKELIST_s);
  const unary_function_ptr at_MAKELIST (&__MAKELIST,0,true);

  gen _INT(const gen & g,GIAC_CONTEXT){
    if (is_positive(g,contextptr))
      return _floor(g,contextptr);
    else
      return _ceil(g,contextptr);
  }
  const string _INT_s("INT");
  const unary_function_eval __INT(&giac::_INT,_INT_s);
  const unary_function_ptr at_INT (&__INT,0,true);

  void hp38_eval(vecteur & v,gen & x,gen& newx,GIAC_CONTEXT){
    x=v[1];
    if (x.is_symb_of_sommet(at_equal))
      x=x._SYMBptr->feuille[0];
    identificateur idx("taylorx");
    newx=idx;
    v[0]=eval(subst(v[0],x,newx,false,contextptr),eval_level(contextptr),contextptr);
    v[1]=newx;
    int s=v.size();
    for (int i=2;i<s;++i)
      v[i]=eval(v[i],eval_level(contextptr),contextptr);
  }

  gen _HPDIFF(const gen & args,GIAC_CONTEXT){
    if (args.type==_VECT && args._VECTptr->size()==2)
      return _derive(makevecteur(args._VECTptr->back(),args._VECTptr->front()),contextptr);
    return _derive(args,contextptr);
  }
  const string _HPDIFF_s("∂");
  const unary_function_eval __HPDIFF(&giac::_HPDIFF,_HPDIFF_s);
  const unary_function_ptr at_HPDIFF (&__HPDIFF,_QUOTE_ARGUMENTS,true);

  gen _HPINT(const gen & args,GIAC_CONTEXT){
    if (args.type==_VECT && args._VECTptr->size()==4){
      vecteur v = *args._VECTptr;
      swapgen(v[0],v[2]);
      swapgen(v[1],v[3]);
      gen x,newx;
      hp38_eval(v,x,newx,contextptr);
      return subst(_integrate(gen(v,_SEQ__VECT),contextptr),newx,x,false,contextptr);
    }
    return _integrate(args,contextptr);
  }
  const string _HPINT_s("∫");
  const unary_function_eval __HPINT(&giac::_HPINT,_HPINT_s);
  const unary_function_ptr at_HPINT (&__HPINT,_QUOTE_ARGUMENTS,true);

  gen _HPSUM(const gen & args,GIAC_CONTEXT){
    if (args.type==_VECT && args._VECTptr->size()==3){
      const vecteur & v = *args._VECTptr;
      if (v[0].is_symb_of_sommet(at_equal)){
	gen var=v[0]._SYMBptr->feuille[0];
	gen inf=v[0]._SYMBptr->feuille[1];
	gen sup=v[1];
	gen expr=v[2];
	return _sum(gen(makevecteur(expr,var,inf,sup),_SEQ__VECT),contextptr);
      }
    }
    return _sum(args,contextptr);
  }
  const string _HPSUM_s("Σ");
  const unary_function_eval __HPSUM(&giac::_HPSUM,_HPSUM_s);
  const unary_function_ptr at_HPSUM (&__HPSUM,_QUOTE_ARGUMENTS,true);

  gen _TAYLOR(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT)
      settypeerr();
    vecteur v = *args._VECTptr;
    if (v.size()<2)
      setsizeerr();
    gen x,newx;
    hp38_eval(v,x,newx,contextptr);
    return subst(_taylor(gen(v,_SEQ__VECT),contextptr),newx,x,false,contextptr);
  }
  const string _TAYLOR_s("TAYLOR");
  const unary_function_eval __TAYLOR(&_TAYLOR,_TAYLOR_s);
  const unary_function_ptr at_TAYLOR (&__TAYLOR,_QUOTE_ARGUMENTS,true);

  const string _POLYCOEF_s("POLYCOEF");
  const unary_function_eval __POLYCOEF(&giac::_pcoeff,_POLYCOEF_s);
  const unary_function_ptr at_POLYCOEF (&__POLYCOEF,0,true);

  gen _horner(const gen & args,GIAC_CONTEXT);
  const string _POLYEVAL_s("POLYEVAL");
  const unary_function_eval __POLYEVAL(&giac::_horner,_POLYEVAL_s);
  const unary_function_ptr at_POLYEVAL (&__POLYEVAL,0,true);

  gen _POLYFORM(const gen & args,GIAC_CONTEXT){
    if (args.type==_VECT){
      const vecteur & v = *args._VECTptr;
      int vs=v.size();
      if (vs==2){
	if (v.back()==at_prod)
	  return _factor(v[0],contextptr);
	if (v.back()==at_plus)
	  return _partfrac(v[0],contextptr);
      }
      if (vs>1)
	return _reorder(makevecteur(v[0],vecteur(v.begin()+1,v.end())),contextptr);
    }
    return recursive_normal(args,contextptr);
  }
  const string _POLYFORM_s("POLYFORM");
  const unary_function_eval __POLYFORM(&giac::_POLYFORM,_POLYFORM_s);
  const unary_function_ptr at_POLYFORM (&__POLYFORM,_QUOTE_ARGUMENTS,true);

  const string _POLYROOT_s("POLYROOT");
  const unary_function_eval __POLYROOT(&giac::_proot,_POLYROOT_s);
  const unary_function_ptr at_POLYROOT (&__POLYROOT,0,true);

  const string _ISOLATE_s("ISOLATE");
  const unary_function_eval __ISOLATE(&giac::_solve,_ISOLATE_s);
  const unary_function_ptr at_ISOLATE (&__ISOLATE,_QUOTE_ARGUMENTS,true);

  const string _QUAD_s("QUAD");
  const unary_function_eval __QUAD(&giac::_solve,_QUAD_s);
  const unary_function_ptr at_QUAD (&__QUAD,_QUOTE_ARGUMENTS,true);

  gen _IS_LINEAR(const gen & args,GIAC_CONTEXT){
    if (args.type==_VECT && args._VECTptr->size()==2){
      vecteur & v = *args._VECTptr;
      gen a,b;
      if (v[1].type==_IDNT && is_linear_wrt(v[0],*v[1]._IDNTptr,a,b,contextptr))
	return makevecteur(eval(a,eval_level(contextptr),contextptr),eval(b, eval_level(contextptr),contextptr));
      else
	return 0;
    }
    settypeerr();
    return 0;
  }
  const string _IS_LINEAR_s("LINEAR?");
  const unary_function_eval __IS_LINEAR(&giac::_IS_LINEAR,_IS_LINEAR_s);
  const unary_function_ptr at_IS_LINEAR (&__IS_LINEAR,_QUOTE_ARGUMENTS,true);

  const string _FNROOT_s("FNROOT");
  const unary_function_eval __FNROOT(&giac::_fsolve,_FNROOT_s);
  const unary_function_ptr at_FNROOT (&__FNROOT,0,true);

  gen _SVD(const gen & args0,GIAC_CONTEXT){
    if (!ckmatrix(args0))
      settypeerr();
    gen args=evalf(args0,1,contextptr);
    return _svd(gen(makevecteur(args,-1),_SEQ__VECT),contextptr);
  }
  const string _SVD_s("SVD");
  const unary_function_eval __SVD(&giac::_SVD,_SVD_s); // FIXME
  const unary_function_ptr at_SVD (&__SVD,0,true);

  gen _SVL(const gen & args0,GIAC_CONTEXT){
    if (!ckmatrix(args0))
      settypeerr();
    gen args=evalf(args0,1,contextptr);
    return _svd(gen(makevecteur(args,-2),_SEQ__VECT),contextptr);
  }
  const string _SVL_s("SVL");
  const unary_function_eval __SVL(&giac::_SVL,_SVL_s); // FIXME
  const unary_function_ptr at_SVL (&__SVL,0,true);

  gen _SPECRAD(const gen & args0,GIAC_CONTEXT){
    gen args=evalf(args0,1,contextptr);
    if (!ckmatrix(args))
      settypeerr();
    vecteur v=megvl(*args._VECTptr,contextptr);
    gen res,tmp;
    int s=v.size();
    for (int i=0;i<s;++i){
      tmp=abs(v[i],contextptr);
      if (ck_is_strictly_greater(tmp,res,contextptr))
	res=tmp;
    }
    return res;
  }
  const string _SPECRAD_s("SPECRAD");
  const unary_function_eval __SPECRAD(&giac::_SPECRAD,_SPECRAD_s); // FIXME
  const unary_function_ptr at_SPECRAD (&__SPECRAD,0,true);

  gen _SPECNORM(const gen & args0,GIAC_CONTEXT){
    gen args=evalf(args0,1,contextptr);
    if (!ckmatrix(args)){
      if (args.type==_VECT)
	return _l2norm(args,contextptr);
      settypeerr();
    }
    matrice mt,mmt;
    mtran(*args._VECTptr,mt);
    mmult(*args._VECTptr,mt,mmt);
    return sqrt(_SPECRAD(mmt,contextptr),contextptr);
  }
  const string _SPECNORM_s("SPECNORM");
  const unary_function_eval __SPECNORM(&giac::_SPECNORM,_SPECNORM_s); // FIXME
  const unary_function_ptr at_SPECNORM (&__SPECNORM,0,true);

  gen _COND(const gen & args0,GIAC_CONTEXT){
    // COND(matrix,2) L2norm condition number
    // otherwise COLNORM(args0)*COLNORM(inv(args0))
    if (args0.type==_VECT && args0._VECTptr->size()==2 && args0._VECTptr->back()==2){
      gen args=args0._VECTptr->front();
      if (!ckmatrix(args))
	settypeerr();
      gen g=_SVL(args,contextptr);
      if (g.type!=_VECT)
	return undef;
      vecteur & v =*g._VECTptr;
      int s=v.size();
      gen mina(plus_inf),maxa(0);
      for (int i=0;i<s;++i){
	gen tmp=abs(v[i],contextptr);
	if (ck_is_strictly_greater(mina,tmp,contextptr))
	  mina=tmp;
	if (ck_is_strictly_greater(tmp,maxa,contextptr))
	  maxa=tmp;
      }
      return maxa/mina;
    }
    else {
      gen args=evalf(args0,1,contextptr);
      return _colNorm(args,contextptr)*_rowNorm(args,contextptr)/abs(_det(args,contextptr),contextptr);
    }
  }
  const string _COND_s("COND");
  const unary_function_eval __COND(&giac::_COND,_COND_s); // FIXME
  const unary_function_ptr at_COND (&__COND,0,true);

  gen _SCHUR(const gen & args,GIAC_CONTEXT){
    if (ckmatrix(args))
      return _hessenberg(gen(makevecteur(args,epsilon(contextptr)),_SEQ__VECT),contextptr); 
    return _hessenberg(args,contextptr);
  }
  const string _SCHUR_s("SCHUR");
  const unary_function_eval __SCHUR(&giac::_SCHUR,_SCHUR_s); // FIXME
  const unary_function_ptr at_SCHUR (&__SCHUR,0,true);

  gen _LQ(const gen & args0,GIAC_CONTEXT){
    gen args=evalf(args0,1,contextptr);
    if (!ckmatrix(args))
      settypeerr();
    gen res=qr(makevecteur(_trn(args,contextptr),-1),contextptr);
    if (res.type!=_VECT)
      setsizeerr();
    vecteur v(*res._VECTptr);
    v[0]=_trn(v[0],contextptr);
    v[1]=_trn(v[1],contextptr);
    swapgen(v[0],v[1]);
    return gen(v,_SEQ__VECT);
  }
  const string _LQ_s("LQ");
  const unary_function_eval __LQ(&giac::_LQ,_LQ_s); 
  const unary_function_ptr at_LQ (&__LQ,0,true);

  const string _RREF_s("RREF");
  const unary_function_eval __RREF(&giac::_rref,_RREF_s); // FIXME
  const unary_function_ptr at_RREF (&__RREF,0,true);

  gen _XPON(const gen & g,GIAC_CONTEXT){
    if (g.type==_VECT)
      return apply(g,_XPON,contextptr);
    gen gf=_floor(log10(abs(g,contextptr),contextptr),contextptr); 
    if (gf.type!=_INT_)
      setsizeerr();
    return gf;
  }
  const string _XPON_s("XPON");
  const unary_function_eval __XPON(&giac::_XPON,_XPON_s); // FIXME
  const unary_function_ptr at_XPON (&__XPON,0,true);

  gen _MANT(const gen & g,GIAC_CONTEXT){
    if (g.type==_VECT)
      return apply(g,_MANT,contextptr);
    gen gabs=abs(g,contextptr);
    gen gf=_floor(log10(gabs,contextptr),contextptr); 
    if (gf.type!=_INT_)
      setsizeerr();
    // FIXME number of digits
    return evalf(gabs*alog10(-gf,contextptr),1,contextptr);
  }
  const string _MANT_s("MANT");
  const unary_function_eval __MANT(&giac::_MANT,_MANT_s); 
  const unary_function_ptr at_MANT (&__MANT,0,true);

  gen _HMS2H(const gen & g0,GIAC_CONTEXT){
    if (g0.type==_VECT)
      return apply(g0,_HMS2H,contextptr);
    gen g(g0);
    if (g.type==_DOUBLE_)
      g = g+1e-12;
    gen h=_floor(g,contextptr);
    gen m=_floor(100*(g-h),contextptr);
    gen s=_floor(100*(100*(g-h)-m),contextptr);
    return h+m/giac_float(60.)+s/giac_float(3600.);
  }
  const string _HMS2H_s("HMS2H");
  const unary_function_eval __HMS2H(&giac::_HMS2H,_HMS2H_s); 
  const unary_function_ptr at_HMS2H (&__HMS2H,0,true);

  gen _H2HMS(const gen & g0,GIAC_CONTEXT){
    if (g0.type==_VECT)
      return apply(g0,_HMS2H,contextptr);
    gen g(g0);
    if (g.type==_DOUBLE_)
      g = g+1e-12;
    gen h=_floor(g,contextptr);
    gen m=_floor(60*(g-h),contextptr);
    gen s=_floor(60*(60*(g-h)-m),contextptr);
    return h+m/giac_float(100.)+s/giac_float(10000.);
  }
  const string _H2HMS_s("H2HMS");
  const unary_function_eval __H2HMS(&giac::_H2HMS,_H2HMS_s); 
  const unary_function_ptr at_H2HMS (&__H2HMS,0,true);

  gen _DEG2RAD(const gen & g,GIAC_CONTEXT){
    if (g.type==_VECT)
      return apply(g,_DEG2RAD,contextptr);
    return g*deg2rad_d;
  }
  const string _DEG2RAD_s("DEG2RAD");
  const unary_function_eval __DEG2RAD(&giac::_DEG2RAD,_DEG2RAD_s); 
  const unary_function_ptr at_DEG2RAD (&__DEG2RAD,0,true);

  gen _RAD2DEG(const gen & g,GIAC_CONTEXT){
    if (g.type==_VECT)
      return apply(g,_RAD2DEG,contextptr);
    return g*rad2deg_d;
  }
  const string _RAD2DEG_s("RAD2DEG");
  const unary_function_eval __RAD2DEG(&giac::_RAD2DEG,_RAD2DEG_s); 
  const unary_function_ptr at_RAD2DEG (&__RAD2DEG,0,true);

  gen _PERCENT(const gen & g,GIAC_CONTEXT){
    if (g.type!=_VECT || g._VECTptr->size()!=2)
      settypeerr();
    return g._VECTptr->front()*g._VECTptr->back()/giac_float(100.);
  }
  const string _PERCENT_s("%"); // FIXME
  const unary_function_eval __PERCENT(&giac::_PERCENT,_PERCENT_s); 
  const unary_function_ptr at_PERCENT (&__PERCENT,0,true);

  gen _PERCENTCHANGE(const gen & g,GIAC_CONTEXT){
    if (g.type!=_VECT || g._VECTptr->size()!=2)
      settypeerr();
    return giac_float(100.)*(g._VECTptr->back()-g._VECTptr->front())/g._VECTptr->front();
  }
  const string _PERCENTCHANGE_s("%CHANGE"); 
  const unary_function_eval __PERCENTCHANGE(&giac::_PERCENTCHANGE,_PERCENTCHANGE_s); 
  const unary_function_ptr at_PERCENTCHANGE (&__PERCENTCHANGE,0,true);

  gen _PERCENTTOTAL(const gen & g,GIAC_CONTEXT){
    if (g.type!=_VECT || g._VECTptr->size()!=2)
      settypeerr();
    return giac_float(100.)*g._VECTptr->back()/g._VECTptr->front();
  }
  const string _PERCENTTOTAL_s("%TOTAL"); 
  const unary_function_eval __PERCENTTOTAL(&giac::_PERCENTTOTAL,_PERCENTTOTAL_s); 
  const unary_function_ptr at_PERCENTTOTAL (&__PERCENTTOTAL,0,true);

  const char _DISP_s []="DISP";
  const unary_function_eval __DISP(&giac::_print,_DISP_s);
  const unary_function_ptr at_DISP (&__DISP,_QUOTE_ARGUMENTS,true);

  const char _WAIT_s []="WAIT";
  const unary_function_eval __WAIT(&giac::_Pause,_WAIT_s);
  const unary_function_ptr at_WAIT (&__WAIT,0,true);

  const char _INPUT_s []="INPUT";
  const unary_function_eval __INPUT(&giac::_input,_INPUT_s);
  const unary_function_ptr at_INPUT (&__INPUT,_QUOTE_ARGUMENTS,true);

  const string _GETKEY_s("GETKEY");
  unary_function_eval __GETKEY(&_getKey,_GETKEY_s);
  const unary_function_ptr at_GETKEY (&__GETKEY,0,true);

  gen _ITERATE(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT || args._VECTptr->size()!=4)
      settypeerr();
    vecteur v = *args._VECTptr;
    gen v0=v[0];
    gen v1=v[1];
    v[2]=eval(v[2],eval_level(contextptr),contextptr);
    v[3]=eval(v[3],eval_level(contextptr),contextptr);
    if (v[3].type!=_INT_)
      settypeerr();
    int n=v[3].val;
    bool all=n<0;
    if (all)
      n=-n;
    gen value=v[2];
    vecteur res(n+1,value);
    for (int i=0;i<n;++i){
      value=eval(subst(v0,v1,value,false,contextptr),eval_level(contextptr),contextptr);
      res[i+1]=value;
    }
    return all?res:value;
  }
  const string _ITERATE_s("ITERATE");
  const unary_function_eval __ITERATE(&_ITERATE,_ITERATE_s);
  const unary_function_ptr at_ITERATE (&__ITERATE,_QUOTE_ARGUMENTS,true);

  gen _RECURSE(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT || args._VECTptr->size()<2)
      settypeerr();
    vecteur v = *args._VECTptr;
    gen v0=v[0];
    gen v1=v[1];
    int s=v.size();
    for (int i=2;i<s;++i)
      v[i]=eval(v[i],eval_level(contextptr),contextptr);
    if (v[0].type!=_IDNT)
      return _rsolve(gen(v,args.subtype),contextptr);
    // HP38 syntax, define recurrent sequence
    // idnt, expression, first term, second term
    settypeerr();
    return undef;
  }  
  const string _RECURSE_s("RECURSE");
  const unary_function_eval __RECURSE(&_RECURSE,_RECURSE_s);
  const unary_function_ptr at_RECURSE (&__RECURSE,_QUOTE_ARGUMENTS,true);

  gen _MAKEMAT(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT || args._VECTptr->size()!=3)
      settypeerr();
    vecteur v = *args._VECTptr;
    gen v0=v[0];
    v[1]=eval(v[1],eval_level(contextptr),contextptr);
    v[2]=eval(v[2],eval_level(contextptr),contextptr);
    v[1]=_floor(v[1],contextptr);
    v[2]=_floor(v[2],contextptr);
    if (v[1].type!=_INT_ || v[2].type!=_INT_)
      setsizeerr();
    int l=max(v[1].val,1),c=max(v[2].val,1);
    identificateur idI("I"),idJ("J");
    vecteur IJ=makevecteur(idI,idJ);
    vecteur IJval(2),res;
    for (int i=0;i<l;++i){
      vecteur ligne(c);
      IJval[0]=i;
      for (int j=0;j<c;++j){
	IJval[1]=j;
	ligne[j]=eval(subst(v0,IJ,IJval,false,contextptr),eval_level(contextptr),contextptr);
      }
      res.push_back(ligne);
    }
    return res;
  }
  const string _MAKEMAT_s("MAKEMAT");
  const unary_function_eval __MAKEMAT(&_MAKEMAT,_MAKEMAT_s);
  const unary_function_ptr at_MAKEMAT (&__MAKEMAT,_QUOTE_ARGUMENTS,true);

  // For over-determined system, find the solution such that || A*X-B || (L2) is min
  // for under-determined system, find a solution X of A*X=B such that ||X|| is min
  gen _LSQ(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT || args._VECTptr->size()!=2)
      settypeerr();
    vecteur & v = *args._VECTptr;
    gen v0=v[0]; // evalf(v[0],1,contextptr)
    gen v1=v[1];
    if (!ckmatrix(v0) || v1.type!=_VECT)
      settypeerr();
    int neq=v0._VECTptr->size(); // neq equations
    v0=_tran(v0,contextptr);
    matrice A=*v0._VECTptr,B;
    if (ckmatrix(v1))
      B=mtran(*v1._VECTptr);
    else
      B=vecteur(1,v1);
    if (B[0]._VECTptr->size()!=neq)
      setdimerr();
    int as=A.size(),bs=B.size(); 
    // bs system to solve, each with neq equations and as variables
    if (as>neq){ // under-determined system, find the smallest solution
      // not optimal since we solve the system for each Bi
      A.push_back(0);
      matrice res;
      for (int i=0;i<bs;++i){
	gen Bi=B[i];
	A[as]=Bi;
	matrice At=mtran(A);
	vecteur B=mker(At,contextptr);
	if (B.empty())
	  return undef;
	// The last element of B must have a non-zero last component
	vecteur Bend=*B.back()._VECTptr;
	gen last=-Bend.back();
	if (is_zero(last))
	  return vecteur(0);
	vecteur R=divvecteur(Bend,last);
	R.pop_back();
	B.pop_back();
	int Bs=B.size();
	for (int j=0;j<Bs;j++)
	  B[j]._VECTptr->pop_back();
	// The solution is R+Vect(B[0],..,B[Bs-1])
	// the smallest solution is the orthogonal projection of 0 on R+Vect(B)
	// i.e. R + projection of -R on Vect(B)
	matrice r,Bg=gramschmidt(B,r,false,contextptr);
	gen Rtmp(R);
	for (int j=0;j<Bs;++j){
	  Rtmp -= dotvecteur(Bg[j],R)/dotvecteur(Bg[j],Bg[j])*Bg[j];
	}
	res.push_back(Rtmp);
      }
      res=mtran(res);
      return res;
    }
    // orthogonal projection of each vector of B on image of A
    matrice r,Ag=gramschmidt(A,r,false,contextptr);
    matrice res;
    for (int i=0;i<bs;++i){
      gen Bi=B[i];
      vecteur tmp(as);
      for (int j=0;j<as;++j){
	tmp[j] = (Ag[j]*Bi)/(Ag[j]*Ag[j]);
      }
      res.push_back(tmp);
    }
    res=mtran(res);
    return mmult(*inv(r,contextptr)._VECTptr,res);
  }
  const string _LSQ_s("LSQ");
  const unary_function_eval __LSQ(&_LSQ,_LSQ_s);
  const unary_function_ptr at_LSQ (&__LSQ,0,true);

#ifndef NO_NAMESPACE_GIAC
} // namespace giac
#endif // ndef NO_NAMESPACE_GIAC

